@@ -30,6 +30,7 @@ import org.apache.helix.model.ExternalView;
 import org.apache.helix.model.IdealState;
 import org.apache.pinot.common.metrics.BrokerMetrics;
 import org.apache.pinot.common.request.BrokerRequest;
+import org.apache.pinot.common.request.PinotQuery;
 import org.apache.pinot.spi.config.table.RoutingConfig;
 import org.apache.pinot.spi.config.table.TableConfig;
 import org.apache.pinot.spi.config.table.TableType;
@@ -46,6 +47,8 @@ import static org.testng.Assert.assertTrue;
 
 
 public class InstanceSelectorTest {
+  private final static List<String> SEGMENTS = Arrays.asList("segment0", "segment1", "segment2", "segment3", "segment4",
+      "segment5", "segment6", "segment7", "segment8", "segment9", "segment10", "segment11");
 
   @Test
   public void testInstanceSelectorFactory() {
@@ -64,26 +67,26 @@ public class InstanceSelectorTest {
 
     // Replica-group instance selector should be returned
     when(routingConfig.getInstanceSelectorType()).thenReturn(RoutingConfig.REPLICA_GROUP_INSTANCE_SELECTOR_TYPE);
-    assertTrue(InstanceSelectorFactory
-        .getInstanceSelector(tableConfig, brokerMetrics) instanceof ReplicaGroupInstanceSelector);
+    assertTrue(InstanceSelectorFactory.getInstanceSelector(tableConfig,
+        brokerMetrics) instanceof ReplicaGroupInstanceSelector);
 
     // Strict replica-group instance selector should be returned
     when(routingConfig.getInstanceSelectorType()).thenReturn(RoutingConfig.STRICT_REPLICA_GROUP_INSTANCE_SELECTOR_TYPE);
-    assertTrue(InstanceSelectorFactory
-        .getInstanceSelector(tableConfig, brokerMetrics) instanceof StrictReplicaGroupInstanceSelector);
+    assertTrue(InstanceSelectorFactory.getInstanceSelector(tableConfig,
+        brokerMetrics) instanceof StrictReplicaGroupInstanceSelector);
 
     // Should be backward-compatible with legacy config
     when(routingConfig.getInstanceSelectorType()).thenReturn(null);
     when(tableConfig.getTableType()).thenReturn(TableType.OFFLINE);
-    when(routingConfig.getRoutingTableBuilderName())
-        .thenReturn(InstanceSelectorFactory.LEGACY_REPLICA_GROUP_OFFLINE_ROUTING);
-    assertTrue(InstanceSelectorFactory
-        .getInstanceSelector(tableConfig, brokerMetrics) instanceof ReplicaGroupInstanceSelector);
+    when(routingConfig.getRoutingTableBuilderName()).thenReturn(
+        InstanceSelectorFactory.LEGACY_REPLICA_GROUP_OFFLINE_ROUTING);
+    assertTrue(InstanceSelectorFactory.getInstanceSelector(tableConfig,
+        brokerMetrics) instanceof ReplicaGroupInstanceSelector);
     when(tableConfig.getTableType()).thenReturn(TableType.REALTIME);
-    when(routingConfig.getRoutingTableBuilderName())
-        .thenReturn(InstanceSelectorFactory.LEGACY_REPLICA_GROUP_REALTIME_ROUTING);
-    assertTrue(InstanceSelectorFactory
-        .getInstanceSelector(tableConfig, brokerMetrics) instanceof ReplicaGroupInstanceSelector);
+    when(routingConfig.getRoutingTableBuilderName()).thenReturn(
+        InstanceSelectorFactory.LEGACY_REPLICA_GROUP_REALTIME_ROUTING);
+    assertTrue(InstanceSelectorFactory.getInstanceSelector(tableConfig,
+        brokerMetrics) instanceof ReplicaGroupInstanceSelector);
   }
 
   @Test
@@ -97,10 +100,10 @@ public class InstanceSelectorTest {
         new StrictReplicaGroupInstanceSelector(offlineTableName, brokerMetrics);
 
     Set<String> enabledInstances = new HashSet<>();
-    ExternalView externalView = new ExternalView(offlineTableName);
-    Map<String, Map<String, String>> externalViewSegmentAssignment = externalView.getRecord().getMapFields();
     IdealState idealState = new IdealState(offlineTableName);
     Map<String, Map<String, String>> idealStateSegmentAssignment = idealState.getRecord().getMapFields();
+    ExternalView externalView = new ExternalView(offlineTableName);
+    Map<String, Map<String, String>> externalViewSegmentAssignment = externalView.getRecord().getMapFields();
     Set<String> onlineSegments = new HashSet<>();
 
     // 'instance0' and 'instance1' are in the same replica-group, 'instance2' and 'instance3' are in the same
@@ -126,41 +129,41 @@ public class InstanceSelectorTest {
     //   [segment2, segment3] -> [instance1, instance3, errorInstance1]
     String segment0 = "segment0";
     String segment1 = "segment1";
-    Map<String, String> externalViewInstanceStateMap0 = new TreeMap<>();
-    externalViewInstanceStateMap0.put(instance0, ONLINE);
-    externalViewInstanceStateMap0.put(instance2, ONLINE);
-    externalViewInstanceStateMap0.put(errorInstance0, ERROR);
     Map<String, String> idealStateInstanceStateMap0 = new TreeMap<>();
     idealStateInstanceStateMap0.put(instance0, ONLINE);
     idealStateInstanceStateMap0.put(instance2, ONLINE);
     idealStateInstanceStateMap0.put(errorInstance0, ONLINE);
-    externalViewSegmentAssignment.put(segment0, externalViewInstanceStateMap0);
-    externalViewSegmentAssignment.put(segment1, externalViewInstanceStateMap0);
     idealStateSegmentAssignment.put(segment0, idealStateInstanceStateMap0);
     idealStateSegmentAssignment.put(segment1, idealStateInstanceStateMap0);
+    Map<String, String> externalViewInstanceStateMap0 = new TreeMap<>();
+    externalViewInstanceStateMap0.put(instance0, ONLINE);
+    externalViewInstanceStateMap0.put(instance2, ONLINE);
+    externalViewInstanceStateMap0.put(errorInstance0, ERROR);
+    externalViewSegmentAssignment.put(segment0, externalViewInstanceStateMap0);
+    externalViewSegmentAssignment.put(segment1, externalViewInstanceStateMap0);
     onlineSegments.add(segment0);
     onlineSegments.add(segment1);
     String segment2 = "segment2";
     String segment3 = "segment3";
-    Map<String, String> externalViewInstanceStateMap1 = new TreeMap<>();
-    externalViewInstanceStateMap1.put(instance1, ONLINE);
-    externalViewInstanceStateMap1.put(instance3, ONLINE);
-    externalViewInstanceStateMap1.put(errorInstance1, ERROR);
     Map<String, String> idealStateInstanceStateMap1 = new TreeMap<>();
     idealStateInstanceStateMap1.put(instance1, ONLINE);
     idealStateInstanceStateMap1.put(instance3, ONLINE);
     idealStateInstanceStateMap1.put(errorInstance1, ONLINE);
-    externalViewSegmentAssignment.put(segment2, externalViewInstanceStateMap1);
-    externalViewSegmentAssignment.put(segment3, externalViewInstanceStateMap1);
     idealStateSegmentAssignment.put(segment2, idealStateInstanceStateMap1);
     idealStateSegmentAssignment.put(segment3, idealStateInstanceStateMap1);
+    Map<String, String> externalViewInstanceStateMap1 = new TreeMap<>();
+    externalViewInstanceStateMap1.put(instance1, ONLINE);
+    externalViewInstanceStateMap1.put(instance3, ONLINE);
+    externalViewInstanceStateMap1.put(errorInstance1, ERROR);
+    externalViewSegmentAssignment.put(segment2, externalViewInstanceStateMap1);
+    externalViewSegmentAssignment.put(segment3, externalViewInstanceStateMap1);
     onlineSegments.add(segment2);
     onlineSegments.add(segment3);
     List<String> segments = Arrays.asList(segment0, segment1, segment2, segment3);
 
-    balancedInstanceSelector.init(enabledInstances, externalView, idealState, onlineSegments);
-    replicaGroupInstanceSelector.init(enabledInstances, externalView, idealState, onlineSegments);
-    strictReplicaGroupInstanceSelector.init(enabledInstances, externalView, idealState, onlineSegments);
+    balancedInstanceSelector.init(enabledInstances, idealState, externalView, onlineSegments);
+    replicaGroupInstanceSelector.init(enabledInstances, idealState, externalView, onlineSegments);
+    strictReplicaGroupInstanceSelector.init(enabledInstances, idealState, externalView, onlineSegments);
 
     // For the 1st request:
     //   BalancedInstanceSelector:
@@ -174,6 +177,9 @@ public class InstanceSelectorTest {
     //     segment2 -> instance1
     //     segment3 -> instance1
     BrokerRequest brokerRequest = mock(BrokerRequest.class);
+    PinotQuery pinotQuery = mock(PinotQuery.class);
+    when(brokerRequest.getPinotQuery()).thenReturn(pinotQuery);
+    when(pinotQuery.getQueryOptions()).thenReturn(null);
     Map<String, String> expectedBalancedInstanceSelectorResult = new HashMap<>();
     expectedBalancedInstanceSelectorResult.put(segment0, instance0);
     expectedBalancedInstanceSelectorResult.put(segment1, instance2);
@@ -294,12 +300,12 @@ public class InstanceSelectorTest {
     assertTrue(selectionResult.getUnavailableSegments().isEmpty());
 
     // Remove segment0 and add segment4
-    externalViewSegmentAssignment.remove(segment0);
     idealStateSegmentAssignment.remove(segment0);
+    externalViewSegmentAssignment.remove(segment0);
     onlineSegments.remove(segment0);
     String segment4 = "segment4";
-    externalViewSegmentAssignment.put(segment4, externalViewInstanceStateMap0);
     idealStateSegmentAssignment.put(segment4, idealStateInstanceStateMap0);
+    externalViewSegmentAssignment.put(segment4, externalViewInstanceStateMap0);
     onlineSegments.add(segment4);
     segments = Arrays.asList(segment1, segment2, segment3, segment4);
 
@@ -364,9 +370,9 @@ public class InstanceSelectorTest {
     assertTrue(selectionResult.getUnavailableSegments().isEmpty());
 
     // Process the changes
-    balancedInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
-    replicaGroupInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
-    strictReplicaGroupInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
+    balancedInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
+    replicaGroupInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
+    strictReplicaGroupInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
 
     // For the 7th request:
     //   BalancedInstanceSelector:
@@ -504,9 +510,9 @@ public class InstanceSelectorTest {
     externalViewInstanceStateMap2.put(instance2, ONLINE);
     externalViewInstanceStateMap2.put(errorInstance0, ERROR);
     externalViewSegmentAssignment.put(segment1, externalViewInstanceStateMap2);
-    balancedInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
-    replicaGroupInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
-    strictReplicaGroupInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
+    balancedInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
+    replicaGroupInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
+    strictReplicaGroupInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
 
     // For the 11th request:
     //   BalancedInstanceSelector:
@@ -581,6 +587,236 @@ public class InstanceSelectorTest {
     assertTrue(selectionResult.getUnavailableSegments().isEmpty());
   }
 
+  @Test
+  public void testReplicaGroupInstanceSelectorNumReplicaGroupsToQuery() {
+    String offlineTableName = "testTable_OFFLINE";
+    BrokerMetrics brokerMetrics = mock(BrokerMetrics.class);
+    BrokerRequest brokerRequest = mock(BrokerRequest.class);
+    PinotQuery pinotQuery = mock(PinotQuery.class);
+    Map<String, String> queryOptions = new HashMap<>();
+    // numReplicas = 3, fanning the query to 2 replica groups
+    queryOptions.put("numReplicaGroupsToQuery", "2");
+    when(brokerRequest.getPinotQuery()).thenReturn(pinotQuery);
+    when(pinotQuery.getQueryOptions()).thenReturn(queryOptions);
+
+    ReplicaGroupInstanceSelector replicaGroupInstanceSelector =
+        new ReplicaGroupInstanceSelector(offlineTableName, brokerMetrics);
+
+    Set<String> enabledInstances = new HashSet<>();
+    IdealState idealState = new IdealState(offlineTableName);
+    Map<String, Map<String, String>> idealStateSegmentAssignment = idealState.getRecord().getMapFields();
+    ExternalView externalView = new ExternalView(offlineTableName);
+    Map<String, Map<String, String>> externalViewSegmentAssignment = externalView.getRecord().getMapFields();
+    Set<String> onlineSegments = new HashSet<>();
+
+    // 12 online segments with each segment having all 3 instances as online
+    // replicas are 3
+    String instance0 = "instance0";
+    String instance1 = "instance1";
+    String instance2 = "instance2";
+    enabledInstances.add(instance0);
+    enabledInstances.add(instance1);
+    enabledInstances.add(instance2);
+
+    Map<String, String> idealStateInstanceStateMap0 = new TreeMap<>();
+    Map<String, String> externalViewInstanceStateMap0 = new TreeMap<>();
+
+    for (String instance : enabledInstances) {
+      idealStateInstanceStateMap0.put(instance, ONLINE);
+      externalViewInstanceStateMap0.put(instance, ONLINE);
+    }
+
+    List<String> segments = getSegments();
+    // add all segments to both idealStateSegmentAssignment and externalViewSegmentAssignment maps and also to online
+    // segments
+    for (String segment: segments) {
+      idealStateSegmentAssignment.put(segment, idealStateInstanceStateMap0);
+      externalViewSegmentAssignment.put(segment, externalViewInstanceStateMap0);
+      onlineSegments.add(segment);
+    }
+
+    replicaGroupInstanceSelector.init(enabledInstances, idealState, externalView, onlineSegments);
+    //   ReplicaGroupInstanceSelector
+    //     segment0 -> instance0
+    //     segment2 -> instance0
+    //     segment4 -> instance0
+    //     segment6 -> instance0
+    //     segment8 -> instance0
+    //     segment10 -> instance0
+    //     segment1 -> instance1
+    //     segment3 -> instance1
+    //     segment5 -> instance1
+    //     segment7 -> instance1
+    //     segment9 -> instance1
+    //     segment11 -> instance1
+
+    Map<String, String> expectedReplicaGroupInstanceSelectorResult = new HashMap<>();
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(0), instance0);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(1), instance1);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(2), instance0);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(3), instance1);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(4), instance0);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(5), instance1);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(6), instance0);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(7), instance1);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(8), instance0);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(9), instance1);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(10), instance0);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(11), instance1);
+    InstanceSelector.SelectionResult selectionResult = replicaGroupInstanceSelector.select(brokerRequest, segments);
+    assertEquals(selectionResult.getSegmentToInstanceMap(), expectedReplicaGroupInstanceSelectorResult);
+    assertTrue(selectionResult.getUnavailableSegments().isEmpty());
+  }
+
+
+  @Test
+  public void testReplicaGroupInstanceSelectorNumReplicaGroupsToQueryGreaterThanReplicas() {
+    String offlineTableName = "testTable_OFFLINE";
+    BrokerMetrics brokerMetrics = mock(BrokerMetrics.class);
+    BrokerRequest brokerRequest = mock(BrokerRequest.class);
+    PinotQuery pinotQuery = mock(PinotQuery.class);
+    Map<String, String> queryOptions = new HashMap<>();
+    queryOptions.put("numReplicaGroupsToQuery", "4");
+
+    when(brokerRequest.getPinotQuery()).thenReturn(pinotQuery);
+    when(pinotQuery.getQueryOptions()).thenReturn(queryOptions);
+
+    ReplicaGroupInstanceSelector replicaGroupInstanceSelector =
+        new ReplicaGroupInstanceSelector(offlineTableName, brokerMetrics);
+
+    Set<String> enabledInstances = new HashSet<>();
+    IdealState idealState = new IdealState(offlineTableName);
+    Map<String, Map<String, String>> idealStateSegmentAssignment = idealState.getRecord().getMapFields();
+    ExternalView externalView = new ExternalView(offlineTableName);
+    Map<String, Map<String, String>> externalViewSegmentAssignment = externalView.getRecord().getMapFields();
+    Set<String> onlineSegments = new HashSet<>();
+
+    // 12 online segments with each segment having all 3 instances as online
+    // replicas are 3
+    String instance0 = "instance0";
+    String instance1 = "instance1";
+    String instance2 = "instance2";
+    enabledInstances.add(instance0);
+    enabledInstances.add(instance1);
+    enabledInstances.add(instance2);
+
+    List<String> segments = getSegments();
+
+    Map<String, String> idealStateInstanceStateMap0 = new TreeMap<>();
+    Map<String, String> externalViewInstanceStateMap0 = new TreeMap<>();
+
+    for (String instance : enabledInstances) {
+      idealStateInstanceStateMap0.put(instance, ONLINE);
+      externalViewInstanceStateMap0.put(instance, ONLINE);
+    }
+
+    // add all segments to both idealStateSegmentAssignment and externalViewSegmentAssignment maps and also to online
+    // segments
+    for (String segment: segments) {
+      idealStateSegmentAssignment.put(segment, idealStateInstanceStateMap0);
+      externalViewSegmentAssignment.put(segment, externalViewInstanceStateMap0);
+      onlineSegments.add(segment);
+    }
+
+    replicaGroupInstanceSelector.init(enabledInstances, idealState, externalView, onlineSegments);
+    //   ReplicaGroupInstanceSelector
+    //     segment0 -> instance0
+    //     segment3 -> instance0
+    //     segment6 -> instance0
+    //     segment9 -> instance0
+    //     segment1 -> instance1
+    //     segment4 -> instance1
+    //     segment7 -> instance1
+    //     segment10 -> instance1
+    //     segment2 -> instance2
+    //     segment5 -> instance2
+    //     segment8 -> instance2
+    //     segment11 -> instance2
+
+    Map<String, String> expectedReplicaGroupInstanceSelectorResult = new HashMap<>();
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(0), instance0);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(1), instance1);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(2), instance2);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(3), instance0);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(4), instance1);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(5), instance2);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(6), instance0);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(7), instance1);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(8), instance2);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(9), instance0);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(10), instance1);
+    expectedReplicaGroupInstanceSelectorResult.put(segments.get(11), instance2);
+    InstanceSelector.SelectionResult selectionResult = replicaGroupInstanceSelector.select(brokerRequest, segments);
+    assertEquals(selectionResult.getSegmentToInstanceMap(), expectedReplicaGroupInstanceSelectorResult);
+    assertTrue(selectionResult.getUnavailableSegments().isEmpty());
+  }
+
+  @Test
+  public void testReplicaGroupInstanceSelectorNumReplicaGroupsNotSet() {
+    String offlineTableName = "testTable_OFFLINE";
+    BrokerMetrics brokerMetrics = mock(BrokerMetrics.class);
+    BrokerRequest brokerRequest = mock(BrokerRequest.class);
+    PinotQuery pinotQuery = mock(PinotQuery.class);
+    Map<String, String> queryOptions = new HashMap<>();
+
+    when(brokerRequest.getPinotQuery()).thenReturn(pinotQuery);
+    when(pinotQuery.getQueryOptions()).thenReturn(queryOptions);
+
+    ReplicaGroupInstanceSelector replicaGroupInstanceSelector =
+        new ReplicaGroupInstanceSelector(offlineTableName, brokerMetrics);
+
+    Set<String> enabledInstances = new HashSet<>();
+    IdealState idealState = new IdealState(offlineTableName);
+    Map<String, Map<String, String>> idealStateSegmentAssignment = idealState.getRecord().getMapFields();
+    ExternalView externalView = new ExternalView(offlineTableName);
+    Map<String, Map<String, String>> externalViewSegmentAssignment = externalView.getRecord().getMapFields();
+    Set<String> onlineSegments = new HashSet<>();
+
+    // 12 online segments with each segment having all 3 instances as online
+    // replicas are 3
+    String instance0 = "instance0";
+    String instance1 = "instance1";
+    String instance2 = "instance2";
+    enabledInstances.add(instance0);
+    enabledInstances.add(instance1);
+    enabledInstances.add(instance2);
+
+    List<String> segments = getSegments();
+
+    Map<String, String> idealStateInstanceStateMap0 = new TreeMap<>();
+    Map<String, String> externalViewInstanceStateMap0 = new TreeMap<>();
+
+    for (String instance : enabledInstances) {
+      idealStateInstanceStateMap0.put(instance, ONLINE);
+      externalViewInstanceStateMap0.put(instance, ONLINE);
+    }
+
+    // add all segments to both idealStateSegmentAssignment and externalViewSegmentAssignment maps and also to online
+    // segments
+    for (String segment: segments) {
+      idealStateSegmentAssignment.put(segment, idealStateInstanceStateMap0);
+      externalViewSegmentAssignment.put(segment, externalViewInstanceStateMap0);
+      onlineSegments.add(segment);
+    }
+
+    replicaGroupInstanceSelector.init(enabledInstances, idealState, externalView, onlineSegments);
+    // since numReplicaGroupsToQuery is not set, first query should go to first replica group,
+    // 2nd query should go to next replica group
+
+    Map<String, String> expectedReplicaGroupInstanceSelectorResult = new HashMap<>();
+    for (String segment: segments) {
+      expectedReplicaGroupInstanceSelectorResult.put(segment, instance0);
+    }
+    InstanceSelector.SelectionResult selectionResult = replicaGroupInstanceSelector.select(brokerRequest, segments);
+    assertEquals(selectionResult.getSegmentToInstanceMap(), expectedReplicaGroupInstanceSelectorResult);
+
+    for (String segment: segments) {
+      expectedReplicaGroupInstanceSelectorResult.put(segment, instance1);
+    }
+    selectionResult = replicaGroupInstanceSelector.select(brokerRequest, segments);
+    assertEquals(selectionResult.getSegmentToInstanceMap(), expectedReplicaGroupInstanceSelectorResult);
+  }
+
   @Test
   public void testUnavailableSegments() {
     String offlineTableName = "testTable_OFFLINE";
@@ -591,29 +827,29 @@ public class InstanceSelectorTest {
         new StrictReplicaGroupInstanceSelector(offlineTableName, brokerMetrics);
 
     Set<String> enabledInstances = new HashSet<>();
-    ExternalView externalView = new ExternalView(offlineTableName);
-    Map<String, Map<String, String>> externalViewSegmentAssignment = externalView.getRecord().getMapFields();
     IdealState idealState = new IdealState(offlineTableName);
     Map<String, Map<String, String>> idealStateSegmentAssignment = idealState.getRecord().getMapFields();
+    ExternalView externalView = new ExternalView(offlineTableName);
+    Map<String, Map<String, String>> externalViewSegmentAssignment = externalView.getRecord().getMapFields();
     Set<String> onlineSegments = new HashSet<>();
 
     String instance = "instance";
     String errorInstance = "errorInstance";
     String segment0 = "segment0";
     String segment1 = "segment1";
+    Map<String, String> idealStateInstanceStateMap = new TreeMap<>();
+    idealStateInstanceStateMap.put(instance, CONSUMING);
+    idealStateInstanceStateMap.put(errorInstance, ONLINE);
+    idealStateSegmentAssignment.put(segment0, idealStateInstanceStateMap);
+    idealStateSegmentAssignment.put(segment1, idealStateInstanceStateMap);
     Map<String, String> externalViewInstanceStateMap0 = new TreeMap<>();
     externalViewInstanceStateMap0.put(instance, CONSUMING);
     externalViewInstanceStateMap0.put(errorInstance, ERROR);
     Map<String, String> externalViewInstanceStateMap1 = new TreeMap<>();
     externalViewInstanceStateMap1.put(instance, CONSUMING);
     externalViewInstanceStateMap1.put(errorInstance, ERROR);
-    Map<String, String> idealStateInstanceStateMap = new TreeMap<>();
-    idealStateInstanceStateMap.put(instance, CONSUMING);
-    idealStateInstanceStateMap.put(errorInstance, ONLINE);
     externalViewSegmentAssignment.put(segment0, externalViewInstanceStateMap0);
     externalViewSegmentAssignment.put(segment1, externalViewInstanceStateMap1);
-    idealStateSegmentAssignment.put(segment0, idealStateInstanceStateMap);
-    idealStateSegmentAssignment.put(segment1, idealStateInstanceStateMap);
     onlineSegments.add(segment0);
     onlineSegments.add(segment1);
     List<String> segments = Arrays.asList(segment0, segment1);
@@ -629,9 +865,12 @@ public class InstanceSelectorTest {
     //     (disabled) errorInstance: ERROR
     //   }
     // }
-    balancedInstanceSelector.init(enabledInstances, externalView, idealState, onlineSegments);
-    strictReplicaGroupInstanceSelector.init(enabledInstances, externalView, idealState, onlineSegments);
+    balancedInstanceSelector.init(enabledInstances, idealState, externalView, onlineSegments);
+    strictReplicaGroupInstanceSelector.init(enabledInstances, idealState, externalView, onlineSegments);
     BrokerRequest brokerRequest = mock(BrokerRequest.class);
+    PinotQuery pinotQuery = mock(PinotQuery.class);
+    when(brokerRequest.getPinotQuery()).thenReturn(pinotQuery);
+    when(pinotQuery.getQueryOptions()).thenReturn(null);
     InstanceSelector.SelectionResult selectionResult = balancedInstanceSelector.select(brokerRequest, segments);
     assertTrue(selectionResult.getSegmentToInstanceMap().isEmpty());
     assertEquals(selectionResult.getUnavailableSegments(), Arrays.asList(segment0, segment1));
@@ -684,7 +923,18 @@ public class InstanceSelectorTest {
       assertEquals(selectionResult.getSegmentToInstanceMap().size(), 2);
       assertTrue(selectionResult.getUnavailableSegments().isEmpty());
 
-      // Change the CONSUMING instance to ONLINE, both segments should be available
+      // Change ideal state for the CONSUMING instance to ONLINE, both segments should be available
+      idealStateInstanceStateMap.put(instance, ONLINE);
+      balancedInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
+      strictReplicaGroupInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
+      selectionResult = balancedInstanceSelector.select(brokerRequest, segments);
+      assertEquals(selectionResult.getSegmentToInstanceMap().size(), 2);
+      assertTrue(selectionResult.getUnavailableSegments().isEmpty());
+      selectionResult = strictReplicaGroupInstanceSelector.select(brokerRequest, segments);
+      assertEquals(selectionResult.getSegmentToInstanceMap().size(), 2);
+      assertTrue(selectionResult.getUnavailableSegments().isEmpty());
+
+      // Change external view for the CONSUMING instance to ONLINE, both segments should be available
       // {
       //   segment0: {
       //     (enabled)  instance: ONLINE,
@@ -697,8 +947,8 @@ public class InstanceSelectorTest {
       // }
       externalViewInstanceStateMap0.put(instance, ONLINE);
       externalViewInstanceStateMap1.put(instance, ONLINE);
-      balancedInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
-      strictReplicaGroupInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
+      balancedInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
+      strictReplicaGroupInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
       selectionResult = balancedInstanceSelector.select(brokerRequest, segments);
       assertEquals(selectionResult.getSegmentToInstanceMap().size(), 2);
       assertTrue(selectionResult.getUnavailableSegments().isEmpty());
@@ -706,8 +956,19 @@ public class InstanceSelectorTest {
       assertEquals(selectionResult.getSegmentToInstanceMap().size(), 2);
       assertTrue(selectionResult.getUnavailableSegments().isEmpty());
 
-      // Switch the instance state for segment1, both segments should be available for BalancedInstanceSelector, but
-      // unavailable for StrictReplicaGroupInstanceSelector
+      // Remove ONLINE instance from the ideal state, both segments should be unavailable
+      idealStateInstanceStateMap.remove(instance);
+      balancedInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
+      strictReplicaGroupInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
+      selectionResult = balancedInstanceSelector.select(brokerRequest, segments);
+      assertTrue(selectionResult.getSegmentToInstanceMap().isEmpty());
+      assertEquals(selectionResult.getUnavailableSegments(), Arrays.asList(segment0, segment1));
+      selectionResult = strictReplicaGroupInstanceSelector.select(brokerRequest, segments);
+      assertTrue(selectionResult.getSegmentToInstanceMap().isEmpty());
+      assertEquals(selectionResult.getUnavailableSegments(), Arrays.asList(segment0, segment1));
+
+      // Add back the ONLINE instance to the ideal state, switch the instance state for segment1, both segments should
+      // be available for BalancedInstanceSelector, but unavailable for StrictReplicaGroupInstanceSelector
       // {
       //   segment0: {
       //     (enabled)  instance: ONLINE,
@@ -718,10 +979,11 @@ public class InstanceSelectorTest {
       //     (enabled)  errorInstance: ONLINE
       //   }
       // }
+      idealStateInstanceStateMap.put(instance, ONLINE);
       externalViewInstanceStateMap1.put(instance, ERROR);
       externalViewInstanceStateMap1.put(errorInstance, ONLINE);
-      balancedInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
-      strictReplicaGroupInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
+      balancedInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
+      strictReplicaGroupInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
       selectionResult = balancedInstanceSelector.select(brokerRequest, segments);
       assertEquals(selectionResult.getSegmentToInstanceMap().size(), 2);
       assertTrue(selectionResult.getUnavailableSegments().isEmpty());
@@ -744,8 +1006,8 @@ public class InstanceSelectorTest {
       externalViewInstanceStateMap0.put(instance, OFFLINE);
       externalViewInstanceStateMap1.put(instance, OFFLINE);
       externalViewInstanceStateMap1.put(errorInstance, ERROR);
-      balancedInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
-      strictReplicaGroupInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
+      balancedInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
+      strictReplicaGroupInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
       selectionResult = balancedInstanceSelector.select(brokerRequest, segments);
       assertTrue(selectionResult.getSegmentToInstanceMap().isEmpty());
       assertTrue(selectionResult.getUnavailableSegments().isEmpty());
@@ -788,8 +1050,8 @@ public class InstanceSelectorTest {
       //   }
       // }
       externalViewInstanceStateMap0.put(errorInstance, ONLINE);
-      balancedInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
-      strictReplicaGroupInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
+      balancedInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
+      strictReplicaGroupInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
       selectionResult = balancedInstanceSelector.select(brokerRequest, segments);
       assertEquals(selectionResult.getSegmentToInstanceMap().size(), 1);
       assertEquals(selectionResult.getUnavailableSegments(), Collections.singletonList(segment1));
@@ -829,11 +1091,12 @@ public class InstanceSelectorTest {
       //     (enabled)  errorInstance: ERROR
       //   }
       // }
+      idealStateInstanceStateMap.put(instance, CONSUMING);
       externalViewInstanceStateMap0.put(instance, CONSUMING);
       externalViewInstanceStateMap0.put(errorInstance, ERROR);
       externalViewInstanceStateMap1.put(instance, CONSUMING);
-      balancedInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
-      strictReplicaGroupInstanceSelector.onExternalViewChange(externalView, idealState, onlineSegments);
+      balancedInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
+      strictReplicaGroupInstanceSelector.onAssignmentChange(idealState, externalView, onlineSegments);
       selectionResult = balancedInstanceSelector.select(brokerRequest, segments);
       assertTrue(selectionResult.getSegmentToInstanceMap().isEmpty());
       assertEquals(selectionResult.getUnavailableSegments(), Arrays.asList(segment0, segment1));
@@ -842,4 +1105,8 @@ public class InstanceSelectorTest {
       assertEquals(selectionResult.getUnavailableSegments(), Arrays.asList(segment0, segment1));
     }
   }
+
+  private static List<String> getSegments() {
+    return SEGMENTS;
+  }
 }
