@@ -24,16 +24,19 @@ import java.util.concurrent.Future;
 /**
  * Interface for plugging different client transports.
  */
-interface PinotClientTransport {
+public interface PinotClientTransport {
+
   BrokerResponse executeQuery(String brokerAddress, String query)
       throws PinotClientException;
 
   Future<BrokerResponse> executeQueryAsync(String brokerAddress, String query)
       throws PinotClientException;
 
+  @Deprecated
   BrokerResponse executeQuery(String brokerAddress, Request request)
       throws PinotClientException;
 
+  @Deprecated
   Future<BrokerResponse> executeQueryAsync(String brokerAddress, Request request)
       throws PinotClientException;
 
