@@ -21,22 +21,27 @@ package org.apache.pinot.common.metadata;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.Map;
+import java.util.Optional;
+import java.util.stream.Collectors;
 import javax.annotation.Nullable;
-import org.I0Itec.zkclient.exception.ZkBadVersionException;
 import org.apache.helix.AccessOption;
-import org.apache.helix.ZNRecord;
 import org.apache.helix.store.zk.ZkHelixPropertyStore;
+import org.apache.helix.zookeeper.datamodel.ZNRecord;
+import org.apache.helix.zookeeper.zkclient.exception.ZkBadVersionException;
 import org.apache.pinot.common.metadata.instance.InstanceZKMetadata;
 import org.apache.pinot.common.metadata.segment.SegmentZKMetadata;
 import org.apache.pinot.common.utils.SchemaUtils;
 import org.apache.pinot.common.utils.SegmentName;
-import org.apache.pinot.common.utils.StringUtil;
+import org.apache.pinot.common.utils.config.AccessControlUserConfigUtils;
 import org.apache.pinot.common.utils.config.TableConfigUtils;
 import org.apache.pinot.spi.config.ConfigUtils;
 import org.apache.pinot.spi.config.table.TableConfig;
 import org.apache.pinot.spi.config.table.TableType;
+import org.apache.pinot.spi.config.user.UserConfig;
 import org.apache.pinot.spi.data.Schema;
 import org.apache.pinot.spi.utils.CommonConstants;
+import org.apache.pinot.spi.utils.StringUtil;
 import org.apache.pinot.spi.utils.builder.TableNameBuilder;
 import org.apache.zookeeper.data.Stat;
 import org.slf4j.Logger;
@@ -50,19 +55,25 @@ public class ZKMetadataProvider {
 
   private static final Logger LOGGER = LoggerFactory.getLogger(ZKMetadataProvider.class);
   private static final String CLUSTER_TENANT_ISOLATION_ENABLED_KEY = "tenantIsolationEnabled";
+  private static final String PROPERTYSTORE_CONTROLLER_JOBS_PREFIX = "/CONTROLLER_JOBS";
   private static final String PROPERTYSTORE_SEGMENTS_PREFIX = "/SEGMENTS";
   private static final String PROPERTYSTORE_SCHEMAS_PREFIX = "/SCHEMAS";
   private static final String PROPERTYSTORE_INSTANCE_PARTITIONS_PREFIX = "/INSTANCE_PARTITIONS";
   private static final String PROPERTYSTORE_TABLE_CONFIGS_PREFIX = "/CONFIGS/TABLE";
+  private static final String PROPERTYSTORE_USER_CONFIGS_PREFIX = "/CONFIGS/USER";
   private static final String PROPERTYSTORE_INSTANCE_CONFIGS_PREFIX = "/CONFIGS/INSTANCE";
   private static final String PROPERTYSTORE_CLUSTER_CONFIGS_PREFIX = "/CONFIGS/CLUSTER";
   private static final String PROPERTYSTORE_SEGMENT_LINEAGE = "/SEGMENT_LINEAGE";
   private static final String PROPERTYSTORE_MINION_TASK_METADATA_PREFIX = "/MINION_TASK_METADATA";
 
+  public static void setUserConfig(ZkHelixPropertyStore<ZNRecord> propertyStore, String username, ZNRecord znRecord) {
+    propertyStore.set(constructPropertyStorePathForUserConfig(username), znRecord, AccessOption.PERSISTENT);
+  }
+
   public static void setRealtimeTableConfig(ZkHelixPropertyStore<ZNRecord> propertyStore, String realtimeTableName,
       ZNRecord znRecord) {
-    propertyStore
-        .set(constructPropertyStorePathForResourceConfig(realtimeTableName), znRecord, AccessOption.PERSISTENT);
+    propertyStore.set(constructPropertyStorePathForResourceConfig(realtimeTableName), znRecord,
+        AccessOption.PERSISTENT);
   }
 
   public static void setOfflineTableConfig(ZkHelixPropertyStore<ZNRecord> propertyStore, String offlineTableName,
@@ -79,8 +90,8 @@ public class ZKMetadataProvider {
 
   public static InstanceZKMetadata getInstanceZKMetadata(ZkHelixPropertyStore<ZNRecord> propertyStore,
       String instanceId) {
-    ZNRecord znRecord = propertyStore
-        .get(StringUtil.join("/", PROPERTYSTORE_INSTANCE_CONFIGS_PREFIX, instanceId), null, AccessOption.PERSISTENT);
+    ZNRecord znRecord = propertyStore.get(StringUtil.join("/", PROPERTYSTORE_INSTANCE_CONFIGS_PREFIX, instanceId), null,
+        AccessOption.PERSISTENT);
     if (znRecord == null) {
       return null;
     }
@@ -99,6 +110,10 @@ public class ZKMetadataProvider {
     return StringUtil.join("/", PROPERTYSTORE_INSTANCE_PARTITIONS_PREFIX, instancePartitionsName);
   }
 
+  public static String constructPropertyStorePathForControllerJob() {
+    return StringUtil.join("/", PROPERTYSTORE_CONTROLLER_JOBS_PREFIX);
+  }
+
   public static String constructPropertyStorePathForResource(String resourceName) {
     return StringUtil.join("/", PROPERTYSTORE_SEGMENTS_PREFIX, resourceName);
   }
@@ -107,6 +122,10 @@ public class ZKMetadataProvider {
     return StringUtil.join("/", PROPERTYSTORE_TABLE_CONFIGS_PREFIX, resourceName);
   }
 
+  public static String constructPropertyStorePathForUserConfig(String resourceName) {
+    return StringUtil.join("/", PROPERTYSTORE_USER_CONFIGS_PREFIX, resourceName);
+  }
+
   public static String constructPropertyStorePathForControllerConfig(String controllerConfigKey) {
     return StringUtil.join("/", PROPERTYSTORE_CLUSTER_CONFIGS_PREFIX, controllerConfigKey);
   }
@@ -115,14 +134,20 @@ public class ZKMetadataProvider {
     return StringUtil.join("/", PROPERTYSTORE_SEGMENT_LINEAGE, tableNameWithType);
   }
 
-  public static String constructPropertyStorePathForMinionTaskMetadata(String taskType, String tableNameWithType) {
+  public static String constructPropertyStorePathForMinionTaskMetadata(String tableNameWithType, String taskType) {
+    return StringUtil.join("/", PROPERTYSTORE_MINION_TASK_METADATA_PREFIX, tableNameWithType, taskType);
+  }
+
+  @Deprecated
+  public static String constructPropertyStorePathForMinionTaskMetadataDeprecated(String taskType,
+      String tableNameWithType) {
     return StringUtil.join("/", PROPERTYSTORE_MINION_TASK_METADATA_PREFIX, taskType, tableNameWithType);
   }
 
   public static boolean isSegmentExisted(ZkHelixPropertyStore<ZNRecord> propertyStore, String resourceNameForResource,
       String segmentName) {
-    return propertyStore
-        .exists(constructPropertyStorePathForSegment(resourceNameForResource, segmentName), AccessOption.PERSISTENT);
+    return propertyStore.exists(constructPropertyStorePathForSegment(resourceNameForResource, segmentName),
+        AccessOption.PERSISTENT);
   }
 
   public static void removeResourceSegmentsFromPropertyStore(ZkHelixPropertyStore<ZNRecord> propertyStore,
@@ -141,13 +166,41 @@ public class ZKMetadataProvider {
     }
   }
 
+  public static void removeUserConfigFromPropertyStore(ZkHelixPropertyStore<ZNRecord> propertyStore, String username) {
+    String propertyStorePath = constructPropertyStorePathForUserConfig(username);
+    if (propertyStore.exists(propertyStorePath, AccessOption.PERSISTENT)) {
+      propertyStore.remove(propertyStorePath, AccessOption.PERSISTENT);
+    }
+  }
+
+  /**
+   * Creates a new znode for SegmentZkMetadata. This call is atomic. If there are concurrent calls trying to create the
+   * same znode, only one of them would succeed.
+   *
+   * @param propertyStore Helix property store
+   * @param tableNameWithType Table name with type
+   * @param segmentZKMetadata Segment Zk metadata
+   * @return boolean indicating success/failure
+   */
+  public static boolean createSegmentZkMetadata(ZkHelixPropertyStore<ZNRecord> propertyStore, String tableNameWithType,
+      SegmentZKMetadata segmentZKMetadata) {
+    try {
+      return propertyStore.create(
+          constructPropertyStorePathForSegment(tableNameWithType, segmentZKMetadata.getSegmentName()),
+          segmentZKMetadata.toZNRecord(), AccessOption.PERSISTENT);
+    } catch (Exception e) {
+      LOGGER.error("Caught exception while creating segmentZkMetadata for table: {}", tableNameWithType, e);
+      return false;
+    }
+  }
+
   public static boolean setSegmentZKMetadata(ZkHelixPropertyStore<ZNRecord> propertyStore, String tableNameWithType,
       SegmentZKMetadata segmentZKMetadata, int expectedVersion) {
     // NOTE: Helix will throw ZkBadVersionException if version does not match
     try {
-      return propertyStore
-          .set(constructPropertyStorePathForSegment(tableNameWithType, segmentZKMetadata.getSegmentName()),
-              segmentZKMetadata.toZNRecord(), expectedVersion, AccessOption.PERSISTENT);
+      return propertyStore.set(
+          constructPropertyStorePathForSegment(tableNameWithType, segmentZKMetadata.getSegmentName()),
+          segmentZKMetadata.toZNRecord(), expectedVersion, AccessOption.PERSISTENT);
     } catch (ZkBadVersionException e) {
       return false;
     }
@@ -173,27 +226,59 @@ public class ZKMetadataProvider {
   @Nullable
   public static SegmentZKMetadata getSegmentZKMetadata(ZkHelixPropertyStore<ZNRecord> propertyStore,
       String tableNameWithType, String segmentName) {
-    ZNRecord znRecord = propertyStore
-        .get(constructPropertyStorePathForSegment(tableNameWithType, segmentName), null, AccessOption.PERSISTENT);
+    ZNRecord znRecord = propertyStore.get(constructPropertyStorePathForSegment(tableNameWithType, segmentName), null,
+        AccessOption.PERSISTENT);
     return znRecord != null ? new SegmentZKMetadata(znRecord) : null;
   }
 
   @Nullable
-  public static TableConfig getTableConfig(ZkHelixPropertyStore<ZNRecord> propertyStore, String tableNameWithType) {
-    ZNRecord znRecord = propertyStore
-        .get(constructPropertyStorePathForResourceConfig(tableNameWithType), null, AccessOption.PERSISTENT);
+  public static UserConfig getUserConfig(ZkHelixPropertyStore<ZNRecord> propertyStore, String username) {
+    ZNRecord znRecord =
+        propertyStore.get(constructPropertyStorePathForUserConfig(username), null, AccessOption.PERSISTENT);
     if (znRecord == null) {
       return null;
     }
     try {
-      TableConfig tableConfig = TableConfigUtils.fromZNRecord(znRecord);
-      return (TableConfig) ConfigUtils.applyConfigWithEnvVariables(tableConfig);
+      UserConfig userConfig = AccessControlUserConfigUtils.fromZNRecord(znRecord);
+      return (UserConfig) ConfigUtils.applyConfigWithEnvVariables(userConfig);
     } catch (Exception e) {
-      LOGGER.error("Caught exception while getting table configuration for table: {}", tableNameWithType, e);
+      LOGGER.error("Caught exception while getting user configuration for user: {}", username, e);
       return null;
     }
   }
 
+  @Nullable
+  public static List<UserConfig> getAllUserConfig(ZkHelixPropertyStore<ZNRecord> propertyStore) {
+    List<ZNRecord> znRecordss =
+        propertyStore.getChildren(PROPERTYSTORE_USER_CONFIGS_PREFIX, null, AccessOption.PERSISTENT);
+
+    try {
+      return Optional.ofNullable(znRecordss).orElseGet(() -> {
+        return new ArrayList<>();
+      }).stream().map(AccessControlUserConfigUtils::fromZNRecord).collect(Collectors.toList());
+    } catch (Exception e) {
+      LOGGER.error("Caught exception while getting user list configuration", e);
+      return null;
+    }
+  }
+
+  @Nullable
+  public static List<String> getAllUserName(ZkHelixPropertyStore<ZNRecord> propertyStore) {
+    return propertyStore.getChildNames(PROPERTYSTORE_USER_CONFIGS_PREFIX, AccessOption.PERSISTENT);
+  }
+
+  @Nullable
+  public static Map<String, UserConfig> getAllUserInfo(ZkHelixPropertyStore<ZNRecord> propertyStore) {
+    return getAllUserConfig(propertyStore).stream()
+        .collect(Collectors.toMap(UserConfig::getUsernameWithComponent, u -> u));
+  }
+
+  @Nullable
+  public static TableConfig getTableConfig(ZkHelixPropertyStore<ZNRecord> propertyStore, String tableNameWithType) {
+    return toTableConfig(propertyStore.get(constructPropertyStorePathForResourceConfig(tableNameWithType), null,
+        AccessOption.PERSISTENT));
+  }
+
   @Nullable
   public static TableConfig getOfflineTableConfig(ZkHelixPropertyStore<ZNRecord> propertyStore, String tableName) {
     return getTableConfig(propertyStore, TableNameBuilder.OFFLINE.tableNameWithType(tableName));
@@ -204,6 +289,43 @@ public class ZKMetadataProvider {
     return getTableConfig(propertyStore, TableNameBuilder.REALTIME.tableNameWithType(tableName));
   }
 
+  public static List<TableConfig> getAllTableConfigs(ZkHelixPropertyStore<ZNRecord> propertyStore) {
+    List<ZNRecord> znRecords =
+        propertyStore.getChildren(PROPERTYSTORE_TABLE_CONFIGS_PREFIX, null, AccessOption.PERSISTENT,
+            CommonConstants.Helix.ZkClient.RETRY_COUNT, CommonConstants.Helix.ZkClient.RETRY_INTERVAL_MS);
+    if (znRecords != null) {
+      int numZNRecords = znRecords.size();
+      List<TableConfig> tableConfigs = new ArrayList<>(numZNRecords);
+      for (ZNRecord znRecord : znRecords) {
+        TableConfig tableConfig = toTableConfig(znRecord);
+        if (tableConfig != null) {
+          tableConfigs.add(tableConfig);
+        }
+      }
+      if (numZNRecords > tableConfigs.size()) {
+        LOGGER.warn("Failed to read {}/{} table configs", numZNRecords - tableConfigs.size(), numZNRecords);
+      }
+      return tableConfigs;
+    } else {
+      LOGGER.warn("Path: {} does not exist", PROPERTYSTORE_TABLE_CONFIGS_PREFIX);
+      return Collections.emptyList();
+    }
+  }
+
+  @Nullable
+  private static TableConfig toTableConfig(@Nullable ZNRecord znRecord) {
+    if (znRecord == null) {
+      return null;
+    }
+    try {
+      TableConfig tableConfig = TableConfigUtils.fromZNRecord(znRecord);
+      return ConfigUtils.applyConfigWithEnvVariables(tableConfig);
+    } catch (Exception e) {
+      LOGGER.error("Caught exception while creating table config from ZNRecord: {}", znRecord.getId(), e);
+      return null;
+    }
+  }
+
   public static void setSchema(ZkHelixPropertyStore<ZNRecord> propertyStore, Schema schema) {
     propertyStore.set(constructPropertyStorePathForSchema(schema.getSchemaName()), SchemaUtils.toZNRecord(schema),
         AccessOption.PERSISTENT);
@@ -261,8 +383,8 @@ public class ZKMetadataProvider {
         }
       }
     }
-    if (schema != null) {
-      LOGGER.warn("Schema name does not match raw table name, schema name: {}, raw table name: {}",
+    if (schema != null && LOGGER.isDebugEnabled()) {
+      LOGGER.debug("Schema name does not match raw table name, schema name: {}, raw table name: {}",
           schema.getSchemaName(), TableNameBuilder.extractRawTableName(tableName));
     }
     return schema;
@@ -275,8 +397,8 @@ public class ZKMetadataProvider {
   public static List<SegmentZKMetadata> getSegmentsZKMetadata(ZkHelixPropertyStore<ZNRecord> propertyStore,
       String tableNameWithType) {
     String parentPath = constructPropertyStorePathForResource(tableNameWithType);
-    List<ZNRecord> znRecords = propertyStore
-        .getChildren(parentPath, null, AccessOption.PERSISTENT, CommonConstants.Helix.ZkClient.RETRY_COUNT,
+    List<ZNRecord> znRecords =
+        propertyStore.getChildren(parentPath, null, AccessOption.PERSISTENT, CommonConstants.Helix.ZkClient.RETRY_COUNT,
             CommonConstants.Helix.ZkClient.RETRY_INTERVAL_MS);
     if (znRecords != null) {
       int numZNRecords = znRecords.size();
