@@ -18,15 +18,27 @@
  */
 package org.apache.pinot.controller.api.resources;
 
+import com.fasterxml.jackson.core.JsonProcessingException;
+import com.fasterxml.jackson.databind.JsonNode;
+import com.google.common.collect.HashBiMap;
 import io.swagger.annotations.Api;
+import io.swagger.annotations.ApiKeyAuthDefinition;
 import io.swagger.annotations.ApiOperation;
 import io.swagger.annotations.ApiParam;
+import io.swagger.annotations.Authorization;
+import io.swagger.annotations.SecurityDefinition;
+import io.swagger.annotations.SwaggerDefinition;
+import java.io.IOException;
+import java.net.URI;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.Executor;
+import java.util.stream.Collectors;
+import javax.annotation.Nullable;
 import javax.inject.Inject;
 import javax.ws.rs.DELETE;
 import javax.ws.rs.DefaultValue;
@@ -36,13 +48,39 @@ import javax.ws.rs.POST;
 import javax.ws.rs.PUT;
 import javax.ws.rs.Path;
 import javax.ws.rs.PathParam;
+import javax.ws.rs.Produces;
 import javax.ws.rs.QueryParam;
+import javax.ws.rs.container.AsyncResponse;
+import javax.ws.rs.container.Suspended;
+import javax.ws.rs.core.Context;
+import javax.ws.rs.core.HttpHeaders;
+import javax.ws.rs.core.MediaType;
+import javax.ws.rs.core.Response;
+import javax.ws.rs.core.UriInfo;
+import org.apache.commons.httpclient.HttpConnectionManager;
+import org.apache.commons.lang3.exception.ExceptionUtils;
+import org.apache.helix.model.InstanceConfig;
+import org.apache.helix.task.TaskPartitionState;
 import org.apache.helix.task.TaskState;
+import org.apache.pinot.common.exception.TableNotFoundException;
+import org.apache.pinot.common.minion.BaseTaskGeneratorInfo;
+import org.apache.pinot.common.minion.TaskManagerStatusCache;
 import org.apache.pinot.controller.api.access.AccessType;
 import org.apache.pinot.controller.api.access.Authenticate;
+import org.apache.pinot.controller.api.exception.ControllerApplicationException;
+import org.apache.pinot.controller.api.exception.NoTaskMetadataException;
+import org.apache.pinot.controller.api.exception.NoTaskScheduledException;
+import org.apache.pinot.controller.api.exception.TaskAlreadyExistsException;
+import org.apache.pinot.controller.api.exception.UnknownTaskTypeException;
+import org.apache.pinot.controller.helix.core.PinotHelixResourceManager;
 import org.apache.pinot.controller.helix.core.minion.PinotHelixTaskResourceManager;
 import org.apache.pinot.controller.helix.core.minion.PinotTaskManager;
+import org.apache.pinot.controller.util.CompletionServiceHelper;
 import org.apache.pinot.core.minion.PinotTaskConfig;
+import org.apache.pinot.spi.config.task.AdhocTaskConfig;
+import org.apache.pinot.spi.utils.JsonUtils;
+import org.glassfish.grizzly.http.server.Request;
+import org.glassfish.jersey.server.ManagedAsync;
 import org.quartz.CronTrigger;
 import org.quartz.JobDataMap;
 import org.quartz.JobDetail;
@@ -53,6 +91,10 @@ import org.quartz.SchedulerMetaData;
 import org.quartz.SimpleTrigger;
 import org.quartz.Trigger;
 import org.quartz.impl.matchers.GroupMatcher;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import static org.apache.pinot.spi.utils.CommonConstants.SWAGGER_AUTHORIZATION_KEY;
 
 
 /**
@@ -65,6 +107,7 @@ import org.quartz.impl.matchers.GroupMatcher;
  *   <li>GET '/tasks/task/{taskName}/state': Get the task state for the given task</li>
  *   <li>GET '/tasks/task/{taskName}/config': Get the task config (a list of child task configs) for the given task</li>
  *   <li>POST '/tasks/schedule': Schedule tasks</li>
+ *   <li>POST '/tasks/execute': Execute an adhoc task</li>
  *   <li>PUT '/tasks/{taskType}/cleanup': Clean up finished tasks (COMPLETED, FAILED) for the given task type</li>
  *   <li>PUT '/tasks/{taskType}/stop': Stop all running/pending tasks (as well as the task queue) for the given task
  *   type</li>
@@ -73,9 +116,13 @@ import org.quartz.impl.matchers.GroupMatcher;
  *   <li>DELETE '/tasks/{taskType}': Delete all tasks (as well as the task queue) for the given task type</li>
  * </ul>
  */
-@Api(tags = Constants.TASK_TAG)
+@Api(tags = Constants.TASK_TAG, authorizations = {@Authorization(value = SWAGGER_AUTHORIZATION_KEY)})
+@SwaggerDefinition(securityDefinition = @SecurityDefinition(apiKeyAuthDefinitions = @ApiKeyAuthDefinition(name =
+    HttpHeaders.AUTHORIZATION, in = ApiKeyAuthDefinition.ApiKeyLocation.HEADER, key = SWAGGER_AUTHORIZATION_KEY)))
 @Path("/")
 public class PinotTaskRestletResource {
+  public static final Logger LOGGER = LoggerFactory.getLogger(PinotTaskRestletResource.class);
+
   private static final String TASK_QUEUE_STATE_STOP = "STOP";
   private static final String TASK_QUEUE_STATE_RESUME = "RESUME";
 
@@ -85,6 +132,21 @@ public class PinotTaskRestletResource {
   @Inject
   PinotTaskManager _pinotTaskManager;
 
+  @Inject
+  TaskManagerStatusCache _taskManagerStatusCache;
+
+  @Inject
+  PinotHelixResourceManager _pinotHelixResourceManager;
+
+  @Inject
+  Executor _executor;
+
+  @Inject
+  HttpConnectionManager _connectionManager;
+
+  @Context
+  private UriInfo _uriInfo;
+
   @GET
   @Path("/tasks/tasktypes")
   @ApiOperation("List all task types")
@@ -134,6 +196,36 @@ public class PinotTaskRestletResource {
     return _pinotHelixTaskResourceManager.getTaskStatesByTable(taskType, tableNameWithType);
   }
 
+  @GET
+  @Path("/tasks/{taskType}/{tableNameWithType}/metadata")
+  @ApiOperation("Get task metadata for the given task type and table")
+  public String getTaskMetadataByTable(
+      @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType,
+      @ApiParam(value = "Table name with type", required = true) @PathParam("tableNameWithType")
+          String tableNameWithType) {
+    try {
+      return _pinotHelixTaskResourceManager.getTaskMetadataByTable(taskType, tableNameWithType);
+    } catch (NoTaskMetadataException e) {
+      throw new ControllerApplicationException(LOGGER, e.getMessage(), Response.Status.NOT_FOUND);
+    } catch (JsonProcessingException e) {
+      throw new ControllerApplicationException(LOGGER, String
+          .format("Failed to format task metadata into Json for task type: %s from table: %s", taskType,
+              tableNameWithType), Response.Status.INTERNAL_SERVER_ERROR, e);
+    }
+  }
+
+  @DELETE
+  @Path("/tasks/{taskType}/{tableNameWithType}/metadata")
+  @ApiOperation("Delete task metadata for the given task type and table")
+  public SuccessResponse deleteTaskMetadataByTable(
+      @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType,
+      @ApiParam(value = "Table name with type", required = true) @PathParam("tableNameWithType")
+          String tableNameWithType) {
+    _pinotHelixTaskResourceManager.deleteTaskMetadataByTable(taskType, tableNameWithType);
+    return new SuccessResponse(
+        String.format("Successfully deleted metadata for task type: %s from table: %s", taskType, tableNameWithType));
+  }
+
   @GET
   @Path("/tasks/{taskType}/taskcounts")
   @ApiOperation("Fetch count of sub-tasks for each of the tasks for the given task type")
@@ -147,17 +239,89 @@ public class PinotTaskRestletResource {
   @ApiOperation("Fetch information for all the tasks for the given task type")
   public Map<String, PinotHelixTaskResourceManager.TaskDebugInfo> getTasksDebugInfo(
       @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType,
-      @ApiParam(value = "verbosity (By default, prints for running and error tasks. Value of >0 prints for all tasks)")
+      @ApiParam(value = "verbosity (Prints information for all the tasks for the given task type."
+          + "By default, only prints subtask details for running and error tasks. "
+          + "Value of > 0 prints subtask details for all tasks)")
       @DefaultValue("0") @QueryParam("verbosity") int verbosity) {
     return _pinotHelixTaskResourceManager.getTasksDebugInfo(taskType, verbosity);
   }
 
+  @GET
+  @Path("/tasks/{taskType}/{tableNameWithType}/debug")
+  @ApiOperation("Fetch information for all the tasks for the given task type and table")
+  public Map<String, PinotHelixTaskResourceManager.TaskDebugInfo> getTasksDebugInfo(
+      @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType,
+      @ApiParam(value = "Table name with type", required = true) @PathParam("tableNameWithType")
+          String tableNameWithType,
+      @ApiParam(value = "verbosity (Prints information for all the tasks for the given task type and table."
+          + "By default, only prints subtask details for running and error tasks. "
+          + "Value of > 0 prints subtask details for all tasks)")
+      @DefaultValue("0") @QueryParam("verbosity") int verbosity) {
+    return _pinotHelixTaskResourceManager.getTasksDebugInfoByTable(taskType, tableNameWithType, verbosity);
+  }
+
+  @GET
+  @Produces(MediaType.APPLICATION_JSON)
+  @Path("/tasks/generator/{tableNameWithType}/{taskType}/debug")
+  @ApiOperation("Fetch task generation information for the recent runs of the given task for the given table")
+  public String getTaskGenerationDebugInto(
+      @Context HttpHeaders httpHeaders,
+      @ApiParam(value = "Task type", required = true) @PathParam("taskType") String taskType,
+      @ApiParam(value = "Table name with type", required = true) @PathParam("tableNameWithType")
+          String tableNameWithType,
+      @ApiParam(value = "Whether to only lookup local cache for logs", defaultValue = "false") @QueryParam("localOnly")
+          boolean localOnly)
+      throws JsonProcessingException {
+    if (localOnly) {
+      BaseTaskGeneratorInfo taskGeneratorMostRecentRunInfo =
+          _taskManagerStatusCache.fetchTaskGeneratorInfo(tableNameWithType, taskType);
+      if (taskGeneratorMostRecentRunInfo == null) {
+        throw new ControllerApplicationException(LOGGER, "Task generation information not found",
+            Response.Status.NOT_FOUND);
+      }
+
+      return JsonUtils.objectToString(taskGeneratorMostRecentRunInfo);
+    }
+
+    // Call all controllers
+    List<InstanceConfig> controllers = _pinotHelixResourceManager.getAllControllerInstanceConfigs();
+    // Relying on original schema that was used to query the controller
+    URI uri = _uriInfo.getRequestUri();
+    String scheme = uri.getScheme();
+    List<String> controllerUrls = controllers.stream().map(controller -> {
+      return String.format("%s://%s:%d/tasks/generator/%s/%s/debug?localOnly=true", scheme, controller.getHostName(),
+          Integer.parseInt(controller.getPort()), tableNameWithType, taskType);
+    }).collect(Collectors.toList());
+
+    CompletionServiceHelper completionServiceHelper =
+        new CompletionServiceHelper(_executor, _connectionManager, HashBiMap.create(0));
+    Map<String, String> requestHeaders = new HashMap<>();
+    httpHeaders.getRequestHeaders().keySet().forEach(header -> {
+      requestHeaders.put(header, httpHeaders.getHeaderString(header));
+    });
+    CompletionServiceHelper.CompletionServiceResponse serviceResponse =
+        completionServiceHelper.doMultiGetRequest(controllerUrls, null, true, requestHeaders, 10000);
+
+    List<JsonNode> result = new ArrayList<>();
+    serviceResponse._httpResponses.values().forEach(resp -> {
+      try {
+        result.add(JsonUtils.stringToJsonNode(resp));
+      } catch (IOException e) {
+        LOGGER.error("Failed to parse controller response {}", resp, e);
+      }
+    });
+
+    return JsonUtils.objectToString(result);
+  }
+
   @GET
   @Path("/tasks/task/{taskName}/debug")
   @ApiOperation("Fetch information for the given task name")
   public PinotHelixTaskResourceManager.TaskDebugInfo getTaskDebugInfo(
       @ApiParam(value = "Task name", required = true) @PathParam("taskName") String taskName,
-      @ApiParam(value = "verbosity (By default, prints for running and error tasks. Value of >0 prints for all tasks)")
+      @ApiParam(value = "verbosity (Prints information for the given task name."
+          + "By default, only prints subtask details for running and error tasks. "
+          + "Value of > 0 prints subtask details for all tasks)")
       @DefaultValue("0") @QueryParam("verbosity") int verbosity) {
     return _pinotHelixTaskResourceManager.getTaskDebugInfo(taskName, verbosity);
   }
@@ -205,6 +369,14 @@ public class PinotTaskRestletResource {
     return new StringResultResponse(_pinotHelixTaskResourceManager.getTaskState(taskName).toString());
   }
 
+  @GET
+  @Path("/tasks/subtask/{taskName}/state")
+  @ApiOperation("Get the states of all the sub tasks for the given task")
+  public Map<String, TaskPartitionState> getSubtaskStates(
+      @ApiParam(value = "Task name", required = true) @PathParam("taskName") String taskName) {
+    return _pinotHelixTaskResourceManager.getSubtaskStates(taskName);
+  }
+
   @GET
   @Path("/tasks/task/{taskName}/config")
   @ApiOperation("Get the task config (a list of child task configs) for the given task")
@@ -222,6 +394,16 @@ public class PinotTaskRestletResource {
     return _pinotHelixTaskResourceManager.getTaskConfigs(taskName);
   }
 
+  @GET
+  @Path("/tasks/subtask/{taskName}/config")
+  @ApiOperation("Get the configs of specified sub tasks for the given task")
+  public Map<String, PinotTaskConfig> getSubtaskConfigs(
+      @ApiParam(value = "Task name", required = true) @PathParam("taskName") String taskName,
+      @ApiParam(value = "Sub task names separated by comma") @QueryParam("subtaskNames") @Nullable
+          String subtaskNames) {
+    return _pinotHelixTaskResourceManager.getSubtaskConfigs(taskName, subtaskNames);
+  }
+
   @GET
   @Path("/tasks/scheduler/information")
   @ApiOperation("Fetch cron scheduler information")
@@ -313,6 +495,8 @@ public class PinotTaskRestletResource {
           triggerMap.put("RepeatInterval", simpleTrigger.getRepeatInterval());
           triggerMap.put("RepeatCount", simpleTrigger.getRepeatCount());
           triggerMap.put("TimesTriggered", simpleTrigger.getTimesTriggered());
+          triggerMap.put("NextFireTime", simpleTrigger.getNextFireTime());
+          triggerMap.put("PreviousFireTime", simpleTrigger.getPreviousFireTime());
         } else if (trigger instanceof CronTrigger) {
           CronTrigger cronTrigger = (CronTrigger) trigger;
           triggerMap.put("TriggerType", CronTrigger.class.getSimpleName());
@@ -346,6 +530,36 @@ public class PinotTaskRestletResource {
     }
   }
 
+  @POST
+  @ManagedAsync
+  @Produces(MediaType.APPLICATION_JSON)
+  @Path("/tasks/execute")
+  @Authenticate(AccessType.CREATE)
+  @ApiOperation("Execute a task on minion")
+  public void executeAdhocTask(AdhocTaskConfig adhocTaskConfig, @Suspended AsyncResponse asyncResponse,
+      @Context Request requestContext) {
+    try {
+      asyncResponse.resume(_pinotTaskManager.createTask(adhocTaskConfig.getTaskType(), adhocTaskConfig.getTableName(),
+          adhocTaskConfig.getTaskName(), adhocTaskConfig.getTaskConfigs()));
+    } catch (TableNotFoundException e) {
+      throw new ControllerApplicationException(LOGGER, "Failed to find table: " + adhocTaskConfig.getTableName(),
+          Response.Status.NOT_FOUND, e);
+    } catch (TaskAlreadyExistsException e) {
+      throw new ControllerApplicationException(LOGGER, "Task already exists: " + adhocTaskConfig.getTaskName(),
+          Response.Status.CONFLICT, e);
+    } catch (UnknownTaskTypeException e) {
+      throw new ControllerApplicationException(LOGGER, "Unknown task type: " + adhocTaskConfig.getTaskType(),
+          Response.Status.NOT_FOUND, e);
+    } catch (NoTaskScheduledException e) {
+      throw new ControllerApplicationException(LOGGER,
+          "No task is generated for table: " + adhocTaskConfig.getTableName() + ", with task type: "
+              + adhocTaskConfig.getTaskType(), Response.Status.BAD_REQUEST);
+    } catch (Exception e) {
+      throw new ControllerApplicationException(LOGGER,
+          "Failed to create adhoc task: " + ExceptionUtils.getStackTrace(e), Response.Status.INTERNAL_SERVER_ERROR, e);
+    }
+  }
+
   @Deprecated
   @PUT
   @Path("/tasks/scheduletasks")
@@ -428,6 +642,18 @@ public class PinotTaskRestletResource {
     return new SuccessResponse("Successfully deleted tasks for task type: " + taskType);
   }
 
+  @DELETE
+  @Path("/tasks/task/{taskName}")
+  @Authenticate(AccessType.DELETE)
+  @ApiOperation("Delete a single task given its task name")
+  public SuccessResponse deleteTask(
+      @ApiParam(value = "Task name", required = true) @PathParam("taskName") String taskName,
+      @ApiParam(value = "Whether to force deleting the task (expert only option, enable with cautious")
+      @DefaultValue("false") @QueryParam("forceDelete") boolean forceDelete) {
+    _pinotHelixTaskResourceManager.deleteTask(taskName, forceDelete);
+    return new SuccessResponse("Successfully deleted task: " + taskName);
+  }
+
   @Deprecated
   @DELETE
   @Path("/tasks/taskqueue/{taskType}")
