@@ -27,15 +27,15 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
-import org.I0Itec.zkclient.IZkChildListener;
-import org.I0Itec.zkclient.IZkDataListener;
-import org.apache.helix.PropertyPathConfig;
-import org.apache.helix.PropertyType;
-import org.apache.helix.ZNRecord;
-import org.apache.helix.manager.zk.ZNRecordSerializer;
-import org.apache.helix.manager.zk.ZkClient;
+import org.apache.commons.lang3.tuple.Pair;
+import org.apache.helix.PropertyPathBuilder;
 import org.apache.helix.model.IdealState;
 import org.apache.helix.store.HelixPropertyListener;
+import org.apache.helix.zookeeper.datamodel.ZNRecord;
+import org.apache.helix.zookeeper.datamodel.serializer.ZNRecordSerializer;
+import org.apache.helix.zookeeper.impl.client.ZkClient;
+import org.apache.helix.zookeeper.zkclient.IZkChildListener;
+import org.apache.helix.zookeeper.zkclient.IZkDataListener;
 import org.apache.pinot.common.Utils;
 import org.apache.pinot.common.metadata.ZKMetadataProvider;
 import org.apache.pinot.common.metadata.instance.InstanceZKMetadata;
@@ -54,9 +54,7 @@ import org.apache.pinot.spi.config.table.TableType;
 import org.apache.pinot.spi.stream.StreamConfig;
 import org.apache.pinot.spi.utils.CommonConstants;
 import org.apache.pinot.spi.utils.CommonConstants.Segment.Realtime.Status;
-import org.apache.pinot.spi.utils.CommonConstants.Segment.SegmentType;
 import org.apache.pinot.spi.utils.IngestionConfigUtils;
-import org.apache.pinot.spi.utils.Pair;
 import org.apache.pinot.spi.utils.builder.TableNameBuilder;
 import org.apache.pinot.spi.utils.retry.RetryPolicies;
 import org.apache.zookeeper.data.Stat;
@@ -89,7 +87,7 @@ public class PinotRealtimeSegmentManager implements HelixPropertyListener, IZkCh
     _pinotHelixResourceManager = pinotManager;
     _leadControllerManager = leadControllerManager;
     String clusterName = _pinotHelixResourceManager.getHelixClusterName();
-    _propertyStorePath = PropertyPathConfig.getPath(PropertyType.PROPERTYSTORE, clusterName);
+    _propertyStorePath = PropertyPathBuilder.propertyStore(clusterName);
     _tableConfigPath = _propertyStorePath + TABLE_CONFIG;
   }
 
@@ -175,7 +173,7 @@ public class PinotRealtimeSegmentManager implements HelixPropertyListener, IZkCh
             String groupId = instanceZKMetadata.getGroupId(realtimeTableName);
             String partitionId = instanceZKMetadata.getPartition(realtimeTableName);
             if (groupId != null && !groupId.isEmpty() && partitionId != null && !partitionId.isEmpty()) {
-              listOfSegmentsToAddToInstances.add(new Pair<String, String>(
+              listOfSegmentsToAddToInstances.add(Pair.of(
                   new HLCSegmentName(groupId, partitionId, String.valueOf(System.currentTimeMillis())).getSegmentName(),
                   instanceId));
             } else {
@@ -219,7 +217,7 @@ public class PinotRealtimeSegmentManager implements HelixPropertyListener, IZkCh
             InstanceZKMetadata instanceZKMetadata = _pinotHelixResourceManager.getInstanceZKMetadata(instanceId);
             String groupId = instanceZKMetadata.getGroupId(realtimeTableName);
             String partitionId = instanceZKMetadata.getPartition(realtimeTableName);
-            listOfSegmentsToAddToInstances.add(new Pair<String, String>(
+            listOfSegmentsToAddToInstances.add(Pair.of(
                 new HLCSegmentName(groupId, partitionId, String.valueOf(System.currentTimeMillis())).getSegmentName(),
                 instanceId));
           }
@@ -235,8 +233,8 @@ public class PinotRealtimeSegmentManager implements HelixPropertyListener, IZkCh
 
     // Add the new segments to the server instances
     for (final Pair<String, String> segmentIdAndInstanceId : listOfSegmentsToAddToInstances) {
-      final String segmentId = segmentIdAndInstanceId.getFirst();
-      final String instanceName = segmentIdAndInstanceId.getSecond();
+      final String segmentId = segmentIdAndInstanceId.getLeft();
+      final String instanceName = segmentIdAndInstanceId.getRight();
 
       try {
         final HLCSegmentName segName = new HLCSegmentName(segmentId);
@@ -247,8 +245,6 @@ public class PinotRealtimeSegmentManager implements HelixPropertyListener, IZkCh
           // No, add it
           // Create the realtime segment metadata
           SegmentZKMetadata segmentMetadataToAdd = new SegmentZKMetadata(segmentId);
-          segmentMetadataToAdd.setTableName(resourceName);
-          segmentMetadataToAdd.setSegmentType(SegmentType.REALTIME);
           segmentMetadataToAdd.setStatus(Status.IN_PROGRESS);
 
           // Add the new metadata to the property store
