@@ -23,7 +23,6 @@ import { FormControlLabel, Grid, Switch, Tooltip } from '@material-ui/core';
 import { RouteComponentProps, useHistory, useLocation } from 'react-router-dom';
 import { UnControlled as CodeMirror } from 'react-codemirror2';
 import { TableData } from 'Models';
-import _ from 'lodash';
 import AppLoader from '../components/AppLoader';
 import CustomizedTables from '../components/Table';
 import TableToolbar from '../components/TableToolbar';
@@ -468,14 +467,14 @@ const TenantPageDetails = ({ match }: RouteComponentProps<Props>) => {
           </Grid>
           <Tooltip title="Uncompressed size of all data segments"  arrow placement="top-start">
           <Grid item xs={2}>
-            <strong>Reported Size:</strong> {tableSummary.reportedSize}
+            <strong>Reported Size:</strong> {Utils.formatBytes(tableSummary.reportedSize)}
           </Grid>
           </Tooltip>
           <Grid item xs={2}></Grid>
           <Tooltip title="Estimated size of all data segments, in case any servers are not reachable for actual size" arrow placement="top-start">
             <Grid item xs={2}>
               <strong>Estimated Size: </strong>
-              {tableSummary.estimatedSize}
+              {Utils.formatBytes(tableSummary.estimatedSize)}
             </Grid>
           </Tooltip>
           <Grid item xs={2}></Grid>
@@ -498,10 +497,9 @@ const TenantPageDetails = ({ match }: RouteComponentProps<Props>) => {
             </SimpleAccordion>
           </div>
           <CustomizedTables
-            title="Segments"
+            title={"Segments - " + segmentList.records.length}
             data={segmentList}
-            isPagination={false}
-            noOfRows={segmentList.records.length}
+            isPagination={true}
             baseURL={
               tenantName && `/tenants/${tenantName}/table/${tableName}/` ||
               instanceName && `/instance/${instanceName}/table/${tableName}/` ||
@@ -548,7 +546,7 @@ const TenantPageDetails = ({ match }: RouteComponentProps<Props>) => {
           </div>
           }
           <CustomizedTables
-            title="Instance Count"
+            title={"Instance Count - " + instanceCountData.records.length}
             data={instanceCountData}
             isPagination={false}
             noOfRows={instanceCountData.records.length}
