@@ -18,7 +18,10 @@
  * under the License.
  */
 
-import _ from 'lodash';
+import React from 'react';
+import ReactDiffViewer, {DiffMethod} from 'react-diff-viewer';
+import { map, isEqual, findIndex, findLast } from 'lodash';
+import app_state from '../app_state';
 
 const sortArray = function (sortingArr, keyName, ascendingFlag) {
   if (ascendingFlag) {
@@ -51,7 +54,7 @@ const tableFormat = (data) => {
   rows.forEach((singleRow) => {
     const obj = {};
     singleRow.forEach((val: any, index: number) => {
-      obj[header[index]] = val;
+      obj[header[index]+app_state.columnNameSeparator+index] = val;
     });
     results.push(obj);
   });
@@ -66,14 +69,44 @@ const getSegmentStatus = (idealStateObj, externalViewObj) => {
   const externalSegmentCount = externalSegmentKeys.length;
 
   if (idealSegmentCount !== externalSegmentCount) {
-    return 'Bad';
+    let segmentStatusComponent = (
+        <ReactDiffViewer
+            oldValue={JSON.stringify(idealStateObj, null, 2)}
+            newValue={JSON.stringify(externalViewObj, null, 2)}
+            splitView={true}
+            showDiffOnly={true}
+            leftTitle={"Ideal State"}
+            rightTitle={"External View"}
+            compareMethod={DiffMethod.WORDS}
+        />
+    )
+    return {
+      value: 'Bad',
+      tooltip: `Ideal Segment Count: ${idealSegmentCount} does not match external Segment Count: ${externalSegmentCount}`,
+      component: segmentStatusComponent,
+    };
   }
 
-  let segmentStatus = 'Good';
+  let segmentStatus = {value: 'Good', tooltip: null, component: null};
   idealSegmentKeys.map((segmentKey) => {
-    if (segmentStatus === 'Good') {
-      if (!_.isEqual(idealStateObj[segmentKey], externalViewObj[segmentKey])) {
-        segmentStatus = 'Bad';
+    if (segmentStatus.value === 'Good') {
+      if (!isEqual(idealStateObj[segmentKey], externalViewObj[segmentKey])) {
+        let segmentStatusComponent = (
+            <ReactDiffViewer
+                oldValue={JSON.stringify(idealStateObj, null, 2)}
+                newValue={JSON.stringify(externalViewObj, null, 2)}
+                splitView={true}
+                showDiffOnly={true}
+                leftTitle={"Ideal State"}
+                rightTitle={"External View"}
+                compareMethod={DiffMethod.WORDS}
+            />
+        )
+        segmentStatus = {
+          value: 'Bad',
+          tooltip: "Ideal Status does not match external status",
+          component: segmentStatusComponent
+        };
       }
     }
   });
@@ -95,7 +128,7 @@ const generateCodeMirrorOptions = (array, type, modeType?) => {
   const arr = [];
   // eslint-disable-next-line no-shadow
   const nestedFields = (arrayList, type, level, oldObj?) => {
-    _.map(arrayList, (a) => {
+    map(arrayList, (a) => {
       const obj = {
         text: a.displayName || a.name || a,
         displayText: a.displayName || a.name || a,
@@ -129,7 +162,7 @@ const generateCodeMirrorOptions = (array, type, modeType?) => {
       if (oldObj === undefined) {
         arr.push(obj);
       } else {
-        const index = _.findIndex(
+        const index = findIndex(
           arr,
           (n) => n.filterText === oldObj.filterText
         );
@@ -167,7 +200,7 @@ const generateCodeMirrorOptions = (array, type, modeType?) => {
 
 const pushNestedObjectInArray = (pathArr, obj, targetList, modeType) => {
   const rollOverFields = (target) => {
-    _.map(target, (list) => {
+    map(target, (list) => {
       if (pathArr === list.filterText) {
         if (modeType === 'sql') {
           obj.displayText = `${pathArr}.${obj.displayText}`;
@@ -194,7 +227,7 @@ const pushNestedObjectInArray = (pathArr, obj, targetList, modeType) => {
 const getNestedObjPathFromList = (list, obj) => {
   const str = [];
   const recursiveFunc = (arr, level) => {
-    _.map(arr, (a) => {
+    map(arr, (a) => {
       if (a.fields) {
         str.push(a.filterText);
         recursiveFunc(a.fields, level + 1);
@@ -202,7 +235,7 @@ const getNestedObjPathFromList = (list, obj) => {
         str.push(a.filterText);
       }
     });
-    return _.findLast(str);
+    return findLast(str);
   };
   return recursiveFunc(list, 0);
 };
@@ -291,6 +324,27 @@ const encodeString = (str: string) => {
   return str;
 }
 
+const formatBytes = (bytes, decimals = 2) => {
+  if (bytes === 0) return '0 Bytes';
+
+  const k = 1024;
+  const dm = decimals < 0 ? 0 : decimals;
+  const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
+
+  const i = Math.floor(Math.log(bytes) / Math.log(k));
+
+  return parseFloat((bytes / Math.pow(k, i)).toFixed(dm)) + ' ' + sizes[i];
+}
+
+const splitStringByLastUnderscore = (str: string) => {
+  if (!str.includes('_')) {
+    return [str, ''];
+  }
+  let beforeUnderscore = str.substring(0, str.lastIndexOf("_"));
+  let afterUnderscore = str.substring(str.lastIndexOf("_") + 1, str.length);
+  return [beforeUnderscore, afterUnderscore];
+}
+
 export default {
   sortArray,
   tableFormat,
@@ -300,5 +354,7 @@ export default {
   serialize,
   navigateToPreviousPage,
   syncTableSchemaData,
-  encodeString
+  encodeString,
+  formatBytes,
+  splitStringByLastUnderscore
 };
