@@ -20,11 +20,8 @@ package org.apache.pinot.core.data.manager.realtime;
 
 import com.google.common.base.Preconditions;
 import java.io.File;
-import java.io.FilenameFilter;
 import java.io.IOException;
 import java.net.URI;
-import java.util.HashMap;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.UUID;
@@ -33,6 +30,7 @@ import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Semaphore;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicBoolean;
 import javax.annotation.concurrent.ThreadSafe;
 import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.io.FileUtils;
@@ -48,28 +46,31 @@ import org.apache.pinot.common.utils.SegmentUtils;
 import org.apache.pinot.common.utils.TarGzCompressionUtils;
 import org.apache.pinot.common.utils.fetcher.SegmentFetcherFactory;
 import org.apache.pinot.core.data.manager.BaseTableDataManager;
+import org.apache.pinot.core.data.manager.offline.ImmutableSegmentDataManager;
 import org.apache.pinot.core.util.PeerServerSegmentFinder;
 import org.apache.pinot.segment.local.data.manager.SegmentDataManager;
+import org.apache.pinot.segment.local.dedup.PartitionDedupMetadataManager;
+import org.apache.pinot.segment.local.dedup.TableDedupMetadataManager;
 import org.apache.pinot.segment.local.indexsegment.immutable.ImmutableSegmentImpl;
 import org.apache.pinot.segment.local.indexsegment.immutable.ImmutableSegmentLoader;
 import org.apache.pinot.segment.local.realtime.impl.RealtimeSegmentStatsHistory;
 import org.apache.pinot.segment.local.segment.index.loader.IndexLoadingConfig;
 import org.apache.pinot.segment.local.segment.index.loader.LoaderUtils;
-import org.apache.pinot.segment.local.segment.readers.PinotSegmentColumnReader;
 import org.apache.pinot.segment.local.segment.virtualcolumn.VirtualColumnProviderFactory;
-import org.apache.pinot.segment.local.upsert.PartialUpsertHandler;
 import org.apache.pinot.segment.local.upsert.PartitionUpsertMetadataManager;
 import org.apache.pinot.segment.local.upsert.TableUpsertMetadataManager;
+import org.apache.pinot.segment.local.upsert.TableUpsertMetadataManagerFactory;
 import org.apache.pinot.segment.local.utils.SchemaUtils;
+import org.apache.pinot.segment.local.utils.tablestate.TableStateUtils;
 import org.apache.pinot.segment.spi.ImmutableSegment;
-import org.apache.pinot.segment.spi.index.ThreadSafeMutableRoaringBitmap;
+import org.apache.pinot.segment.spi.IndexSegment;
+import org.apache.pinot.segment.spi.creator.SegmentGeneratorConfig;
+import org.apache.pinot.spi.config.table.DedupConfig;
 import org.apache.pinot.spi.config.table.IndexingConfig;
 import org.apache.pinot.spi.config.table.TableConfig;
 import org.apache.pinot.spi.config.table.UpsertConfig;
 import org.apache.pinot.spi.data.FieldSpec;
 import org.apache.pinot.spi.data.Schema;
-import org.apache.pinot.spi.data.readers.PrimaryKey;
-import org.apache.pinot.spi.utils.ByteArray;
 import org.apache.pinot.spi.utils.CommonConstants;
 import org.apache.pinot.spi.utils.CommonConstants.Segment.Realtime.Status;
 
@@ -110,10 +111,10 @@ public class RealtimeTableDataManager extends BaseTableDataManager {
   // likely that we get fresh data each time instead of multiple copies of roughly same data.
   private static final int MIN_INTERVAL_BETWEEN_STATS_UPDATES_MINUTES = 30;
 
-  private UpsertConfig.Mode _upsertMode;
+  private final AtomicBoolean _allSegmentsLoaded = new AtomicBoolean();
+
+  private TableDedupMetadataManager _tableDedupMetadataManager;
   private TableUpsertMetadataManager _tableUpsertMetadataManager;
-  private List<String> _primaryKeyColumns;
-  private String _upsertComparisonColumn;
 
   public RealtimeTableDataManager(Semaphore segmentBuildSemaphore) {
     _segmentBuildSemaphore = segmentBuildSemaphore;
@@ -127,9 +128,8 @@ public class RealtimeTableDataManager extends BaseTableDataManager {
     try {
       _statsHistory = RealtimeSegmentStatsHistory.deserialzeFrom(statsFile);
     } catch (IOException | ClassNotFoundException e) {
-      _logger
-          .error("Error reading history object for table {} from {}", _tableNameWithType, statsFile.getAbsolutePath(),
-              e);
+      _logger.error("Error reading history object for table {} from {}", _tableNameWithType,
+          statsFile.getAbsolutePath(), e);
       File savedFile = new File(_tableDataDir, STATS_FILE_NAME + "." + UUID.randomUUID());
       try {
         FileUtils.moveFile(statsFile, savedFile);
@@ -150,41 +150,10 @@ public class RealtimeTableDataManager extends BaseTableDataManager {
 
     String consumerDirPath = getConsumerDir();
     File consumerDir = new File(consumerDirPath);
-
-    // NOTE: Upsert has to be set up when starting the server. Changing the table config without restarting the server
-    //       won't enable/disable the upsert on the fly.
-    TableConfig tableConfig = ZKMetadataProvider.getTableConfig(_propertyStore, _tableNameWithType);
-    Preconditions.checkState(tableConfig != null, "Failed to find table config for table: %s", _tableNameWithType);
-    _upsertMode = tableConfig.getUpsertMode();
-    if (isUpsertEnabled()) {
-      UpsertConfig upsertConfig = tableConfig.getUpsertConfig();
-      assert upsertConfig != null;
-      Schema schema = ZKMetadataProvider.getTableSchema(_propertyStore, _tableNameWithType);
-      Preconditions.checkState(schema != null, "Failed to find schema for table: %s", _tableNameWithType);
-
-      PartialUpsertHandler partialUpsertHandler = null;
-      if (isPartialUpsertEnabled()) {
-        partialUpsertHandler =
-            new PartialUpsertHandler(_helixManager, _tableNameWithType, upsertConfig.getPartialUpsertStrategies());
-      }
-      UpsertConfig.HashFunction hashFunction = upsertConfig.getHashFunction();
-      _tableUpsertMetadataManager =
-          new TableUpsertMetadataManager(_tableNameWithType, _serverMetrics, partialUpsertHandler, hashFunction);
-      _primaryKeyColumns = schema.getPrimaryKeyColumns();
-      Preconditions.checkState(!CollectionUtils.isEmpty(_primaryKeyColumns),
-          "Primary key columns must be configured for upsert");
-      String comparisonColumn = upsertConfig.getComparisonColumn();
-      _upsertComparisonColumn =
-          comparisonColumn != null ? comparisonColumn : tableConfig.getValidationConfig().getTimeColumnName();
-    }
-
     if (consumerDir.exists()) {
-      File[] segmentFiles = consumerDir.listFiles(new FilenameFilter() {
-        @Override
-        public boolean accept(File dir, String name) {
-          return !name.equals(STATS_FILE_NAME);
-        }
-      });
+      File[] segmentFiles = consumerDir.listFiles((dir, name) -> !name.equals(STATS_FILE_NAME));
+      Preconditions.checkState(segmentFiles != null, "Failed to list segment files from consumer dir: {} for table: {}",
+          consumerDirPath, _tableNameWithType);
       for (File file : segmentFiles) {
         if (file.delete()) {
           _logger.info("Deleted old file {}", file.getAbsolutePath());
@@ -193,6 +162,34 @@ public class RealtimeTableDataManager extends BaseTableDataManager {
         }
       }
     }
+
+    // Set up dedup/upsert metadata manager
+    // NOTE: Dedup/upsert has to be set up when starting the server. Changing the table config without restarting the
+    //       server won't enable/disable them on the fly.
+    TableConfig tableConfig = ZKMetadataProvider.getTableConfig(_propertyStore, _tableNameWithType);
+    Preconditions.checkState(tableConfig != null, "Failed to find table config for table: %s", _tableNameWithType);
+
+    DedupConfig dedupConfig = tableConfig.getDedupConfig();
+    boolean dedupEnabled = dedupConfig != null && dedupConfig.isDedupEnabled();
+    if (dedupEnabled) {
+      Schema schema = ZKMetadataProvider.getTableSchema(_propertyStore, _tableNameWithType);
+      Preconditions.checkState(schema != null, "Failed to find schema for table: %s", _tableNameWithType);
+
+      List<String> primaryKeyColumns = schema.getPrimaryKeyColumns();
+      Preconditions.checkState(!CollectionUtils.isEmpty(primaryKeyColumns),
+          "Primary key columns must be configured for dedup");
+      _tableDedupMetadataManager = new TableDedupMetadataManager(_tableNameWithType, primaryKeyColumns, _serverMetrics,
+          dedupConfig.getHashFunction());
+    }
+
+    UpsertConfig upsertConfig = tableConfig.getUpsertConfig();
+    if (upsertConfig != null && upsertConfig.getMode() != UpsertConfig.Mode.NONE) {
+      Preconditions.checkState(!dedupEnabled, "Dedup and upsert cannot be both enabled for table: %s",
+          _tableUpsertMetadataManager);
+      Schema schema = ZKMetadataProvider.getTableSchema(_propertyStore, _tableNameWithType);
+      Preconditions.checkState(schema != null, "Failed to find schema for table: %s", _tableNameWithType);
+      _tableUpsertMetadataManager = TableUpsertMetadataManagerFactory.create(tableConfig, schema, this, _serverMetrics);
+    }
   }
 
   @Override
@@ -202,6 +199,13 @@ public class RealtimeTableDataManager extends BaseTableDataManager {
   @Override
   protected void doShutdown() {
     _segmentAsyncExecutorService.shutdown();
+    if (_tableUpsertMetadataManager != null) {
+      try {
+        _tableUpsertMetadataManager.close();
+      } catch (IOException e) {
+        _logger.warn("Cannot close upsert metadata manager properly for table: {}", _tableNameWithType, e);
+      }
+    }
     for (SegmentDataManager segmentDataManager : _segmentDataManagerMap.values()) {
       segmentDataManager.destroy();
     }
@@ -239,12 +243,17 @@ public class RealtimeTableDataManager extends BaseTableDataManager {
     return consumerDir.getAbsolutePath();
   }
 
+  public boolean isDedupEnabled() {
+    return _tableDedupMetadataManager != null;
+  }
+
   public boolean isUpsertEnabled() {
-    return _upsertMode != UpsertConfig.Mode.NONE;
+    return _tableUpsertMetadataManager != null;
   }
 
   public boolean isPartialUpsertEnabled() {
-    return _upsertMode == UpsertConfig.Mode.PARTIAL;
+    return _tableUpsertMetadataManager != null
+        && _tableUpsertMetadataManager.getUpsertMode() == UpsertConfig.Mode.PARTIAL;
   }
 
   /*
@@ -285,64 +294,74 @@ public class RealtimeTableDataManager extends BaseTableDataManager {
     // of the index directory and loading segment from it
     LoaderUtils.reloadFailureRecovery(segmentDir);
 
-    boolean isLLCSegment = SegmentName.isLowLevelConsumerSegmentName(segmentName);
-    if (segmentDir.exists()) {
-      // Segment already exists on disk
-      if (segmentZKMetadata.getStatus() == Status.DONE) {
-        // Metadata has been committed, load the local segment
+    boolean isHLCSegment = SegmentName.isHighLevelConsumerSegmentName(segmentName);
+    if (segmentZKMetadata.getStatus().isCompleted()) {
+      if (segmentDir.exists()) {
+        // Local segment exists, try to load it
         try {
           addSegment(ImmutableSegmentLoader.load(segmentDir, indexLoadingConfig, schema));
           return;
         } catch (Exception e) {
-          if (isLLCSegment) {
-            // For LLC and segments, delete the local copy and download a new copy from the controller
-            FileUtils.deleteQuietly(segmentDir);
+          if (!isHLCSegment) {
+            // For LLC and uploaded segments, delete the local copy and download a new copy
             _logger.error("Caught exception while loading segment: {}, downloading a new copy", segmentName, e);
+            FileUtils.deleteQuietly(segmentDir);
           } else {
             // For HLC segments, throw out the exception because there is no way to recover (controller does not have a
             // copy of the segment)
-            throw e;
+            throw new RuntimeException("Failed to load local HLC segment: " + segmentName, e);
           }
         }
       } else {
-        // Metadata has not been committed, delete the local segment
-        FileUtils.deleteQuietly(segmentDir);
+        if (isHLCSegment) {
+          throw new RuntimeException("Failed to find local copy for committed HLC segment: " + segmentName);
+        }
       }
-    } else if (segmentZKMetadata.getStatus() == Status.UPLOADED) {
-      // The segment is uploaded to an upsert enabled realtime table. Download the segment and load.
-      String downloadUrl = segmentZKMetadata.getDownloadUrl();
-      Preconditions.checkNotNull(downloadUrl, "Upload segment metadata has no download url");
-      downloadSegmentFromDeepStore(segmentName, indexLoadingConfig, downloadUrl);
-      _logger
-          .info("Downloaded, untarred and add segment {} of table {} from {}", segmentName, tableConfig.getTableName(),
-              downloadUrl);
+      // Local segment doesn't exist or cannot load, download a new copy
+      downloadAndReplaceSegment(segmentName, segmentZKMetadata, indexLoadingConfig, tableConfig);
       return;
+    } else {
+      // Metadata has not been committed, delete the local segment if exists
+      FileUtils.deleteQuietly(segmentDir);
     }
 
-    // Start a new consuming segment or download the segment from the controller
-
+    // Start a new consuming segment
     if (!isValid(schema, tableConfig.getIndexingConfig())) {
       _logger.error("Not adding segment {}", segmentName);
       throw new RuntimeException("Mismatching schema/table config for " + _tableNameWithType);
     }
     VirtualColumnProviderFactory.addBuiltInVirtualColumnsToSegmentSchema(schema, segmentName);
 
-    if (isLLCSegment) {
-      if (segmentZKMetadata.getStatus() == Status.DONE) {
-        downloadAndReplaceSegment(segmentName, segmentZKMetadata, indexLoadingConfig, tableConfig);
-        return;
-      }
-
+    if (!isHLCSegment) {
       // Generates only one semaphore for every partitionGroupId
       LLCSegmentName llcSegmentName = new LLCSegmentName(segmentName);
       int partitionGroupId = llcSegmentName.getPartitionGroupId();
       Semaphore semaphore = _partitionGroupIdToSemaphoreMap.computeIfAbsent(partitionGroupId, k -> new Semaphore(1));
       PartitionUpsertMetadataManager partitionUpsertMetadataManager =
-          _tableUpsertMetadataManager != null ? _tableUpsertMetadataManager
-              .getOrCreatePartitionManager(partitionGroupId) : null;
+          _tableUpsertMetadataManager != null ? _tableUpsertMetadataManager.getOrCreatePartitionManager(
+              partitionGroupId) : null;
+      PartitionDedupMetadataManager partitionDedupMetadataManager =
+          _tableDedupMetadataManager != null ? _tableDedupMetadataManager.getOrCreatePartitionManager(partitionGroupId)
+              : null;
+      // For dedup and partial-upsert, wait for all segments loaded before creating the consuming segment
+      if (isDedupEnabled() || isPartialUpsertEnabled()) {
+        if (!_allSegmentsLoaded.get()) {
+          synchronized (_allSegmentsLoaded) {
+            if (!_allSegmentsLoaded.get()) {
+              TableStateUtils.waitForAllSegmentsLoaded(_helixManager, _tableNameWithType);
+              _allSegmentsLoaded.set(true);
+            }
+          }
+        }
+      }
+
+      schema = SegmentGeneratorConfig.updateSchemaWithTimestampIndexes(schema,
+          SegmentGeneratorConfig.extractTimestampIndexConfigsFromTableConfig(tableConfig));
+
       segmentDataManager =
           new LLRealtimeSegmentDataManager(segmentZKMetadata, tableConfig, this, _indexDir.getAbsolutePath(),
-              indexLoadingConfig, schema, llcSegmentName, semaphore, _serverMetrics, partitionUpsertMetadataManager);
+              indexLoadingConfig, schema, llcSegmentName, semaphore, _serverMetrics, partitionUpsertMetadataManager,
+              partitionDedupMetadataManager);
     } else {
       InstanceZKMetadata instanceZKMetadata = ZKMetadataProvider.getInstanceZKMetadata(_propertyStore, _instanceId);
       segmentDataManager = new HLRealtimeSegmentDataManager(segmentZKMetadata, tableConfig, instanceZKMetadata, this,
@@ -357,61 +376,63 @@ public class RealtimeTableDataManager extends BaseTableDataManager {
   @Override
   public void addSegment(ImmutableSegment immutableSegment) {
     if (isUpsertEnabled()) {
-      handleUpsert((ImmutableSegmentImpl) immutableSegment);
+      handleUpsert(immutableSegment);
+      return;
+    }
+
+    // TODO: Change dedup handling to handle segment replacement
+    if (isDedupEnabled()) {
+      buildDedupMeta((ImmutableSegmentImpl) immutableSegment);
     }
     super.addSegment(immutableSegment);
   }
 
-  private void handleUpsert(ImmutableSegmentImpl immutableSegment) {
+  private void buildDedupMeta(ImmutableSegmentImpl immutableSegment) {
+    // TODO(saurabh) refactor commons code with handleUpsert
     String segmentName = immutableSegment.getSegmentName();
-    int partitionGroupId = SegmentUtils
-        .getRealtimeSegmentPartitionId(segmentName, _tableNameWithType, _helixManager, _primaryKeyColumns.get(0));
+    Integer partitionGroupId =
+        SegmentUtils.getRealtimeSegmentPartitionId(segmentName, _tableNameWithType, _helixManager, null);
+    Preconditions.checkNotNull(partitionGroupId,
+        String.format("PartitionGroupId is not available for segment: '%s' (dedup-enabled table: %s)", segmentName,
+            _tableNameWithType));
+    PartitionDedupMetadataManager partitionDedupMetadataManager =
+        _tableDedupMetadataManager.getOrCreatePartitionManager(partitionGroupId);
+    immutableSegment.enableDedup(partitionDedupMetadataManager);
+    partitionDedupMetadataManager.addSegment(immutableSegment);
+  }
+
+  private void handleUpsert(ImmutableSegment immutableSegment) {
+    String segmentName = immutableSegment.getSegmentName();
+    _logger.info("Adding immutable segment: {} to upsert-enabled table: {}", segmentName, _tableNameWithType);
+
+    Integer partitionId =
+        SegmentUtils.getRealtimeSegmentPartitionId(segmentName, _tableNameWithType, _helixManager, null);
+    Preconditions.checkNotNull(partitionId,
+        String.format("Failed to get partition id for segment: %s (upsert-enabled table: %s)", segmentName,
+            _tableNameWithType));
     PartitionUpsertMetadataManager partitionUpsertMetadataManager =
-        _tableUpsertMetadataManager.getOrCreatePartitionManager(partitionGroupId);
-    ThreadSafeMutableRoaringBitmap validDocIds = new ThreadSafeMutableRoaringBitmap();
-    immutableSegment.enableUpsert(partitionUpsertMetadataManager, validDocIds);
+        _tableUpsertMetadataManager.getOrCreatePartitionManager(partitionId);
 
-    Map<String, PinotSegmentColumnReader> columnToReaderMap = new HashMap<>();
-    for (String primaryKeyColumn : _primaryKeyColumns) {
-      columnToReaderMap.put(primaryKeyColumn, new PinotSegmentColumnReader(immutableSegment, primaryKeyColumn));
+    _serverMetrics.addValueToTableGauge(_tableNameWithType, ServerGauge.DOCUMENT_COUNT,
+        immutableSegment.getSegmentMetadata().getTotalDocs());
+    _serverMetrics.addValueToTableGauge(_tableNameWithType, ServerGauge.SEGMENT_COUNT, 1L);
+    ImmutableSegmentDataManager newSegmentManager = new ImmutableSegmentDataManager(immutableSegment);
+    SegmentDataManager oldSegmentManager = _segmentDataManagerMap.put(segmentName, newSegmentManager);
+    if (oldSegmentManager == null) {
+      partitionUpsertMetadataManager.addSegment(immutableSegment);
+      _logger.info("Added new immutable segment: {} to upsert-enabled table: {}", segmentName, _tableNameWithType);
+    } else {
+      IndexSegment oldSegment = oldSegmentManager.getSegment();
+      partitionUpsertMetadataManager.replaceSegment(immutableSegment, oldSegment);
+      _logger.info("Replaced {} segment: {} of upsert-enabled table: {}",
+          oldSegment instanceof ImmutableSegment ? "immutable" : "mutable", segmentName, _tableNameWithType);
+      releaseSegment(oldSegmentManager);
     }
-    columnToReaderMap
-        .put(_upsertComparisonColumn, new PinotSegmentColumnReader(immutableSegment, _upsertComparisonColumn));
-    int numTotalDocs = immutableSegment.getSegmentMetadata().getTotalDocs();
-    int numPrimaryKeyColumns = _primaryKeyColumns.size();
-    Iterator<PartitionUpsertMetadataManager.RecordInfo> recordInfoIterator =
-        new Iterator<PartitionUpsertMetadataManager.RecordInfo>() {
-          private int _docId = 0;
-
-          @Override
-          public boolean hasNext() {
-            return _docId < numTotalDocs;
-          }
-
-          @Override
-          public PartitionUpsertMetadataManager.RecordInfo next() {
-            Object[] values = new Object[numPrimaryKeyColumns];
-            for (int i = 0; i < numPrimaryKeyColumns; i++) {
-              Object value = columnToReaderMap.get(_primaryKeyColumns.get(i)).getValue(_docId);
-              if (value instanceof byte[]) {
-                value = new ByteArray((byte[]) value);
-              }
-              values[i] = value;
-            }
-            PrimaryKey primaryKey = new PrimaryKey(values);
-            Object upsertComparisonValue = columnToReaderMap.get(_upsertComparisonColumn).getValue(_docId);
-            Preconditions.checkState(upsertComparisonValue instanceof Comparable,
-                "Upsert comparison column: %s must be comparable", _upsertComparisonColumn);
-            return new PartitionUpsertMetadataManager.RecordInfo(primaryKey, _docId++,
-                (Comparable) upsertComparisonValue);
-          }
-        };
-    partitionUpsertMetadataManager.addSegment(immutableSegment, recordInfoIterator);
   }
 
   @Override
   protected boolean allowDownload(String segmentName, SegmentZKMetadata zkMetadata) {
-    // Only LLC immutable segment allows download.
+    // Cannot download HLC segment or consuming segment
     if (SegmentName.isHighLevelConsumerSegmentName(segmentName) || zkMetadata.getStatus() == Status.IN_PROGRESS) {
       return false;
     }
@@ -465,9 +486,9 @@ public class RealtimeTableDataManager extends BaseTableDataManager {
    */
   private void untarAndMoveSegment(String segmentName, IndexLoadingConfig indexLoadingConfig, File segmentTarFile)
       throws IOException {
-    // TODO: This could leave temporary directories in _indexDir if JVM shuts down before the temporary directory is
-    //       deleted. Consider cleaning up all temporary directories when starting the server.
-    File tempSegmentDir = new File(_indexDir, "tmp-" + segmentName + "." + System.currentTimeMillis());
+    // This could leave temporary directories in _indexDir if JVM shuts down before the temp directory is deleted.
+    // This is fine since the temporary directories are deleted when the table data manager calls init.
+    File tempSegmentDir = getTmpSegmentDataDir("tmp-" + segmentName + "." + System.currentTimeMillis());
     try {
       File tempIndexDir = TarGzCompressionUtils.untar(segmentTarFile, tempSegmentDir).get(0);
       _logger.info("Uncompressed file {} into tmp dir {}", segmentTarFile, tempSegmentDir);
@@ -484,8 +505,8 @@ public class RealtimeTableDataManager extends BaseTableDataManager {
   private boolean isPeerSegmentDownloadEnabled(TableConfig tableConfig) {
     return
         CommonConstants.HTTP_PROTOCOL.equalsIgnoreCase(tableConfig.getValidationConfig().getPeerSegmentDownloadScheme())
-            || CommonConstants.HTTPS_PROTOCOL
-            .equalsIgnoreCase(tableConfig.getValidationConfig().getPeerSegmentDownloadScheme());
+            || CommonConstants.HTTPS_PROTOCOL.equalsIgnoreCase(
+            tableConfig.getValidationConfig().getPeerSegmentDownloadScheme());
   }
 
   private void downloadSegmentFromPeer(String segmentName, String downloadScheme,
