@@ -20,11 +20,11 @@ package org.apache.pinot.core.operator.blocks;
 
 import com.google.common.annotations.VisibleForTesting;
 import java.io.IOException;
+import java.math.BigDecimal;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Map;
 import javax.annotation.Nullable;
 import org.apache.pinot.common.exception.QueryException;
 import org.apache.pinot.common.response.ProcessingException;
@@ -38,6 +38,7 @@ import org.apache.pinot.core.common.BlockDocIdValueSet;
 import org.apache.pinot.core.common.BlockMetadata;
 import org.apache.pinot.core.common.BlockValSet;
 import org.apache.pinot.core.common.datatable.DataTableBuilder;
+import org.apache.pinot.core.common.datatable.DataTableFactory;
 import org.apache.pinot.core.data.table.IntermediateRecord;
 import org.apache.pinot.core.data.table.Record;
 import org.apache.pinot.core.data.table.Table;
@@ -45,18 +46,21 @@ import org.apache.pinot.core.query.aggregation.function.AggregationFunction;
 import org.apache.pinot.core.query.aggregation.groupby.AggregationGroupByResult;
 import org.apache.pinot.core.query.selection.SelectionOperatorUtils;
 import org.apache.pinot.spi.utils.ByteArray;
+import org.apache.pinot.spi.utils.NullValueUtils;
+import org.roaringbitmap.RoaringBitmap;
 
 
 /**
  * The <code>IntermediateResultsBlock</code> class is the holder of the server side inter-segment results.
  */
+@SuppressWarnings("rawtypes")
 public class IntermediateResultsBlock implements Block {
   private DataSchema _dataSchema;
+  private boolean _nullHandlingEnabled;
   private Collection<Object[]> _selectionResult;
   private AggregationFunction[] _aggregationFunctions;
   private List<Object> _aggregationResult;
   private AggregationGroupByResult _aggregationGroupByResult;
-  private List<Map<String, Object>> _combinedAggregationGroupByResult;
   private List<ProcessingException> _processingExceptions;
   private Collection<IntermediateRecord> _intermediateRecords;
   private long _numDocsScanned;
@@ -65,11 +69,14 @@ public class IntermediateResultsBlock implements Block {
   private long _numTotalDocs;
   private int _numSegmentsProcessed;
   private int _numSegmentsMatched;
+  private int _numConsumingSegmentsProcessed;
+  private int _numConsumingSegmentsMatched;
   private boolean _numGroupsLimitReached;
   private int _numResizes;
   private long _resizeTimeMs;
   private long _executionThreadCpuTimeNs;
   private int _numServerThreads;
+  private boolean _queryHasMVSelectionOrderBy;
 
   private Table _table;
 
@@ -79,9 +86,11 @@ public class IntermediateResultsBlock implements Block {
   /**
    * Constructor for selection result.
    */
-  public IntermediateResultsBlock(DataSchema dataSchema, Collection<Object[]> selectionResult) {
+  public IntermediateResultsBlock(DataSchema dataSchema, Collection<Object[]> selectionResult,
+      boolean nullHandlingEnabled) {
     _dataSchema = dataSchema;
     _selectionResult = selectionResult;
+    _nullHandlingEnabled = nullHandlingEnabled;
   }
 
   /**
@@ -89,34 +98,23 @@ public class IntermediateResultsBlock implements Block {
    * <p>For aggregation only, the result is a list of values.
    * <p>For aggregation group-by, the result is a list of maps from group keys to aggregation values.
    */
-  @SuppressWarnings("unchecked")
-  public IntermediateResultsBlock(AggregationFunction[] aggregationFunctions, List aggregationResult,
-      boolean isGroupBy) {
+  public IntermediateResultsBlock(AggregationFunction[] aggregationFunctions, List<Object> aggregationResult,
+      boolean nullHandlingEnabled) {
     _aggregationFunctions = aggregationFunctions;
-    if (isGroupBy) {
-      _combinedAggregationGroupByResult = aggregationResult;
-    } else {
-      _aggregationResult = aggregationResult;
-    }
-  }
-
-  /**
-   * Constructor for aggregation group-by result with {@link AggregationGroupByResult}.
-   */
-  public IntermediateResultsBlock(AggregationFunction[] aggregationFunctions,
-      @Nullable AggregationGroupByResult aggregationGroupByResults) {
-    _aggregationFunctions = aggregationFunctions;
-    _aggregationGroupByResult = aggregationGroupByResults;
+    _aggregationResult = aggregationResult;
+    _nullHandlingEnabled = nullHandlingEnabled;
   }
 
   /**
    * Constructor for aggregation group-by order-by result with {@link AggregationGroupByResult}.
    */
   public IntermediateResultsBlock(AggregationFunction[] aggregationFunctions,
-      @Nullable AggregationGroupByResult aggregationGroupByResults, DataSchema dataSchema) {
+      @Nullable AggregationGroupByResult aggregationGroupByResults, DataSchema dataSchema,
+      boolean nullHandlingEnabled) {
     _aggregationFunctions = aggregationFunctions;
     _aggregationGroupByResult = aggregationGroupByResults;
     _dataSchema = dataSchema;
+    _nullHandlingEnabled = nullHandlingEnabled;
   }
 
   /**
@@ -124,15 +122,17 @@ public class IntermediateResultsBlock implements Block {
    * with a collection of intermediate records.
    */
   public IntermediateResultsBlock(AggregationFunction[] aggregationFunctions,
-      Collection<IntermediateRecord> intermediateRecords, DataSchema dataSchema) {
+      Collection<IntermediateRecord> intermediateRecords, DataSchema dataSchema, boolean nullHandlingEnabled) {
     _aggregationFunctions = aggregationFunctions;
     _dataSchema = dataSchema;
     _intermediateRecords = intermediateRecords;
+    _nullHandlingEnabled = nullHandlingEnabled;
   }
 
-  public IntermediateResultsBlock(Table table) {
+  public IntermediateResultsBlock(Table table, boolean nullHandlingEnabled) {
     _table = table;
     _dataSchema = table.getDataSchema();
+    _nullHandlingEnabled = nullHandlingEnabled;
   }
 
   /**
@@ -141,6 +141,7 @@ public class IntermediateResultsBlock implements Block {
   public IntermediateResultsBlock(ProcessingException processingException, Exception e) {
     _processingExceptions = new ArrayList<>();
     _processingExceptions.add(QueryException.getException(processingException, e));
+    _nullHandlingEnabled = false;
   }
 
   /**
@@ -276,6 +277,24 @@ public class IntermediateResultsBlock implements Block {
     _numSegmentsMatched = numSegmentsMatched;
   }
 
+  @VisibleForTesting
+  public int getNumConsumingSegmentsProcessed() {
+    return _numConsumingSegmentsProcessed;
+  }
+
+  public void setNumConsumingSegmentsProcessed(int numConsumingSegmentsProcessed) {
+    _numConsumingSegmentsProcessed = numConsumingSegmentsProcessed;
+  }
+
+  @VisibleForTesting
+  public int getNumConsumingSegmentsMatched() {
+    return _numConsumingSegmentsMatched;
+  }
+
+  public void setNumConsumingSegmentsMatched(int numConsumingSegmentsMatched) {
+    _numConsumingSegmentsMatched = numConsumingSegmentsMatched;
+  }
+
   @VisibleForTesting
   public long getNumTotalDocs() {
     return _numTotalDocs;
@@ -294,6 +313,14 @@ public class IntermediateResultsBlock implements Block {
     _numGroupsLimitReached = numGroupsLimitReached;
   }
 
+  public boolean isQueryHasMVSelectionOrderBy() {
+    return _queryHasMVSelectionOrderBy;
+  }
+
+  public void setQueryHasMVSelectionOrderBy(boolean queryHasMVSelectionOrderBy) {
+    _queryHasMVSelectionOrderBy = queryHasMVSelectionOrderBy;
+  }
+
   /**
    * Get the collection of intermediate records
    */
@@ -318,27 +345,54 @@ public class IntermediateResultsBlock implements Block {
       return getAggregationResultDataTable();
     }
 
-    if (_combinedAggregationGroupByResult != null) {
-      return getAggregationGroupByResultDataTable();
-    }
-
     return getMetadataDataTable();
   }
 
   private DataTable getResultDataTable()
       throws IOException {
-    DataTableBuilder dataTableBuilder = new DataTableBuilder(_dataSchema);
+    DataTableBuilder dataTableBuilder = DataTableFactory.getDataTableBuilder(_dataSchema);
     ColumnDataType[] storedColumnDataTypes = _dataSchema.getStoredColumnDataTypes();
+    int numColumns = _dataSchema.size();
     Iterator<Record> iterator = _table.iterator();
-    while (iterator.hasNext()) {
-      Record record = iterator.next();
-      dataTableBuilder.startRow();
-      int columnIndex = 0;
-      for (Object value : record.getValues()) {
-        setDataTableColumn(storedColumnDataTypes[columnIndex], dataTableBuilder, columnIndex, value);
-        columnIndex++;
+    if (_nullHandlingEnabled) {
+      RoaringBitmap[] nullBitmaps = new RoaringBitmap[numColumns];
+      Object[] colDefaultNullValues = new Object[numColumns];
+      for (int colId = 0; colId < numColumns; colId++) {
+        if (storedColumnDataTypes[colId] != ColumnDataType.OBJECT) {
+          colDefaultNullValues[colId] =
+              NullValueUtils.getDefaultNullValue(storedColumnDataTypes[colId].toDataType());
+        }
+        nullBitmaps[colId] = new RoaringBitmap();
+      }
+      int rowId = 0;
+      while (iterator.hasNext()) {
+        Object[] values = iterator.next().getValues();
+        dataTableBuilder.startRow();
+        for (int columnIndex = 0; columnIndex < values.length; columnIndex++) {
+          Object value = values[columnIndex];
+          if (value == null) {
+            value = colDefaultNullValues[columnIndex];
+            nullBitmaps[columnIndex].add(rowId);
+          }
+          setDataTableColumn(storedColumnDataTypes[columnIndex], dataTableBuilder, columnIndex, value);
+        }
+        dataTableBuilder.finishRow();
+        rowId++;
+      }
+      for (int colId = 0; colId < numColumns; colId++) {
+        dataTableBuilder.setNullRowIds(nullBitmaps[colId]);
+      }
+    } else {
+      while (iterator.hasNext()) {
+        Record record = iterator.next();
+        dataTableBuilder.startRow();
+        int columnIndex = 0;
+        for (Object value : record.getValues()) {
+          setDataTableColumn(storedColumnDataTypes[columnIndex], dataTableBuilder, columnIndex, value);
+          columnIndex++;
+        }
+        dataTableBuilder.finishRow();
       }
-      dataTableBuilder.finishRow();
     }
     DataTable dataTable = dataTableBuilder.build();
     return attachMetadataToDataTable(dataTable);
@@ -360,6 +414,9 @@ public class IntermediateResultsBlock implements Block {
       case DOUBLE:
         dataTableBuilder.setColumn(columnIndex, (double) value);
         break;
+      case BIG_DECIMAL:
+        dataTableBuilder.setColumn(columnIndex, (BigDecimal) value);
+        break;
       case STRING:
         dataTableBuilder.setColumn(columnIndex, (String) value);
         break;
@@ -391,7 +448,8 @@ public class IntermediateResultsBlock implements Block {
 
   private DataTable getSelectionResultDataTable()
       throws Exception {
-    return attachMetadataToDataTable(SelectionOperatorUtils.getDataTableFromRows(_selectionResult, _dataSchema));
+    return attachMetadataToDataTable(SelectionOperatorUtils.getDataTableFromRows(
+        _selectionResult, _dataSchema, _nullHandlingEnabled));
   }
 
   private DataTable getAggregationResultDataTable()
@@ -405,46 +463,74 @@ public class IntermediateResultsBlock implements Block {
       columnNames[i] = aggregationFunction.getColumnName();
       columnDataTypes[i] = aggregationFunction.getIntermediateResultColumnType();
     }
-
-    // Build the data table.
-    DataTableBuilder dataTableBuilder = new DataTableBuilder(new DataSchema(columnNames, columnDataTypes));
-    dataTableBuilder.startRow();
-    for (int i = 0; i < numAggregationFunctions; i++) {
-      switch (columnDataTypes[i]) {
-        case LONG:
-          dataTableBuilder.setColumn(i, ((Number) _aggregationResult.get(i)).longValue());
-          break;
-        case DOUBLE:
-          dataTableBuilder.setColumn(i, ((Double) _aggregationResult.get(i)).doubleValue());
-          break;
-        case OBJECT:
-          dataTableBuilder.setColumn(i, _aggregationResult.get(i));
-          break;
-        default:
-          throw new UnsupportedOperationException(
-              "Unsupported aggregation column data type: " + columnDataTypes[i] + " for column: " + columnNames[i]);
+    RoaringBitmap[] nullBitmaps = null;
+    Object[] colDefaultNullValues = null;
+    if (_nullHandlingEnabled) {
+      colDefaultNullValues = new Object[numAggregationFunctions];
+      nullBitmaps = new RoaringBitmap[numAggregationFunctions];
+      for (int i = 0; i < numAggregationFunctions; i++) {
+        if (columnDataTypes[i] != ColumnDataType.OBJECT) {
+          colDefaultNullValues[i] = NullValueUtils.getDefaultNullValue(columnDataTypes[i].toDataType());
+        }
+        nullBitmaps[i] = new RoaringBitmap();
       }
     }
-    dataTableBuilder.finishRow();
-    DataTable dataTable = dataTableBuilder.build();
-
-    return attachMetadataToDataTable(dataTable);
-  }
-
-  private DataTable getAggregationGroupByResultDataTable()
-      throws Exception {
-    String[] columnNames = new String[]{"functionName", "GroupByResultMap"};
-    ColumnDataType[] columnDataTypes = new ColumnDataType[]{ColumnDataType.STRING, ColumnDataType.OBJECT};
 
     // Build the data table.
-    DataTableBuilder dataTableBuilder = new DataTableBuilder(new DataSchema(columnNames, columnDataTypes));
-    int numAggregationFunctions = _aggregationFunctions.length;
-    for (int i = 0; i < numAggregationFunctions; i++) {
-      dataTableBuilder.startRow();
-      AggregationFunction aggregationFunction = _aggregationFunctions[i];
-      dataTableBuilder.setColumn(0, aggregationFunction.getColumnName());
-      dataTableBuilder.setColumn(1, _combinedAggregationGroupByResult.get(i));
-      dataTableBuilder.finishRow();
+    DataTableBuilder dataTableBuilder =
+        DataTableFactory.getDataTableBuilder(new DataSchema(columnNames, columnDataTypes));
+    dataTableBuilder.startRow();
+    if (_nullHandlingEnabled) {
+      for (int i = 0; i < numAggregationFunctions; i++) {
+        Object value = _aggregationResult.get(i);
+        // OBJECT (e.g. DistinctTable) calls toBytes() (e.g. DistinctTable.toBytes()) which takes care of replacing
+        // nulls with default values, and building presence vector and serializing both.
+        if (columnDataTypes[i] != ColumnDataType.OBJECT) {
+          if (value == null) {
+            value = colDefaultNullValues[i];
+            nullBitmaps[i].add(0);
+          }
+        }
+
+        switch (columnDataTypes[i]) {
+          case LONG:
+            dataTableBuilder.setColumn(i, ((Number) value).longValue());
+            break;
+          case DOUBLE:
+            dataTableBuilder.setColumn(i, ((Double) value).doubleValue());
+            break;
+          case OBJECT:
+            dataTableBuilder.setColumn(i, value);
+            break;
+          default:
+            throw new UnsupportedOperationException(
+                "Unsupported aggregation column data type: " + columnDataTypes[i] + " for column: " + columnNames[i]);
+        }
+      }
+    } else {
+      for (int i = 0; i < numAggregationFunctions; i++) {
+        switch (columnDataTypes[i]) {
+          case LONG:
+            dataTableBuilder.setColumn(i, ((Number) _aggregationResult.get(i)).longValue());
+            break;
+          case DOUBLE:
+            dataTableBuilder.setColumn(i, ((Double) _aggregationResult.get(i)).doubleValue());
+            break;
+          case OBJECT:
+            dataTableBuilder.setColumn(i, _aggregationResult.get(i));
+            break;
+          default:
+            throw new UnsupportedOperationException(
+                "Unsupported aggregation column data type: " + columnDataTypes[i] + " for column: " + columnNames[i]);
+        }
+      }
+    }
+
+    dataTableBuilder.finishRow();
+    if (_nullHandlingEnabled) {
+      for (int i = 0; i < numAggregationFunctions; i++) {
+        dataTableBuilder.setNullRowIds(nullBitmaps[i]);
+      }
     }
     DataTable dataTable = dataTableBuilder.build();
 
@@ -452,7 +538,7 @@ public class IntermediateResultsBlock implements Block {
   }
 
   private DataTable getMetadataDataTable() {
-    return attachMetadataToDataTable(DataTableBuilder.getEmptyDataTable());
+    return attachMetadataToDataTable(DataTableFactory.getEmptyDataTable());
   }
 
   private DataTable attachMetadataToDataTable(DataTable dataTable) {
@@ -463,6 +549,10 @@ public class IntermediateResultsBlock implements Block {
         .put(MetadataKey.NUM_ENTRIES_SCANNED_POST_FILTER.getName(), String.valueOf(_numEntriesScannedPostFilter));
     dataTable.getMetadata().put(MetadataKey.NUM_SEGMENTS_PROCESSED.getName(), String.valueOf(_numSegmentsProcessed));
     dataTable.getMetadata().put(MetadataKey.NUM_SEGMENTS_MATCHED.getName(), String.valueOf(_numSegmentsMatched));
+    dataTable.getMetadata().put(MetadataKey.NUM_CONSUMING_SEGMENTS_PROCESSED.getName(),
+        String.valueOf(_numConsumingSegmentsProcessed));
+    dataTable.getMetadata().put(MetadataKey.NUM_CONSUMING_SEGMENTS_MATCHED.getName(),
+        String.valueOf(_numConsumingSegmentsMatched));
     dataTable.getMetadata().put(MetadataKey.NUM_RESIZES.getName(), String.valueOf(_numResizes));
     dataTable.getMetadata().put(MetadataKey.RESIZE_TIME_MS.getName(), String.valueOf(_resizeTimeMs));
 
