@@ -18,6 +18,9 @@
  */
 package org.apache.pinot.core.operator.query;
 
+import java.util.Collections;
+import java.util.List;
+import org.apache.pinot.core.common.Operator;
 import org.apache.pinot.core.operator.BaseOperator;
 import org.apache.pinot.core.operator.ExecutionStatistics;
 import org.apache.pinot.core.operator.blocks.IntermediateResultsBlock;
@@ -34,21 +37,23 @@ import org.apache.pinot.core.startree.executor.StarTreeAggregationExecutor;
  */
 @SuppressWarnings("rawtypes")
 public class AggregationOperator extends BaseOperator<IntermediateResultsBlock> {
-  private static final String OPERATOR_NAME = "AggregationOperator";
+  private static final String EXPLAIN_NAME = "AGGREGATE";
 
   private final AggregationFunction[] _aggregationFunctions;
   private final TransformOperator _transformOperator;
   private final long _numTotalDocs;
   private final boolean _useStarTree;
+  private final boolean _nullHandlingEnabled;
 
   private int _numDocsScanned = 0;
 
   public AggregationOperator(AggregationFunction[] aggregationFunctions, TransformOperator transformOperator,
-      long numTotalDocs, boolean useStarTree) {
+      long numTotalDocs, boolean useStarTree, boolean nullHandlingEnabled) {
     _aggregationFunctions = aggregationFunctions;
     _transformOperator = transformOperator;
     _numTotalDocs = numTotalDocs;
     _useStarTree = useStarTree;
+    _nullHandlingEnabled = nullHandlingEnabled;
   }
 
   @Override
@@ -67,12 +72,12 @@ public class AggregationOperator extends BaseOperator<IntermediateResultsBlock>
     }
 
     // Build intermediate result block based on aggregation result from the executor
-    return new IntermediateResultsBlock(_aggregationFunctions, aggregationExecutor.getResult(), false);
+    return new IntermediateResultsBlock(_aggregationFunctions, aggregationExecutor.getResult(), _nullHandlingEnabled);
   }
 
   @Override
-  public String getOperatorName() {
-    return OPERATOR_NAME;
+  public List<Operator> getChildOperators() {
+    return Collections.singletonList(_transformOperator);
   }
 
   @Override
@@ -82,4 +87,17 @@ public class AggregationOperator extends BaseOperator<IntermediateResultsBlock>
     return new ExecutionStatistics(_numDocsScanned, numEntriesScannedInFilter, numEntriesScannedPostFilter,
         _numTotalDocs);
   }
+
+  @Override
+  public String toExplainString() {
+    StringBuilder stringBuilder = new StringBuilder(EXPLAIN_NAME).append("(aggregations:");
+    if (_aggregationFunctions.length > 0) {
+      stringBuilder.append(_aggregationFunctions[0].toExplainString());
+      for (int i = 1; i < _aggregationFunctions.length; i++) {
+        stringBuilder.append(", ").append(_aggregationFunctions[i].toExplainString());
+      }
+    }
+
+    return stringBuilder.append(')').toString();
+  }
 }
