@@ -18,7 +18,9 @@
  */
 package org.apache.pinot.core.operator.query;
 
+import java.math.BigDecimal;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -30,6 +32,7 @@ import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.request.context.OrderByExpressionContext;
 import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.core.common.BlockValSet;
+import org.apache.pinot.core.common.Operator;
 import org.apache.pinot.core.common.RowBasedBlockValueFetcher;
 import org.apache.pinot.core.operator.BaseOperator;
 import org.apache.pinot.core.operator.BitmapDocIdSetOperator;
@@ -45,7 +48,6 @@ import org.apache.pinot.segment.spi.IndexSegment;
 import org.apache.pinot.segment.spi.datasource.DataSource;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.spi.utils.ByteArray;
-import org.apache.pinot.spi.utils.CommonConstants.Segment.BuiltInVirtualColumn;
 import org.roaringbitmap.RoaringBitmap;
 
 
@@ -68,9 +70,11 @@ import org.roaringbitmap.RoaringBitmap;
  * </ul>
  */
 public class SelectionOrderByOperator extends BaseOperator<IntermediateResultsBlock> {
-  private static final String OPERATOR_NAME = "SelectionOrderByOperator";
+
+  private static final String EXPLAIN_NAME = "SELECT_ORDERBY";
 
   private final IndexSegment _indexSegment;
+  private final boolean _nullHandlingEnabled;
   // Deduped order-by expressions followed by output expressions from SelectionOperatorUtils.extractExpressions()
   private final List<ExpressionContext> _expressions;
   private final TransformOperator _transformOperator;
@@ -78,15 +82,20 @@ public class SelectionOrderByOperator extends BaseOperator<IntermediateResultsBl
   private final TransformResultMetadata[] _orderByExpressionMetadata;
   private final int _numRowsToKeep;
   private final PriorityQueue<Object[]> _rows;
+  private final boolean _allOrderByColsPreSorted;
 
   private int _numDocsScanned = 0;
   private long _numEntriesScannedPostFilter = 0;
 
+  private boolean _queryHasMVSelectionOrderBy = false;
+
   public SelectionOrderByOperator(IndexSegment indexSegment, QueryContext queryContext,
-      List<ExpressionContext> expressions, TransformOperator transformOperator) {
+      List<ExpressionContext> expressions, TransformOperator transformOperator, boolean allOrderByColsPreSorted) {
     _indexSegment = indexSegment;
+    _nullHandlingEnabled = queryContext.isNullHandlingEnabled();
     _expressions = expressions;
     _transformOperator = transformOperator;
+    _allOrderByColsPreSorted = allOrderByColsPreSorted;
 
     _orderByExpressions = queryContext.getOrderByExpressions();
     assert _orderByExpressions != null;
@@ -102,6 +111,18 @@ public class SelectionOrderByOperator extends BaseOperator<IntermediateResultsBl
         getComparator());
   }
 
+  @Override
+  public String toExplainString() {
+    StringBuilder stringBuilder = new StringBuilder(EXPLAIN_NAME).append("(selectList:");
+    if (!_expressions.isEmpty()) {
+      stringBuilder.append(_expressions.get(0));
+      for (int i = 1; i < _expressions.size(); i++) {
+        stringBuilder.append(", ").append(_expressions.get(i));
+      }
+    }
+    return stringBuilder.append(')').toString();
+  }
+
   private Comparator<Object[]> getComparator() {
     // Compare all single-value columns
     int numOrderByExpressions = _orderByExpressions.size();
@@ -109,6 +130,8 @@ public class SelectionOrderByOperator extends BaseOperator<IntermediateResultsBl
     for (int i = 0; i < numOrderByExpressions; i++) {
       if (_orderByExpressionMetadata[i].isSingleValue()) {
         valueIndexList.add(i);
+      } else {
+        _queryHasMVSelectionOrderBy = true;
       }
     }
 
@@ -124,52 +147,160 @@ public class SelectionOrderByOperator extends BaseOperator<IntermediateResultsBl
       multipliers[i] = _orderByExpressions.get(valueIndex).isAsc() ? -1 : 1;
     }
 
-    return (o1, o2) -> {
-      for (int i = 0; i < numValuesToCompare; i++) {
-        int index = valueIndices[i];
+    if (_nullHandlingEnabled) {
+      return (Object[] o1, Object[] o2) -> {
+        for (int i = 0; i < numValuesToCompare; i++) {
+          int index = valueIndices[i];
 
-        // TODO: Evaluate the performance of casting to Comparable and avoid the switch
-        Object v1 = o1[index];
-        Object v2 = o2[index];
-        int result;
-        switch (storedTypes[i]) {
-          case INT:
-            result = ((Integer) v1).compareTo((Integer) v2);
-            break;
-          case LONG:
-            result = ((Long) v1).compareTo((Long) v2);
-            break;
-          case FLOAT:
-            result = ((Float) v1).compareTo((Float) v2);
-            break;
-          case DOUBLE:
-            result = ((Double) v1).compareTo((Double) v2);
-            break;
-          case STRING:
-            result = ((String) v1).compareTo((String) v2);
-            break;
-          case BYTES:
-            result = ((ByteArray) v1).compareTo((ByteArray) v2);
-            break;
-          // NOTE: Multi-value columns are not comparable, so we should not reach here
-          default:
-            throw new IllegalStateException();
+          // TODO: Evaluate the performance of casting to Comparable and avoid the switch
+          Object v1 = o1[index];
+          Object v2 = o2[index];
+          if (v1 == null) {
+            // The default null ordering is: 'NULLS LAST', regardless of the ordering direction.
+            return v2 == null ? 0 : -multipliers[i];
+          } else if (v2 == null) {
+            return multipliers[i];
+          }
+          int result;
+          switch (storedTypes[i]) {
+            case INT:
+              result = ((Integer) v1).compareTo((Integer) v2);
+              break;
+            case LONG:
+              result = ((Long) v1).compareTo((Long) v2);
+              break;
+            case FLOAT:
+              result = ((Float) v1).compareTo((Float) v2);
+              break;
+            case DOUBLE:
+              result = ((Double) v1).compareTo((Double) v2);
+              break;
+            case BIG_DECIMAL:
+              result = ((BigDecimal) v1).compareTo((BigDecimal) v2);
+              break;
+            case STRING:
+              result = ((String) v1).compareTo((String) v2);
+              break;
+            case BYTES:
+              result = ((ByteArray) v1).compareTo((ByteArray) v2);
+              break;
+            // NOTE: Multi-value columns are not comparable, so we should not reach here
+            default:
+              throw new IllegalStateException();
+          }
+          if (result != 0) {
+            return result * multipliers[i];
+          }
         }
-        if (result != 0) {
-          return result * multipliers[i];
+        return 0;
+      };
+    } else {
+      return (Object[] o1, Object[] o2) -> {
+        for (int i = 0; i < numValuesToCompare; i++) {
+          int index = valueIndices[i];
+
+          // TODO: Evaluate the performance of casting to Comparable and avoid the switch
+          Object v1 = o1[index];
+          Object v2 = o2[index];
+          int result;
+          switch (storedTypes[i]) {
+            case INT:
+              result = ((Integer) v1).compareTo((Integer) v2);
+              break;
+            case LONG:
+              result = ((Long) v1).compareTo((Long) v2);
+              break;
+            case FLOAT:
+              result = ((Float) v1).compareTo((Float) v2);
+              break;
+            case DOUBLE:
+              result = ((Double) v1).compareTo((Double) v2);
+              break;
+            case BIG_DECIMAL:
+              result = ((BigDecimal) v1).compareTo((BigDecimal) v2);
+              break;
+            case STRING:
+              result = ((String) v1).compareTo((String) v2);
+              break;
+            case BYTES:
+              result = ((ByteArray) v1).compareTo((ByteArray) v2);
+              break;
+            // NOTE: Multi-value columns are not comparable, so we should not reach here
+            default:
+              throw new IllegalStateException();
+          }
+          if (result != 0) {
+            return result * multipliers[i];
+          }
         }
-      }
-      return 0;
-    };
+        return 0;
+      };
+    }
   }
 
   @Override
   protected IntermediateResultsBlock getNextBlock() {
-    if (_expressions.size() == _orderByExpressions.size()) {
-      return computeAllOrdered();
+    IntermediateResultsBlock resultsBlock;
+    if (_allOrderByColsPreSorted) {
+      resultsBlock = computeAllPreSorted();
+    } else if (_expressions.size() == _orderByExpressions.size()) {
+      resultsBlock = computeAllOrdered();
     } else {
-      return computePartiallyOrdered();
+      resultsBlock = computePartiallyOrdered();
     }
+    resultsBlock.setQueryHasMVSelectionOrderBy(_queryHasMVSelectionOrderBy);
+    return resultsBlock;
+  }
+
+  private IntermediateResultsBlock computeAllPreSorted() {
+    int numExpressions = _expressions.size();
+
+    // Fetch all the expressions and insert them into the priority queue
+    BlockValSet[] blockValSets = new BlockValSet[numExpressions];
+    int numColumnsProjected = _transformOperator.getNumColumnsProjected();
+    TransformBlock transformBlock;
+    while (_numDocsScanned < _numRowsToKeep && (transformBlock = _transformOperator.nextBlock()) != null) {
+      for (int i = 0; i < numExpressions; i++) {
+        ExpressionContext expression = _expressions.get(i);
+        blockValSets[i] = transformBlock.getBlockValueSet(expression);
+      }
+      RowBasedBlockValueFetcher blockValueFetcher = new RowBasedBlockValueFetcher(blockValSets);
+      int numDocsFetched = transformBlock.getNumDocs();
+      if (_nullHandlingEnabled) {
+        RoaringBitmap[] nullBitmaps = new RoaringBitmap[numExpressions];
+        for (int i = 0; i < numExpressions; i++) {
+          nullBitmaps[i] = blockValSets[i].getNullBitmap();
+        }
+        for (int rowId = 0; rowId < numDocsFetched && (_numDocsScanned < _numRowsToKeep); rowId++) {
+          Object[] row = blockValueFetcher.getRow(rowId);
+          for (int colId = 0; colId < numExpressions; colId++) {
+            if (nullBitmaps[colId] != null && nullBitmaps[colId].contains(rowId)) {
+              row[colId] = null;
+            }
+          }
+        }
+      }
+      for (int i = 0; i < numDocsFetched && (_numDocsScanned < _numRowsToKeep); i++) {
+        SelectionOperatorUtils.addToPriorityQueue(blockValueFetcher.getRow(i), _rows, _numRowsToKeep);
+        _numDocsScanned++;
+      }
+    }
+    _numEntriesScannedPostFilter = (long) _numDocsScanned * numColumnsProjected;
+
+    // Create the data schema
+    String[] columnNames = new String[numExpressions];
+    DataSchema.ColumnDataType[] columnDataTypes = new DataSchema.ColumnDataType[numExpressions];
+    for (int i = 0; i < numExpressions; i++) {
+      ExpressionContext expression = _expressions.get(i);
+      columnNames[i] = expression.toString();
+      TransformResultMetadata expressionMetadata = _transformOperator.getResultMetadata(expression);
+      columnDataTypes[i] =
+          DataSchema.ColumnDataType.fromDataType(expressionMetadata.getDataType(), expressionMetadata.isSingleValue());
+    }
+
+    DataSchema dataSchema = new DataSchema(columnNames, columnDataTypes);
+
+    return new IntermediateResultsBlock(dataSchema, _rows, _nullHandlingEnabled);
   }
 
   /**
@@ -189,12 +320,29 @@ public class SelectionOrderByOperator extends BaseOperator<IntermediateResultsBl
       }
       RowBasedBlockValueFetcher blockValueFetcher = new RowBasedBlockValueFetcher(blockValSets);
       int numDocsFetched = transformBlock.getNumDocs();
-      for (int i = 0; i < numDocsFetched; i++) {
-        SelectionOperatorUtils.addToPriorityQueue(blockValueFetcher.getRow(i), _rows, _numRowsToKeep);
+      if (_nullHandlingEnabled) {
+        RoaringBitmap[] nullBitmaps = new RoaringBitmap[numExpressions];
+        for (int i = 0; i < numExpressions; i++) {
+          nullBitmaps[i] = blockValSets[i].getNullBitmap();
+        }
+        for (int rowId = 0; rowId < numDocsFetched; rowId++) {
+          // Note: Everytime blockValueFetcher.getRow is called, a new row instance is created.
+          Object[] row = blockValueFetcher.getRow(rowId);
+          for (int colId = 0; colId < numExpressions; colId++) {
+            if (nullBitmaps[colId] != null && nullBitmaps[colId].contains(rowId)) {
+              row[colId] = null;
+            }
+          }
+          SelectionOperatorUtils.addToPriorityQueue(row, _rows, _numRowsToKeep);
+        }
+      } else {
+        for (int i = 0; i < numDocsFetched; i++) {
+          SelectionOperatorUtils.addToPriorityQueue(blockValueFetcher.getRow(i), _rows, _numRowsToKeep);
+        }
       }
       _numDocsScanned += numDocsFetched;
-      _numEntriesScannedPostFilter += numDocsFetched * numColumnsProjected;
     }
+    _numEntriesScannedPostFilter = (long) _numDocsScanned * numColumnsProjected;
 
     // Create the data schema
     String[] columnNames = new String[numExpressions];
@@ -207,7 +355,7 @@ public class SelectionOrderByOperator extends BaseOperator<IntermediateResultsBl
     }
     DataSchema dataSchema = new DataSchema(columnNames, columnDataTypes);
 
-    return new IntermediateResultsBlock(dataSchema, _rows);
+    return new IntermediateResultsBlock(dataSchema, _rows, _nullHandlingEnabled);
   }
 
   /**
@@ -218,7 +366,7 @@ public class SelectionOrderByOperator extends BaseOperator<IntermediateResultsBl
     int numOrderByExpressions = _orderByExpressions.size();
 
     // Fetch the order-by expressions and docIds and insert them into the priority queue
-    BlockValSet[] blockValSets = new BlockValSet[numOrderByExpressions + 1];
+    BlockValSet[] blockValSets = new BlockValSet[numOrderByExpressions];
     int numColumnsProjected = _transformOperator.getNumColumnsProjected();
     TransformBlock transformBlock;
     while ((transformBlock = _transformOperator.nextBlock()) != null) {
@@ -226,20 +374,40 @@ public class SelectionOrderByOperator extends BaseOperator<IntermediateResultsBl
         ExpressionContext expression = _orderByExpressions.get(i).getExpression();
         blockValSets[i] = transformBlock.getBlockValueSet(expression);
       }
-      blockValSets[numOrderByExpressions] = transformBlock.getBlockValueSet(BuiltInVirtualColumn.DOCID);
       RowBasedBlockValueFetcher blockValueFetcher = new RowBasedBlockValueFetcher(blockValSets);
       int numDocsFetched = transformBlock.getNumDocs();
-      for (int i = 0; i < numDocsFetched; i++) {
-        // NOTE: We pre-allocate the complete row so that we can fill up the non-order-by output expression values later
-        //       without creating extra rows or re-constructing the priority queue. We can change the values in-place
-        //       because the comparator only compare the values for the order-by expressions.
-        Object[] row = new Object[numExpressions];
-        blockValueFetcher.getRow(i, row, 0);
-        SelectionOperatorUtils.addToPriorityQueue(row, _rows, _numRowsToKeep);
+      int[] docIds = transformBlock.getDocIds();
+      if (_nullHandlingEnabled) {
+        RoaringBitmap[] nullBitmaps = new RoaringBitmap[numOrderByExpressions];
+        for (int i = 0; i < numOrderByExpressions; i++) {
+          nullBitmaps[i] = blockValSets[i].getNullBitmap();
+        }
+        for (int rowId = 0; rowId < numDocsFetched; rowId++) {
+          Object[] row = new Object[numExpressions];
+          blockValueFetcher.getRow(rowId, row, 0);
+          row[numOrderByExpressions] = docIds[rowId];
+          for (int colId = 0; colId < numOrderByExpressions; colId++) {
+            if (nullBitmaps[colId] != null && nullBitmaps[colId].contains(rowId)) {
+              row[colId] = null;
+            }
+          }
+          SelectionOperatorUtils.addToPriorityQueue(row, _rows, _numRowsToKeep);
+        }
+      } else {
+        for (int i = 0; i < numDocsFetched; i++) {
+          // NOTE: We pre-allocate the complete row so that we can fill up the non-order-by output expression values
+          // later
+          //       without creating extra rows or re-constructing the priority queue. We can change the values in-place
+          //       because the comparator only compare the values for the order-by expressions.
+          Object[] row = new Object[numExpressions];
+          blockValueFetcher.getRow(i, row, 0);
+          row[numOrderByExpressions] = docIds[i];
+          SelectionOperatorUtils.addToPriorityQueue(row, _rows, _numRowsToKeep);
+        }
       }
       _numDocsScanned += numDocsFetched;
-      _numEntriesScannedPostFilter += numDocsFetched * numColumnsProjected;
     }
+    _numEntriesScannedPostFilter = (long) _numDocsScanned * numColumnsProjected;
 
     // Copy the rows (shallow copy so that any modification will also be reflected to the priority queue) into a list,
     // and store the document ids into a bitmap
@@ -284,7 +452,7 @@ public class SelectionOrderByOperator extends BaseOperator<IntermediateResultsBl
       for (int i = 0; i < numDocsFetched; i++) {
         blockValueFetcher.getRow(i, rowList.get(rowBaseId + i), numOrderByExpressions);
       }
-      _numEntriesScannedPostFilter += numDocsFetched * numColumns;
+      _numEntriesScannedPostFilter += (long) numDocsFetched * numColumns;
       rowBaseId += numDocsFetched;
     }
 
@@ -306,15 +474,15 @@ public class SelectionOrderByOperator extends BaseOperator<IntermediateResultsBl
     }
     DataSchema dataSchema = new DataSchema(columnNames, columnDataTypes);
 
-    return new IntermediateResultsBlock(dataSchema, _rows);
+    return new IntermediateResultsBlock(dataSchema, _rows, _nullHandlingEnabled);
   }
 
-  @Override
-  public String getOperatorName() {
-    return OPERATOR_NAME;
-  }
 
   @Override
+  public List<Operator> getChildOperators() {
+    return Collections.singletonList(_transformOperator);
+  }
+
   public IndexSegment getIndexSegment() {
     return _indexSegment;
   }
