@@ -130,6 +130,9 @@ public abstract class BinaryOperatorTransformFunction extends BaseTransformFunct
       case DOUBLE:
         fillResultDouble(projectionBlock, length);
         break;
+      case BIG_DECIMAL:
+        fillResultBigDecimal(projectionBlock, length);
+        break;
       case STRING:
         fillResultString(projectionBlock, length);
         break;
@@ -157,6 +160,9 @@ public abstract class BinaryOperatorTransformFunction extends BaseTransformFunct
       case DOUBLE:
         fillDoubleResultArray(projectionBlock, leftIntValues, length);
         break;
+      case BIG_DECIMAL:
+        fillBigDecimalResultArray(projectionBlock, leftIntValues, length);
+        break;
       case STRING:
         fillStringResultArray(projectionBlock, leftIntValues, length);
         break;
@@ -180,6 +186,9 @@ public abstract class BinaryOperatorTransformFunction extends BaseTransformFunct
       case DOUBLE:
         fillDoubleResultArray(projectionBlock, leftLongValues, length);
         break;
+      case BIG_DECIMAL:
+        fillBigDecimalResultArray(projectionBlock, leftLongValues, length);
+        break;
       case STRING:
         fillStringResultArray(projectionBlock, leftLongValues, length);
         break;
@@ -203,6 +212,9 @@ public abstract class BinaryOperatorTransformFunction extends BaseTransformFunct
       case DOUBLE:
         fillDoubleResultArray(projectionBlock, leftFloatValues, length);
         break;
+      case BIG_DECIMAL:
+        fillBigDecimalResultArray(projectionBlock, leftFloatValues, length);
+        break;
       case STRING:
         fillStringResultArray(projectionBlock, leftFloatValues, length);
         break;
@@ -226,6 +238,9 @@ public abstract class BinaryOperatorTransformFunction extends BaseTransformFunct
       case DOUBLE:
         fillDoubleResultArray(projectionBlock, leftDoubleValues, length);
         break;
+      case BIG_DECIMAL:
+        fillBigDecimalResultArray(projectionBlock, leftDoubleValues, length);
+        break;
       case STRING:
         fillStringResultArray(projectionBlock, leftDoubleValues, length);
         break;
@@ -234,6 +249,32 @@ public abstract class BinaryOperatorTransformFunction extends BaseTransformFunct
     }
   }
 
+  private void fillResultBigDecimal(ProjectionBlock projectionBlock, int length) {
+    BigDecimal[] leftBigDecimalValues = _leftTransformFunction.transformToBigDecimalValuesSV(projectionBlock);
+    switch (_rightStoredType) {
+      case INT:
+        fillIntResultArray(projectionBlock, leftBigDecimalValues, length);
+        break;
+      case LONG:
+        fillLongResultArray(projectionBlock, leftBigDecimalValues, length);
+        break;
+      case FLOAT:
+        fillFloatResultArray(projectionBlock, leftBigDecimalValues, length);
+        break;
+      case DOUBLE:
+        fillDoubleResultArray(projectionBlock, leftBigDecimalValues, length);
+        break;
+      case STRING:
+        fillStringResultArray(projectionBlock, leftBigDecimalValues, length);
+        break;
+      case BIG_DECIMAL:
+        fillBigDecimalResultArray(projectionBlock, leftBigDecimalValues, length);
+        break;
+      default:
+        throw illegalState();
+    }
+  }
+
   private IllegalStateException illegalState() {
     throw new IllegalStateException(String.format(
         "Unsupported data type for comparison: [Left Transform Function [%s] result type is [%s], Right "
@@ -285,6 +326,13 @@ public abstract class BinaryOperatorTransformFunction extends BaseTransformFunct
     }
   }
 
+  private void fillBigDecimalResultArray(ProjectionBlock projectionBlock, int[] leftValues, int length) {
+    BigDecimal[] rightBigDecimalValues = _rightTransformFunction.transformToBigDecimalValuesSV(projectionBlock);
+    for (int i = 0; i < length; i++) {
+      _results[i] = getIntResult(BigDecimal.valueOf(leftValues[i]).compareTo(rightBigDecimalValues[i]));
+    }
+  }
+
   private void fillStringResultArray(ProjectionBlock projectionBlock, int[] leftValues, int length) {
     String[] rightStringValues = _rightTransformFunction.transformToStringValuesSV(projectionBlock);
     for (int i = 0; i < length; i++) {
@@ -324,6 +372,13 @@ public abstract class BinaryOperatorTransformFunction extends BaseTransformFunct
     }
   }
 
+  private void fillBigDecimalResultArray(ProjectionBlock projectionBlock, long[] leftValues, int length) {
+    BigDecimal[] rightBigDecimalValues = _rightTransformFunction.transformToBigDecimalValuesSV(projectionBlock);
+    for (int i = 0; i < length; i++) {
+      _results[i] = getIntResult(BigDecimal.valueOf(leftValues[i]).compareTo(rightBigDecimalValues[i]));
+    }
+  }
+
   private void fillStringResultArray(ProjectionBlock projectionBlock, long[] leftValues, int length) {
     String[] rightStringValues = _rightTransformFunction.transformToStringValuesSV(projectionBlock);
     for (int i = 0; i < length; i++) {
@@ -363,6 +418,13 @@ public abstract class BinaryOperatorTransformFunction extends BaseTransformFunct
     }
   }
 
+  private void fillBigDecimalResultArray(ProjectionBlock projectionBlock, float[] leftValues, int length) {
+    BigDecimal[] rightBigDecimalValues = _rightTransformFunction.transformToBigDecimalValuesSV(projectionBlock);
+    for (int i = 0; i < length; i++) {
+      _results[i] = getIntResult(BigDecimal.valueOf(leftValues[i]).compareTo(rightBigDecimalValues[i]));
+    }
+  }
+
   private void fillStringResultArray(ProjectionBlock projectionBlock, float[] leftValues, int length) {
     String[] rightStringValues = _rightTransformFunction.transformToStringValuesSV(projectionBlock);
     for (int i = 0; i < length; i++) {
@@ -402,6 +464,13 @@ public abstract class BinaryOperatorTransformFunction extends BaseTransformFunct
     }
   }
 
+  private void fillBigDecimalResultArray(ProjectionBlock projectionBlock, double[] leftValues, int length) {
+    BigDecimal[] rightBigDecimalValues = _rightTransformFunction.transformToBigDecimalValuesSV(projectionBlock);
+    for (int i = 0; i < length; i++) {
+      _results[i] = getIntResult(BigDecimal.valueOf(leftValues[i]).compareTo(rightBigDecimalValues[i]));
+    }
+  }
+
   private void fillStringResultArray(ProjectionBlock projectionBlock, double[] leftValues, int length) {
     String[] rightStringValues = _rightTransformFunction.transformToStringValuesSV(projectionBlock);
     for (int i = 0; i < length; i++) {
@@ -413,6 +482,48 @@ public abstract class BinaryOperatorTransformFunction extends BaseTransformFunct
     }
   }
 
+  private void fillIntResultArray(ProjectionBlock projectionBlock, BigDecimal[] leftValues, int length) {
+    int[] rightIntValues = _rightTransformFunction.transformToIntValuesSV(projectionBlock);
+    for (int i = 0; i < length; i++) {
+      _results[i] = getIntResult(leftValues[i].compareTo(BigDecimal.valueOf(rightIntValues[i])));
+    }
+  }
+
+  private void fillLongResultArray(ProjectionBlock projectionBlock, BigDecimal[] leftValues, int length) {
+    long[] rightLongValues = _rightTransformFunction.transformToLongValuesSV(projectionBlock);
+    for (int i = 0; i < length; i++) {
+      _results[i] = getIntResult(leftValues[i].compareTo(BigDecimal.valueOf(rightLongValues[i])));
+    }
+  }
+
+  private void fillFloatResultArray(ProjectionBlock projectionBlock, BigDecimal[] leftValues, int length) {
+    float[] rightFloatValues = _rightTransformFunction.transformToFloatValuesSV(projectionBlock);
+    for (int i = 0; i < length; i++) {
+      _results[i] = getIntResult(leftValues[i].compareTo(BigDecimal.valueOf(rightFloatValues[i])));
+    }
+  }
+
+  private void fillDoubleResultArray(ProjectionBlock projectionBlock, BigDecimal[] leftValues, int length) {
+    double[] rightDoubleValues = _rightTransformFunction.transformToDoubleValuesSV(projectionBlock);
+    for (int i = 0; i < length; i++) {
+      _results[i] = getIntResult(leftValues[i].compareTo(BigDecimal.valueOf(rightDoubleValues[i])));
+    }
+  }
+
+  private void fillBigDecimalResultArray(ProjectionBlock projectionBlock, BigDecimal[] leftValues, int length) {
+    BigDecimal[] rightBigDecimalValues = _rightTransformFunction.transformToBigDecimalValuesSV(projectionBlock);
+    for (int i = 0; i < length; i++) {
+      _results[i] = getIntResult(leftValues[i].compareTo(rightBigDecimalValues[i]));
+    }
+  }
+
+  private void fillStringResultArray(ProjectionBlock projectionBlock, BigDecimal[] leftValues, int length) {
+    String[] rightStringValues = _rightTransformFunction.transformToStringValuesSV(projectionBlock);
+    for (int i = 0; i < length; i++) {
+      _results[i] = getIntResult(leftValues[i].compareTo(new BigDecimal(rightStringValues[i])));
+    }
+  }
+
   private int compare(long left, double right) {
     if (Math.abs(left) <= 1L << 53) {
       return getIntResult(Double.compare(left, right));
