@@ -31,7 +31,6 @@ import org.apache.pinot.core.operator.transform.TransformOperator;
 import org.apache.pinot.core.query.request.context.QueryContext;
 import org.apache.pinot.core.query.selection.SelectionOperatorUtils;
 import org.apache.pinot.segment.spi.IndexSegment;
-import org.apache.pinot.spi.utils.CommonConstants.Segment.BuiltInVirtualColumn;
 
 
 /**
@@ -60,23 +59,27 @@ public class SelectionPlanNode implements PlanNode {
         return new SelectionOnlyOperator(_indexSegment, _queryContext, expressions, transformOperator);
       } else {
         // Selection order-by
-        if (orderByExpressions.size() == expressions.size()) {
+        if (isAllOrderByColumnsSorted(orderByExpressions)) {
+          // All order-by columns are sorted, no need to sort the records
+          TransformOperator transformOperator = new TransformPlanNode(_indexSegment, _queryContext, expressions,
+              Math.min(limit + _queryContext.getOffset(), DocIdSetPlanNode.MAX_DOC_PER_CALL)).run();
+          return new SelectionOrderByOperator(_indexSegment, _queryContext, expressions, transformOperator, true);
+        } else if (orderByExpressions.size() == expressions.size()) {
           // All output expressions are ordered
           TransformOperator transformOperator =
               new TransformPlanNode(_indexSegment, _queryContext, expressions, DocIdSetPlanNode.MAX_DOC_PER_CALL).run();
-          return new SelectionOrderByOperator(_indexSegment, _queryContext, expressions, transformOperator);
+          return new SelectionOrderByOperator(_indexSegment, _queryContext, expressions, transformOperator, false);
         } else {
           // Not all output expressions are ordered, only fetch the order-by expressions and docId to avoid the
           // unnecessary data fetch
-          List<ExpressionContext> expressionsToTransform = new ArrayList<>(orderByExpressions.size() + 1);
+          List<ExpressionContext> expressionsToTransform = new ArrayList<>(orderByExpressions.size());
           for (OrderByExpressionContext orderByExpression : orderByExpressions) {
             expressionsToTransform.add(orderByExpression.getExpression());
           }
-          expressionsToTransform.add(ExpressionContext.forIdentifier(BuiltInVirtualColumn.DOCID));
           TransformOperator transformOperator =
               new TransformPlanNode(_indexSegment, _queryContext, expressionsToTransform,
                   DocIdSetPlanNode.MAX_DOC_PER_CALL).run();
-          return new SelectionOrderByOperator(_indexSegment, _queryContext, expressions, transformOperator);
+          return new SelectionOrderByOperator(_indexSegment, _queryContext, expressions, transformOperator, false);
         }
       }
     } else {
@@ -85,4 +88,29 @@ public class SelectionPlanNode implements PlanNode {
       return new EmptySelectionOperator(_indexSegment, expressions, transformOperator);
     }
   }
+
+  /**
+   *  This function checks whether all columns in order by clause are pre-sorted.
+   *  This is used to optimize order by limit clauses.
+   *  For eg:
+   *  A query like "select * from table order by col1, col2 limit 10"
+   *  will take all the n matching rows and add it to a priority queue of size 10.
+   *  This is nlogk operation which can be quite expensive for a large n.
+   *  In the above example, if the docs in the segment are already sorted by col1 and col2 then there is no need for
+   *  sorting at all (only limit is needed).
+   * @return true is all columns in order by clause are sorted . False otherwise
+   */
+  private boolean isAllOrderByColumnsSorted(List<OrderByExpressionContext> orderByExpressions) {
+    for (OrderByExpressionContext orderByExpression : orderByExpressions) {
+      if (!(orderByExpression.getExpression().getType() == ExpressionContext.Type.IDENTIFIER)
+          || !orderByExpression.isAsc()) {
+        return false;
+      }
+      String column = orderByExpression.getExpression().getIdentifier();
+      if (!_indexSegment.getDataSource(column).getDataSourceMetadata().isSorted()) {
+        return false;
+      }
+    }
+    return true;
+  }
 }
