@@ -29,11 +29,9 @@ import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
 import javax.annotation.Nullable;
-import org.apache.pinot.common.request.AggregationInfo;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.core.common.BlockValSet;
 import org.apache.pinot.core.operator.blocks.TransformBlock;
-import org.apache.pinot.parsers.CompilerConstants;
 import org.apache.pinot.segment.spi.AggregationFunctionType;
 import org.apache.pinot.segment.spi.index.startree.AggregationFunctionColumnPair;
 
@@ -46,24 +44,6 @@ public class AggregationFunctionUtils {
   private AggregationFunctionUtils() {
   }
 
-  /**
-   * Extracts the aggregation function arguments (could be column name, transform function or constant) from the
-   * {@link AggregationInfo} as an array of Strings.
-   * <p>NOTE: For backward-compatibility, uses the new Thrift field `expressions` if found, or falls back to the old
-   * aggregationParams based approach.
-   */
-  public static List<String> getArguments(AggregationInfo aggregationInfo) {
-    List<String> expressions = aggregationInfo.getExpressions();
-    if (expressions != null) {
-      return expressions;
-    } else {
-      // NOTE: When the server is upgraded before the broker, the expressions won't be set. Falls back to the old
-      //       aggregationParams based approach.
-      String column = aggregationInfo.getAggregationParams().get(CompilerConstants.COLUMN_KEY_IN_AGGREGATION_INFO);
-      return Arrays.asList(column.split(CompilerConstants.AGGREGATION_FUNCTION_ARG_SEPARATOR));
-    }
-  }
-
   /**
    * (For Star-Tree) Creates an {@link AggregationFunctionColumnPair} from the {@link AggregationFunction}. Returns
    * {@code null} if the {@link AggregationFunction} cannot be represented as an {@link AggregationFunctionColumnPair}
@@ -92,8 +72,8 @@ public class AggregationFunctionUtils {
 
       // NOTE: String.format() is very expensive, so avoid it for whole numbers that can fit in Long.
       //       We simply append ".00000" to long, in order to keep the existing behavior.
-      if (doubleValue <= Long.MAX_VALUE && doubleValue >= Long.MIN_VALUE && DoubleMath
-          .isMathematicalInteger(doubleValue)) {
+      if (doubleValue <= Long.MAX_VALUE && doubleValue >= Long.MIN_VALUE && DoubleMath.isMathematicalInteger(
+          doubleValue)) {
         return (long) doubleValue + ".00000";
       } else {
         return String.format(Locale.US, "%1.5f", doubleValue);
@@ -111,17 +91,6 @@ public class AggregationFunctionUtils {
     }
   }
 
-  /**
-   * Helper function to concatenate arguments using separator.
-   *
-   * @param arguments Arguments to concatenate
-   * @return Concatenated String of arguments
-   */
-  public static String concatArgs(String[] arguments) {
-    return arguments.length > 1 ? String.join(CompilerConstants.AGGREGATION_FUNCTION_ARG_SEPARATOR, arguments)
-        : arguments[0];
-  }
-
   /**
    * Collects all transform expressions required for aggregation/group-by queries.
    * <p>NOTE: We don't need to consider order-by columns here as the ordering is only allowed for aggregation functions
