@@ -29,6 +29,8 @@ import it.unimi.dsi.fastutil.longs.Long2IntOpenHashMap;
 import it.unimi.dsi.fastutil.objects.Object2IntMap;
 import it.unimi.dsi.fastutil.objects.Object2IntOpenHashMap;
 import it.unimi.dsi.fastutil.objects.ObjectIterator;
+import java.math.BigDecimal;
+import java.util.Arrays;
 import java.util.Iterator;
 import java.util.Map;
 import org.apache.pinot.common.request.context.ExpressionContext;
@@ -37,6 +39,7 @@ import org.apache.pinot.core.operator.blocks.TransformBlock;
 import org.apache.pinot.core.operator.transform.TransformOperator;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.spi.utils.ByteArray;
+import org.roaringbitmap.RoaringBitmap;
 
 
 /**
@@ -50,15 +53,23 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
   private final DataType _storedType;
   private final Map _groupKeyMap;
   private final int _globalGroupIdUpperBound;
+  // TODO(nhejazi): Most of the logic between _nullHandlingEnabled=true/false is not sharable, so consider making a
+  //  base implementation, and 2 derived classes, one for null enabled, one for disabled.
+  private final boolean _nullHandlingEnabled;
+
+  private Integer _groupIdForNullValue = null;
+  private final boolean _isSingleValueExpression;
 
   private int _numGroups = 0;
 
   public NoDictionarySingleColumnGroupKeyGenerator(TransformOperator transformOperator,
-      ExpressionContext groupByExpression, int numGroupsLimit) {
+      ExpressionContext groupByExpression, int numGroupsLimit, boolean nullHandlingEnabled) {
     _groupByExpression = groupByExpression;
     _storedType = transformOperator.getResultMetadata(_groupByExpression).getDataType().getStoredType();
     _groupKeyMap = createGroupKeyMap(_storedType);
     _globalGroupIdUpperBound = numGroupsLimit;
+    _nullHandlingEnabled = nullHandlingEnabled;
+    _isSingleValueExpression = transformOperator.getResultMetadata(groupByExpression).isSingleValue();
   }
 
   @Override
@@ -69,6 +80,13 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
   @Override
   public void generateKeysForBlock(TransformBlock transformBlock, int[] groupKeys) {
     BlockValSet blockValSet = transformBlock.getBlockValueSet(_groupByExpression);
+    if (_nullHandlingEnabled) {
+      RoaringBitmap nullBitmap = blockValSet.getNullBitmap();
+      if (nullBitmap != null && !nullBitmap.isEmpty()) {
+        generateKeysForBlockNullHandlingEnabled(transformBlock, groupKeys, nullBitmap);
+        return;
+      }
+    }
     int numDocs = transformBlock.getNumDocs();
 
     switch (_storedType) {
@@ -96,6 +114,12 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
           groupKeys[i] = getKeyForValue(doubleValues[i]);
         }
         break;
+      case BIG_DECIMAL:
+        BigDecimal[] bigDecimalValues = blockValSet.getBigDecimalValuesSV();
+        for (int i = 0; i < numDocs; i++) {
+          groupKeys[i] = getKeyForValue(bigDecimalValues[i]);
+        }
+        break;
       case STRING:
         String[] stringValues = blockValSet.getStringValuesSV();
         for (int i = 0; i < numDocs; i++) {
@@ -113,6 +137,88 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
     }
   }
 
+  public void generateKeysForBlockNullHandlingEnabled(TransformBlock transformBlock, int[] groupKeys,
+      RoaringBitmap nullBitmap) {
+    assert nullBitmap != null;
+    BlockValSet blockValSet = transformBlock.getBlockValueSet(_groupByExpression);
+    int numDocs = transformBlock.getNumDocs();
+
+    switch (_storedType) {
+      case INT:
+        int[] intValues = blockValSet.getIntValuesSV();
+        if (nullBitmap.getCardinality() < numDocs) {
+          for (int i = 0; i < numDocs; i++) {
+            groupKeys[i] = nullBitmap.contains(i) ? getKeyForNullValue() : getKeyForValue(intValues[i]);
+          }
+        } else if (numDocs > 0) {
+          Arrays.fill(groupKeys, 0, numDocs, getKeyForNullValue());
+        }
+        break;
+      case LONG:
+        long[] longValues = blockValSet.getLongValuesSV();
+        if (nullBitmap.getCardinality() < numDocs) {
+          for (int i = 0; i < numDocs; i++) {
+            groupKeys[i] = nullBitmap.contains(i) ? getKeyForNullValue() : getKeyForValue(longValues[i]);
+          }
+        } else if (numDocs > 0) {
+          Arrays.fill(groupKeys, 0, numDocs, getKeyForNullValue());
+        }
+        break;
+      case FLOAT:
+        float[] floatValues = blockValSet.getFloatValuesSV();
+        if (nullBitmap.getCardinality() < numDocs) {
+          for (int i = 0; i < numDocs; i++) {
+            groupKeys[i] = nullBitmap.contains(i) ? getKeyForNullValue() : getKeyForValue(floatValues[i]);
+          }
+        } else if (numDocs > 0) {
+          Arrays.fill(groupKeys, 0, numDocs, getKeyForNullValue());
+        }
+        break;
+      case DOUBLE:
+        double[] doubleValues = blockValSet.getDoubleValuesSV();
+        if (nullBitmap.getCardinality() < numDocs) {
+          for (int i = 0; i < numDocs; i++) {
+            groupKeys[i] = nullBitmap.contains(i) ? getKeyForNullValue() : getKeyForValue(doubleValues[i]);
+          }
+        } else if (numDocs > 0) {
+          Arrays.fill(groupKeys, 0, numDocs, getKeyForNullValue());
+        }
+        break;
+      case BIG_DECIMAL:
+        BigDecimal[] bigDecimalValues = blockValSet.getBigDecimalValuesSV();
+        if (nullBitmap.getCardinality() < numDocs) {
+          for (int i = 0; i < numDocs; i++) {
+            groupKeys[i] = getKeyForValue(nullBitmap.contains(i) ? null : bigDecimalValues[i]);
+          }
+        } else if (numDocs > 0) {
+          Arrays.fill(groupKeys, 0, numDocs, getKeyForValue((BigDecimal) null));
+        }
+        break;
+      case STRING:
+        String[] stringValues = blockValSet.getStringValuesSV();
+        if (nullBitmap.getCardinality() < numDocs) {
+          for (int i = 0; i < numDocs; i++) {
+            groupKeys[i] = getKeyForValue(nullBitmap.contains(i) ? null : stringValues[i]);
+          }
+        } else if (numDocs > 0) {
+          Arrays.fill(groupKeys, 0, numDocs, getKeyForValue((String) null));
+        }
+        break;
+      case BYTES:
+        byte[][] bytesValues = blockValSet.getBytesValuesSV();
+        if (nullBitmap.getCardinality() < numDocs) {
+          for (int i = 0; i < numDocs; i++) {
+            groupKeys[i] = getKeyForValue(nullBitmap.contains(i) ? null : new ByteArray(bytesValues[i]));
+          }
+        } else if (numDocs > 0) {
+          Arrays.fill(groupKeys, 0, numDocs, getKeyForValue((ByteArray) null));
+        }
+        break;
+      default:
+        throw new IllegalArgumentException("Illegal data type for no-dictionary key generator: " + _storedType);
+    }
+  }
+
   /**
    * Helper method to create the group-key map, depending on the data type.
    * Uses primitive maps when possible.
@@ -138,6 +244,10 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
         Double2IntOpenHashMap doubleMap = new Double2IntOpenHashMap();
         doubleMap.defaultReturnValue(INVALID_ID);
         return doubleMap;
+      case BIG_DECIMAL:
+        Object2IntOpenHashMap<BigDecimal> bigDecimalMap = new Object2IntOpenHashMap<BigDecimal>();
+        bigDecimalMap.defaultReturnValue(INVALID_ID);
+        return bigDecimalMap;
       case STRING:
         Object2IntOpenHashMap<String> stringMap = new Object2IntOpenHashMap<>();
         stringMap.defaultReturnValue(INVALID_ID);
@@ -153,8 +263,113 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
 
   @Override
   public void generateKeysForBlock(TransformBlock transformBlock, int[][] groupKeys) {
-    // TODO: Support generating keys for multi-valued columns.
-    throw new UnsupportedOperationException("Operation not supported");
+    int numDocs = transformBlock.getNumDocs();
+    BlockValSet blockValSet = transformBlock.getBlockValueSet(_groupByExpression);
+
+    if (_isSingleValueExpression) {
+      switch (_storedType) {
+        case INT:
+          int[] intValues = blockValSet.getIntValuesSV();
+          for (int i = 0; i < numDocs; i++) {
+            groupKeys[i] = new int[]{getKeyForValue(intValues[i])};
+          }
+          break;
+        case LONG:
+          long[] longValues = blockValSet.getLongValuesSV();
+          for (int i = 0; i < numDocs; i++) {
+            groupKeys[i] = new int[]{getKeyForValue(longValues[i])};
+          }
+          break;
+        case FLOAT:
+          float[] floatValues = blockValSet.getFloatValuesSV();
+          for (int i = 0; i < numDocs; i++) {
+            groupKeys[i] = new int[]{getKeyForValue(floatValues[i])};
+          }
+          break;
+        case DOUBLE:
+          double[] doubleValues = blockValSet.getDoubleValuesSV();
+          for (int i = 0; i < numDocs; i++) {
+            groupKeys[i] = new int[]{getKeyForValue(doubleValues[i])};
+          }
+          break;
+        case STRING:
+          String[] stringValues = blockValSet.getStringValuesSV();
+          for (int i = 0; i < numDocs; i++) {
+            groupKeys[i] = new int[]{getKeyForValue(stringValues[i])};
+          }
+          break;
+        case BYTES:
+          byte[][] byteValues = blockValSet.getBytesValuesSV();
+          for (int i = 0; i < numDocs; i++) {
+            groupKeys[i] = new int[]{getKeyForValue(new ByteArray(byteValues[i]))};
+          }
+          break;
+        default:
+          throw new IllegalArgumentException(
+              "Illegal data type for no-dictionary key generator: " + _storedType);
+      }
+    } else {
+      switch (_storedType) {
+        case INT:
+          int[][] intValues = blockValSet.getIntValuesMV();
+          for (int i = 0; i < numDocs; i++) {
+            int mvSize = intValues[i].length;
+            int[] mvKeys = new int[mvSize];
+            for (int j = 0; j < mvSize; j++) {
+              mvKeys[j] = getKeyForValue(intValues[i][j]);
+            }
+            groupKeys[i] = mvKeys;
+          }
+          break;
+        case LONG:
+          long[][] longValues = blockValSet.getLongValuesMV();
+          for (int i = 0; i < numDocs; i++) {
+            int mvSize = longValues[i].length;
+            int[] mvKeys = new int[mvSize];
+            for (int j = 0; j < mvSize; j++) {
+              mvKeys[j] = getKeyForValue(longValues[i][j]);
+            }
+            groupKeys[i] = mvKeys;
+          }
+          break;
+        case FLOAT:
+          float[][] floatValues = blockValSet.getFloatValuesMV();
+          for (int i = 0; i < numDocs; i++) {
+            int mvSize = floatValues[i].length;
+            int[] mvKeys = new int[mvSize];
+            for (int j = 0; j < mvSize; j++) {
+              mvKeys[j] = getKeyForValue(floatValues[i][j]);
+            }
+            groupKeys[i] = mvKeys;
+          }
+          break;
+        case DOUBLE:
+          double[][] doubleValues = blockValSet.getDoubleValuesMV();
+          for (int i = 0; i < numDocs; i++) {
+            int mvSize = doubleValues[i].length;
+            int[] mvKeys = new int[mvSize];
+            for (int j = 0; j < mvSize; j++) {
+              mvKeys[j] = getKeyForValue(doubleValues[i][j]);
+            }
+            groupKeys[i] = mvKeys;
+          }
+          break;
+        case STRING:
+          String[][] stringValues = blockValSet.getStringValuesMV();
+          for (int i = 0; i < numDocs; i++) {
+            int mvSize = stringValues[i].length;
+            int[] mvKeys = new int[mvSize];
+            for (int j = 0; j < mvSize; j++) {
+              mvKeys[j] = getKeyForValue(stringValues[i][j]);
+            }
+            groupKeys[i] = mvKeys;
+          }
+          break;
+        default:
+          throw new IllegalArgumentException(
+              "Illegal data type for no-dictionary key generator: " + _storedType);
+      }
+    }
   }
 
   @Override
@@ -166,13 +381,14 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
   public Iterator<GroupKey> getGroupKeys() {
     switch (_storedType) {
       case INT:
-        return new IntGroupKeyIterator((Int2IntOpenHashMap) _groupKeyMap);
+        return new IntGroupKeyIterator((Int2IntOpenHashMap) _groupKeyMap, _groupIdForNullValue);
       case LONG:
-        return new LongGroupKeyIterator((Long2IntOpenHashMap) _groupKeyMap);
+        return new LongGroupKeyIterator((Long2IntOpenHashMap) _groupKeyMap, _groupIdForNullValue);
       case FLOAT:
-        return new FloatGroupKeyIterator((Float2IntOpenHashMap) _groupKeyMap);
+        return new FloatGroupKeyIterator((Float2IntOpenHashMap) _groupKeyMap, _groupIdForNullValue);
       case DOUBLE:
-        return new DoubleGroupKeyIterator((Double2IntOpenHashMap) _groupKeyMap);
+        return new DoubleGroupKeyIterator((Double2IntOpenHashMap) _groupKeyMap, _groupIdForNullValue);
+      case BIG_DECIMAL:
       case STRING:
       case BYTES:
         return new ObjectGroupKeyIterator((Object2IntOpenHashMap) _groupKeyMap);
@@ -181,23 +397,15 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
     }
   }
 
-  @Override
-  public Iterator<StringGroupKey> getStringGroupKeys() {
-    switch (_storedType) {
-      case INT:
-        return new IntStringGroupKeyIterator((Int2IntOpenHashMap) _groupKeyMap);
-      case LONG:
-        return new LongStringGroupKeyIterator((Long2IntOpenHashMap) _groupKeyMap);
-      case FLOAT:
-        return new FloatStringGroupKeyIterator((Float2IntOpenHashMap) _groupKeyMap);
-      case DOUBLE:
-        return new DoubleStringGroupKeyIterator((Double2IntOpenHashMap) _groupKeyMap);
-      case STRING:
-      case BYTES:
-        return new ObjectStringGroupKeyIterator((Object2IntOpenHashMap) _groupKeyMap);
-      default:
-        throw new IllegalStateException();
+  private int getKeyForNullValue() {
+    if (_groupIdForNullValue != null) {
+      return _groupIdForNullValue;
     }
+    if (_numGroups < _globalGroupIdUpperBound) {
+      _groupIdForNullValue = _numGroups++;
+      return _groupIdForNullValue;
+    }
+    return INVALID_ID;
   }
 
   @Override
@@ -245,6 +453,16 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
     return groupId;
   }
 
+  private int getKeyForValue(BigDecimal value) {
+    Object2IntMap<BigDecimal> map = (Object2IntMap<BigDecimal>) _groupKeyMap;
+    int groupId = map.getInt(value);
+    if (groupId == INVALID_ID && _numGroups < _globalGroupIdUpperBound) {
+      groupId = _numGroups++;
+      map.put(value, groupId);
+    }
+    return groupId;
+  }
+
   private int getKeyForValue(String value) {
     Object2IntMap<String> map = (Object2IntMap<String>) _groupKeyMap;
     int groupId = map.getInt(value);
@@ -268,19 +486,28 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
   private static class IntGroupKeyIterator implements Iterator<GroupKey> {
     final Iterator<Int2IntMap.Entry> _iterator;
     final GroupKey _groupKey;
+    Integer _groupKeyForNullValue;
 
-    IntGroupKeyIterator(Int2IntOpenHashMap intMap) {
+    IntGroupKeyIterator(Int2IntOpenHashMap intMap, Integer groupKeyForNullValue) {
       _iterator = intMap.int2IntEntrySet().fastIterator();
       _groupKey = new GroupKey();
+      _groupKeyForNullValue = groupKeyForNullValue;
     }
 
     @Override
     public boolean hasNext() {
-      return _iterator.hasNext();
+      return _iterator.hasNext() || _groupKeyForNullValue != null;
     }
 
     @Override
     public GroupKey next() {
+      // TODO(nhejazi): revisit to avoid adding overhead to the regular case where null handling is not enabled.
+      if (_groupKeyForNullValue != null) {
+        _groupKey._groupId = _groupKeyForNullValue;
+        _groupKey._keys = new Object[]{null};
+        _groupKeyForNullValue = null;
+        return _groupKey;
+      }
       Int2IntMap.Entry entry = _iterator.next();
       _groupKey._groupId = entry.getIntValue();
       _groupKey._keys = new Object[]{entry.getIntKey()};
@@ -296,19 +523,27 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
   private static class LongGroupKeyIterator implements Iterator<GroupKey> {
     final Iterator<Long2IntMap.Entry> _iterator;
     final GroupKey _groupKey;
+    Integer _groupKeyForNullValue;
 
-    LongGroupKeyIterator(Long2IntOpenHashMap longMap) {
+    LongGroupKeyIterator(Long2IntOpenHashMap longMap, Integer groupKeyForNullValue) {
       _iterator = longMap.long2IntEntrySet().fastIterator();
       _groupKey = new GroupKey();
+      _groupKeyForNullValue = groupKeyForNullValue;
     }
 
     @Override
     public boolean hasNext() {
-      return _iterator.hasNext();
+      return _iterator.hasNext() || _groupKeyForNullValue != null;
     }
 
     @Override
     public GroupKey next() {
+      if (_groupKeyForNullValue != null) {
+        _groupKey._groupId = _groupKeyForNullValue;
+        _groupKey._keys = new Object[]{null};
+        _groupKeyForNullValue = null;
+        return _groupKey;
+      }
       Long2IntMap.Entry entry = _iterator.next();
       _groupKey._groupId = entry.getIntValue();
       _groupKey._keys = new Object[]{entry.getLongKey()};
@@ -324,19 +559,27 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
   private static class FloatGroupKeyIterator implements Iterator<GroupKey> {
     final Iterator<Float2IntMap.Entry> _iterator;
     final GroupKey _groupKey;
+    Integer _groupKeyForNullValue;
 
-    FloatGroupKeyIterator(Float2IntOpenHashMap floatMap) {
+    FloatGroupKeyIterator(Float2IntOpenHashMap floatMap, Integer groupKeyForNullValue) {
       _iterator = floatMap.float2IntEntrySet().fastIterator();
       _groupKey = new GroupKey();
+      _groupKeyForNullValue = groupKeyForNullValue;
     }
 
     @Override
     public boolean hasNext() {
-      return _iterator.hasNext();
+      return _iterator.hasNext() || _groupKeyForNullValue != null;
     }
 
     @Override
     public GroupKey next() {
+      if (_groupKeyForNullValue != null) {
+        _groupKey._groupId = _groupKeyForNullValue;
+        _groupKey._keys = new Object[]{null};
+        _groupKeyForNullValue = null;
+        return _groupKey;
+      }
       Float2IntMap.Entry entry = _iterator.next();
       _groupKey._groupId = entry.getIntValue();
       _groupKey._keys = new Object[]{entry.getFloatKey()};
@@ -352,19 +595,27 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
   private static class DoubleGroupKeyIterator implements Iterator<GroupKey> {
     final Iterator<Double2IntMap.Entry> _iterator;
     final GroupKey _groupKey;
+    Integer _groupKeyForNullValue;
 
-    DoubleGroupKeyIterator(Double2IntOpenHashMap doubleMap) {
+    DoubleGroupKeyIterator(Double2IntOpenHashMap doubleMap, Integer groupKeyForNullValue) {
       _iterator = doubleMap.double2IntEntrySet().fastIterator();
       _groupKey = new GroupKey();
+      _groupKeyForNullValue = groupKeyForNullValue;
     }
 
     @Override
     public boolean hasNext() {
-      return _iterator.hasNext();
+      return _iterator.hasNext() || _groupKeyForNullValue != null;
     }
 
     @Override
     public GroupKey next() {
+      if (_groupKeyForNullValue != null) {
+        _groupKey._groupId = _groupKeyForNullValue;
+        _groupKey._keys = new Object[]{null};
+        _groupKeyForNullValue = null;
+        return _groupKey;
+      }
       Double2IntMap.Entry entry = _iterator.next();
       _groupKey._groupId = entry.getIntValue();
       _groupKey._keys = new Object[]{entry.getDoubleKey()};
@@ -404,144 +655,4 @@ public class NoDictionarySingleColumnGroupKeyGenerator implements GroupKeyGenera
       throw new UnsupportedOperationException();
     }
   }
-
-  private static class IntStringGroupKeyIterator implements Iterator<StringGroupKey> {
-    final Iterator<Int2IntMap.Entry> _iterator;
-    final StringGroupKey _groupKey;
-
-    IntStringGroupKeyIterator(Int2IntOpenHashMap intMap) {
-      _iterator = intMap.int2IntEntrySet().fastIterator();
-      _groupKey = new StringGroupKey();
-    }
-
-    @Override
-    public boolean hasNext() {
-      return _iterator.hasNext();
-    }
-
-    @Override
-    public StringGroupKey next() {
-      Int2IntMap.Entry entry = _iterator.next();
-      _groupKey._groupId = entry.getIntValue();
-      _groupKey._stringKey = Integer.toString(entry.getIntKey());
-      return _groupKey;
-    }
-
-    @Override
-    public void remove() {
-      throw new UnsupportedOperationException();
-    }
-  }
-
-  private static class LongStringGroupKeyIterator implements Iterator<StringGroupKey> {
-    final Iterator<Long2IntMap.Entry> _iterator;
-    final StringGroupKey _groupKey;
-
-    LongStringGroupKeyIterator(Long2IntOpenHashMap longMap) {
-      _iterator = longMap.long2IntEntrySet().fastIterator();
-      _groupKey = new StringGroupKey();
-    }
-
-    @Override
-    public boolean hasNext() {
-      return _iterator.hasNext();
-    }
-
-    @Override
-    public StringGroupKey next() {
-      Long2IntMap.Entry entry = _iterator.next();
-      _groupKey._groupId = entry.getIntValue();
-      _groupKey._stringKey = Long.toString(entry.getLongKey());
-      return _groupKey;
-    }
-
-    @Override
-    public void remove() {
-      throw new UnsupportedOperationException();
-    }
-  }
-
-  private static class FloatStringGroupKeyIterator implements Iterator<StringGroupKey> {
-    final Iterator<Float2IntMap.Entry> _iterator;
-    final StringGroupKey _groupKey;
-
-    FloatStringGroupKeyIterator(Float2IntOpenHashMap floatMap) {
-      _iterator = floatMap.float2IntEntrySet().fastIterator();
-      _groupKey = new StringGroupKey();
-    }
-
-    @Override
-    public boolean hasNext() {
-      return _iterator.hasNext();
-    }
-
-    @Override
-    public StringGroupKey next() {
-      Float2IntMap.Entry entry = _iterator.next();
-      _groupKey._groupId = entry.getIntValue();
-      _groupKey._stringKey = Float.toString(entry.getFloatKey());
-      return _groupKey;
-    }
-
-    @Override
-    public void remove() {
-      throw new UnsupportedOperationException();
-    }
-  }
-
-  private static class DoubleStringGroupKeyIterator implements Iterator<StringGroupKey> {
-    final Iterator<Double2IntMap.Entry> _iterator;
-    final StringGroupKey _groupKey;
-
-    DoubleStringGroupKeyIterator(Double2IntOpenHashMap doubleMap) {
-      _iterator = doubleMap.double2IntEntrySet().fastIterator();
-      _groupKey = new StringGroupKey();
-    }
-
-    @Override
-    public boolean hasNext() {
-      return _iterator.hasNext();
-    }
-
-    @Override
-    public StringGroupKey next() {
-      Double2IntMap.Entry entry = _iterator.next();
-      _groupKey._groupId = entry.getIntValue();
-      _groupKey._stringKey = Double.toString(entry.getDoubleKey());
-      return _groupKey;
-    }
-
-    @Override
-    public void remove() {
-      throw new UnsupportedOperationException();
-    }
-  }
-
-  private static class ObjectStringGroupKeyIterator implements Iterator<StringGroupKey> {
-    final ObjectIterator<Object2IntMap.Entry> _iterator;
-    final StringGroupKey _groupKey;
-
-    ObjectStringGroupKeyIterator(Object2IntOpenHashMap objectMap) {
-      _iterator = objectMap.object2IntEntrySet().fastIterator();
-      _groupKey = new StringGroupKey();
-    }
-
-    @Override
-    public boolean hasNext() {
-      return _iterator.hasNext();
-    }
-
-    @Override
-    public StringGroupKey next() {
-      Object2IntMap.Entry entry = _iterator.next();
-      _groupKey._groupId = entry.getIntValue();
-      _groupKey._stringKey = entry.getKey().toString();
-      return _groupKey;
-    }
-
-    @Override
-    public void remove() {
-      throw new UnsupportedOperationException();
-    }
-  }
 }
