@@ -21,18 +21,19 @@ package org.apache.pinot.core.startree.operator;
 import it.unimi.dsi.fastutil.ints.IntIterator;
 import it.unimi.dsi.fastutil.ints.IntOpenHashSet;
 import it.unimi.dsi.fastutil.ints.IntSet;
+import java.util.ArrayDeque;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
-import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Queue;
 import java.util.Set;
 import javax.annotation.Nullable;
+import org.apache.pinot.core.common.Operator;
 import org.apache.pinot.core.operator.blocks.EmptyFilterBlock;
 import org.apache.pinot.core.operator.blocks.FilterBlock;
 import org.apache.pinot.core.operator.filter.BaseFilterOperator;
@@ -40,6 +41,7 @@ import org.apache.pinot.core.operator.filter.BitmapBasedFilterOperator;
 import org.apache.pinot.core.operator.filter.EmptyFilterOperator;
 import org.apache.pinot.core.operator.filter.FilterOperatorUtils;
 import org.apache.pinot.core.operator.filter.predicate.PredicateEvaluator;
+import org.apache.pinot.core.query.request.context.QueryContext;
 import org.apache.pinot.core.startree.CompositePredicateEvaluator;
 import org.apache.pinot.segment.spi.datasource.DataSource;
 import org.apache.pinot.segment.spi.index.startree.StarTree;
@@ -84,6 +86,7 @@ import org.roaringbitmap.buffer.MutableRoaringBitmap;
  * </ul>
  */
 public class StarTreeFilterOperator extends BaseFilterOperator {
+  private static final String EXPLAIN_NAME = "FILTER_STARTREE_INDEX";
 
   /**
    * Helper class to wrap the information needed when traversing the star tree.
@@ -113,28 +116,22 @@ public class StarTreeFilterOperator extends BaseFilterOperator {
     }
   }
 
-  private static final String OPERATOR_NAME = "StarTreeFilterOperator";
   // If (number of matching dictionary ids * threshold) > (number of child nodes), use scan to traverse nodes instead of
   // binary search on each dictionary id
   private static final int USE_SCAN_TO_TRAVERSE_NODES_THRESHOLD = 10;
 
-  // Star-tree
+  private final QueryContext _queryContext;
   private final StarTreeV2 _starTreeV2;
-  // Map from column to predicate evaluators
   private final Map<String, List<CompositePredicateEvaluator>> _predicateEvaluatorsMap;
-  // Set of group-by columns
   private final Set<String> _groupByColumns;
 
-  private final Map<String, String> _debugOptions;
-
   boolean _resultEmpty = false;
 
-  public StarTreeFilterOperator(StarTreeV2 starTreeV2,
-      Map<String, List<CompositePredicateEvaluator>> predicateEvaluatorsMap, Set<String> groupByColumns,
-      @Nullable Map<String, String> debugOptions) {
+  public StarTreeFilterOperator(QueryContext queryContext, StarTreeV2 starTreeV2,
+      Map<String, List<CompositePredicateEvaluator>> predicateEvaluatorsMap, @Nullable Set<String> groupByColumns) {
+    _queryContext = queryContext;
     _starTreeV2 = starTreeV2;
     _predicateEvaluatorsMap = predicateEvaluatorsMap;
-    _debugOptions = debugOptions;
 
     if (groupByColumns != null) {
       _groupByColumns = new HashSet<>(groupByColumns);
@@ -159,8 +156,13 @@ public class StarTreeFilterOperator extends BaseFilterOperator {
   }
 
   @Override
-  public String getOperatorName() {
-    return OPERATOR_NAME;
+  public String toExplainString() {
+    return EXPLAIN_NAME;
+  }
+
+  @Override
+  public List<Operator> getChildOperators() {
+    return Collections.emptyList();
   }
 
   /**
@@ -204,13 +206,13 @@ public class StarTreeFilterOperator extends BaseFilterOperator {
             orChildFilterOperators
                 .add(FilterOperatorUtils.getLeafFilterOperator(childPredicateEvaluator, dataSource, numDocs));
           }
-          childFilterOperators
-              .add(FilterOperatorUtils.getOrFilterOperator(orChildFilterOperators, numDocs, _debugOptions));
+          childFilterOperators.add(
+              FilterOperatorUtils.getOrFilterOperator(_queryContext, orChildFilterOperators, numDocs));
         }
       }
     }
 
-    return FilterOperatorUtils.getAndFilterOperator(childFilterOperators, numDocs, _debugOptions);
+    return FilterOperatorUtils.getAndFilterOperator(_queryContext, childFilterOperators, numDocs);
   }
 
   /**
@@ -229,7 +231,7 @@ public class StarTreeFilterOperator extends BaseFilterOperator {
     StarTreeNode starTreeRootNode = starTree.getRoot();
 
     // Use BFS to traverse the star tree
-    Queue<SearchEntry> queue = new LinkedList<>();
+    Queue<SearchEntry> queue = new ArrayDeque<>();
     queue.add(new SearchEntry(starTreeRootNode, _predicateEvaluatorsMap.keySet(), _groupByColumns));
     SearchEntry searchEntry;
     while ((searchEntry = queue.poll()) != null) {
