@@ -25,6 +25,7 @@ import io.netty.buffer.ByteBuf;
 import io.netty.buffer.Unpooled;
 import io.netty.channel.ChannelHandlerContext;
 import io.netty.channel.SimpleChannelInboundHandler;
+import io.netty.handler.ssl.SslHandshakeCompletionEvent;
 import java.util.Map;
 import java.util.concurrent.TimeUnit;
 import javax.annotation.Nullable;
@@ -36,13 +37,15 @@ import org.apache.pinot.common.metrics.ServerTimer;
 import org.apache.pinot.common.request.InstanceRequest;
 import org.apache.pinot.common.utils.DataTable;
 import org.apache.pinot.common.utils.DataTable.MetadataKey;
-import org.apache.pinot.core.common.datatable.DataTableBuilder;
+import org.apache.pinot.core.common.datatable.DataTableFactory;
 import org.apache.pinot.core.query.request.ServerQueryRequest;
 import org.apache.pinot.core.query.scheduler.QueryScheduler;
+import org.apache.pinot.server.access.AccessControl;
 import org.apache.pinot.spi.utils.BytesUtils;
 import org.apache.thrift.TDeserializer;
 import org.apache.thrift.TException;
 import org.apache.thrift.protocol.TCompactProtocol;
+import org.apache.thrift.transport.TTransportException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -57,13 +60,32 @@ public class InstanceRequestHandler extends SimpleChannelInboundHandler<ByteBuf>
   // TODO: make it configurable
   private static final int SLOW_QUERY_LATENCY_THRESHOLD_MS = 100;
 
-  private final TDeserializer _deserializer = new TDeserializer(new TCompactProtocol.Factory());
+  private final TDeserializer _deserializer;
   private final QueryScheduler _queryScheduler;
   private final ServerMetrics _serverMetrics;
+  private final AccessControl _accessControl;
 
-  public InstanceRequestHandler(QueryScheduler queryScheduler, ServerMetrics serverMetrics) {
+  public InstanceRequestHandler(QueryScheduler queryScheduler, ServerMetrics serverMetrics,
+      AccessControl accessControl) {
     _queryScheduler = queryScheduler;
     _serverMetrics = serverMetrics;
+    _accessControl = accessControl;
+    try {
+      _deserializer = new TDeserializer(new TCompactProtocol.Factory());
+    } catch (TTransportException e) {
+      throw new RuntimeException("Failed to initialize Thrift Deserializer", e);
+    }
+  }
+
+  @Override
+  public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
+    super.userEventTriggered(ctx, evt);
+    if (evt instanceof SslHandshakeCompletionEvent) {
+      if (!_accessControl.isAuthorizedChannel(ctx)) {
+        ctx.disconnect();
+        LOGGER.error("Exception while processing instance request: Unauthorized access to pinot-server");
+      }
+    }
   }
 
   /**
@@ -98,7 +120,7 @@ public class InstanceRequestHandler extends SimpleChannelInboundHandler<ByteBuf>
       tableNameWithType = queryRequest.getTableNameWithType();
 
       // Submit query for execution and register callback for execution results.
-      Futures.addCallback(_queryScheduler.submit(queryRequest),
+      Futures.addCallback(_queryScheduler.submitQuery(queryRequest),
           createCallback(ctx, tableNameWithType, queryArrivalTimeMs, instanceRequest, queryRequest),
           MoreExecutors.directExecutor());
     } catch (Exception e) {
@@ -111,7 +133,7 @@ public class InstanceRequestHandler extends SimpleChannelInboundHandler<ByteBuf>
       String hexString = requestBytes != null ? BytesUtils.toHexString(requestBytes) : "";
       long reqestId = instanceRequest != null ? instanceRequest.getRequestId() : 0;
       LOGGER.error("Exception while processing instance request: {}", hexString, e);
-      sendErrorResponse(ctx, reqestId, tableNameWithType, queryArrivalTimeMs, DataTableBuilder.getEmptyDataTable(), e);
+      sendErrorResponse(ctx, reqestId, tableNameWithType, queryArrivalTimeMs, DataTableFactory.getEmptyDataTable(), e);
     }
   }
 
@@ -126,7 +148,7 @@ public class InstanceRequestHandler extends SimpleChannelInboundHandler<ByteBuf>
         } else {
           // Send exception response.
           sendErrorResponse(ctx, queryRequest.getRequestId(), tableNameWithType, queryArrivalTimeMs,
-              DataTableBuilder.getEmptyDataTable(), new Exception("Null query response."));
+              DataTableFactory.getEmptyDataTable(), new Exception("Null query response."));
         }
       }
 
@@ -135,7 +157,7 @@ public class InstanceRequestHandler extends SimpleChannelInboundHandler<ByteBuf>
         // Send exception response.
         LOGGER.error("Exception while processing instance request", t);
         sendErrorResponse(ctx, instanceRequest.getRequestId(), tableNameWithType, queryArrivalTimeMs,
-            DataTableBuilder.getEmptyDataTable(), new Exception(t));
+            DataTableFactory.getEmptyDataTable(), new Exception(t));
       }
     };
   }
@@ -146,7 +168,7 @@ public class InstanceRequestHandler extends SimpleChannelInboundHandler<ByteBuf>
     // will only be called if for some remote reason we are unable to handle exceptions in channelRead0.
     String message = "Unhandled Exception in " + getClass().getCanonicalName();
     LOGGER.error(message, cause);
-    sendErrorResponse(ctx, 0, null, System.currentTimeMillis(), DataTableBuilder.getEmptyDataTable(),
+    sendErrorResponse(ctx, 0, null, System.currentTimeMillis(), DataTableFactory.getEmptyDataTable(),
         new Exception(message, cause));
   }
 
