@@ -19,6 +19,7 @@
 package org.apache.pinot.core.operator.combine;
 
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
@@ -64,7 +65,7 @@ public class CombineSlowOperatorsTest {
   @Test
   public void testSelectionOnlyCombineOperator() {
     List<Operator> operators = getOperators();
-    QueryContext queryContext = QueryContextConverterUtils.getQueryContextFromSQL("SELECT * FROM testTable");
+    QueryContext queryContext = QueryContextConverterUtils.getQueryContext("SELECT * FROM testTable");
     queryContext.setEndTimeMs(System.currentTimeMillis() + TIMEOUT_MS);
     SelectionOnlyCombineOperator combineOperator =
         new SelectionOnlyCombineOperator(operators, queryContext, _executorService);
@@ -77,28 +78,18 @@ public class CombineSlowOperatorsTest {
   @Test
   public void testAggregationOnlyCombineOperator() {
     List<Operator> operators = getOperators();
-    QueryContext queryContext = QueryContextConverterUtils.getQueryContextFromSQL("SELECT COUNT(*) FROM testTable");
+    QueryContext queryContext = QueryContextConverterUtils.getQueryContext("SELECT COUNT(*) FROM testTable");
     queryContext.setEndTimeMs(System.currentTimeMillis() + TIMEOUT_MS);
     AggregationOnlyCombineOperator combineOperator =
         new AggregationOnlyCombineOperator(operators, queryContext, _executorService);
     testCombineOperator(operators, combineOperator);
   }
 
-  @Test
-  public void testGroupByCombineOperator() {
-    List<Operator> operators = getOperators();
-    QueryContext queryContext =
-        QueryContextConverterUtils.getQueryContextFromSQL("SELECT COUNT(*) FROM testTable GROUP BY column");
-    queryContext.setEndTimeMs(System.currentTimeMillis() + TIMEOUT_MS);
-    GroupByCombineOperator combineOperator = new GroupByCombineOperator(operators, queryContext, _executorService);
-    testCombineOperator(operators, combineOperator);
-  }
-
   @Test
   public void testGroupByOrderByCombineOperator() {
     List<Operator> operators = getOperators();
     QueryContext queryContext =
-        QueryContextConverterUtils.getQueryContextFromSQL("SELECT COUNT(*) FROM testTable GROUP BY column");
+        QueryContextConverterUtils.getQueryContext("SELECT COUNT(*) FROM testTable GROUP BY column");
     queryContext.setEndTimeMs(System.currentTimeMillis() + TIMEOUT_MS);
     GroupByOrderByCombineOperator combineOperator =
         new GroupByOrderByCombineOperator(operators, queryContext, _executorService);
@@ -139,6 +130,8 @@ public class CombineSlowOperatorsTest {
   }
 
   private static class SlowOperator extends BaseOperator {
+    private static final String EXPLAIN_NAME = "SLOW";
+
     final AtomicBoolean _operationInProgress = new AtomicBoolean();
     final AtomicBoolean _notInterrupted = new AtomicBoolean();
 
@@ -164,8 +157,13 @@ public class CombineSlowOperatorsTest {
     }
 
     @Override
-    public String getOperatorName() {
-      return "SlowOperator";
+    public String toExplainString() {
+      return EXPLAIN_NAME;
+    }
+
+    @Override
+    public List<Operator> getChildOperators() {
+      return Collections.emptyList();
     }
 
     @Override
