@@ -31,13 +31,14 @@ import static org.testng.Assert.assertTrue;
 
 @SuppressWarnings("rawtypes")
 public class AggregationFunctionFactoryTest {
-  private static final String ARGUMENT = "(column)";
+  private static final String ARGUMENT_COLUMN = "(column)";
+  private static final String ARGUMENT_STAR = "(*)";
   private static final QueryContext DUMMY_QUERY_CONTEXT =
-      QueryContextConverterUtils.getQueryContextFromSQL("SELECT * FROM testTable");
+      QueryContextConverterUtils.getQueryContext("SELECT * FROM testTable");
 
   @Test
   public void testGetAggregationFunction() {
-    FunctionContext function = getFunction("CoUnT");
+    FunctionContext function = getFunction("CoUnT", ARGUMENT_STAR);
     AggregationFunction aggregationFunction =
         AggregationFunctionFactory.getAggregationFunction(function, DUMMY_QUERY_CONTEXT);
     assertTrue(aggregationFunction instanceof CountAggregationFunction);
@@ -87,6 +88,48 @@ public class AggregationFunctionFactoryTest {
     assertEquals(aggregationFunction.getColumnName(), "mode_column");
     assertEquals(aggregationFunction.getResultColumnName(), function.toString());
 
+    function = getFunction("FiRsTwItHtImE", "(column,timeColumn,'BOOLEAN')");
+    aggregationFunction = AggregationFunctionFactory.getAggregationFunction(function, DUMMY_QUERY_CONTEXT);
+    assertTrue(aggregationFunction instanceof FirstIntValueWithTimeAggregationFunction);
+    assertEquals(aggregationFunction.getType(), AggregationFunctionType.FIRSTWITHTIME);
+    assertEquals(aggregationFunction.getColumnName(), "firstWithTime_column_timeColumn_BOOLEAN");
+    assertEquals(aggregationFunction.getResultColumnName(), function.toString());
+
+    function = getFunction("FiRsTwItHtImE", "(column,timeColumn,'INT')");
+    aggregationFunction = AggregationFunctionFactory.getAggregationFunction(function, DUMMY_QUERY_CONTEXT);
+    assertTrue(aggregationFunction instanceof FirstIntValueWithTimeAggregationFunction);
+    assertEquals(aggregationFunction.getType(), AggregationFunctionType.FIRSTWITHTIME);
+    assertEquals(aggregationFunction.getColumnName(), "firstWithTime_column_timeColumn_INT");
+    assertEquals(aggregationFunction.getResultColumnName(), function.toString());
+
+    function = getFunction("FiRsTwItHtImE", "(column,timeColumn,'LONG')");
+    aggregationFunction = AggregationFunctionFactory.getAggregationFunction(function, DUMMY_QUERY_CONTEXT);
+    assertTrue(aggregationFunction instanceof FirstLongValueWithTimeAggregationFunction);
+    assertEquals(aggregationFunction.getType(), AggregationFunctionType.FIRSTWITHTIME);
+    assertEquals(aggregationFunction.getColumnName(), "firstWithTime_column_timeColumn_LONG");
+    assertEquals(aggregationFunction.getResultColumnName(), function.toString());
+
+    function = getFunction("FiRsTwItHtImE", "(column,timeColumn,'FLOAT')");
+    aggregationFunction = AggregationFunctionFactory.getAggregationFunction(function, DUMMY_QUERY_CONTEXT);
+    assertTrue(aggregationFunction instanceof FirstFloatValueWithTimeAggregationFunction);
+    assertEquals(aggregationFunction.getType(), AggregationFunctionType.FIRSTWITHTIME);
+    assertEquals(aggregationFunction.getColumnName(), "firstWithTime_column_timeColumn_FLOAT");
+    assertEquals(aggregationFunction.getResultColumnName(), function.toString());
+
+    function = getFunction("FiRsTwItHtImE", "(column,timeColumn,'DOUBLE')");
+    aggregationFunction = AggregationFunctionFactory.getAggregationFunction(function, DUMMY_QUERY_CONTEXT);
+    assertTrue(aggregationFunction instanceof FirstDoubleValueWithTimeAggregationFunction);
+    assertEquals(aggregationFunction.getType(), AggregationFunctionType.FIRSTWITHTIME);
+    assertEquals(aggregationFunction.getColumnName(), "firstWithTime_column_timeColumn_DOUBLE");
+    assertEquals(aggregationFunction.getResultColumnName(), function.toString());
+
+    function = getFunction("FiRsTwItHtImE", "(column,timeColumn,'STRING')");
+    aggregationFunction = AggregationFunctionFactory.getAggregationFunction(function, DUMMY_QUERY_CONTEXT);
+    assertTrue(aggregationFunction instanceof FirstStringValueWithTimeAggregationFunction);
+    assertEquals(aggregationFunction.getType(), AggregationFunctionType.FIRSTWITHTIME);
+    assertEquals(aggregationFunction.getColumnName(), "firstWithTime_column_timeColumn_STRING");
+    assertEquals(aggregationFunction.getResultColumnName(), function.toString());
+
     function = getFunction("LaStWiThTiMe", "(column,timeColumn,'BOOLEAN')");
     aggregationFunction = AggregationFunctionFactory.getAggregationFunction(function, DUMMY_QUERY_CONTEXT);
     assertTrue(aggregationFunction instanceof LastIntValueWithTimeAggregationFunction);
@@ -404,17 +447,17 @@ public class AggregationFunctionFactoryTest {
   }
 
   private FunctionContext getFunction(String functionName) {
-    return getFunction(functionName, ARGUMENT);
+    return getFunction(functionName, ARGUMENT_COLUMN);
   }
 
   private FunctionContext getFunction(String functionName, String args) {
-    return RequestContextUtils.getExpressionFromSQL(functionName + args).getFunction();
+    return RequestContextUtils.getExpression(functionName + args).getFunction();
   }
 
   @Test
   public void testAggregationFunctionWithMultipleArgs() {
     QueryContext queryContext =
-        QueryContextConverterUtils.getQueryContextFromSQL("SELECT DISTINCT column1, column2, column3 FROM testTable");
+        QueryContextConverterUtils.getQueryContext("SELECT DISTINCT column1, column2, column3 FROM testTable");
     AggregationFunction aggregationFunction = AggregationFunctionFactory
         .getAggregationFunction(queryContext.getSelectExpressions().get(0).getFunction(), queryContext);
     assertTrue(aggregationFunction instanceof DistinctAggregationFunction);
