@@ -36,7 +36,7 @@ import java.util.List;
 import java.util.Map;
 import org.apache.commons.io.FileUtils;
 import org.apache.pinot.spi.env.PinotConfiguration;
-import org.apache.pinot.spi.filesystem.PinotFS;
+import org.apache.pinot.spi.filesystem.BasePinotFS;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import software.amazon.awssdk.auth.credentials.AwsBasicCredentials;
@@ -70,7 +70,7 @@ import software.amazon.awssdk.services.s3.model.ServerSideEncryption;
 /**
  * Implementation of PinotFS for AWS S3 file system
  */
-public class S3PinotFS extends PinotFS {
+public class S3PinotFS extends BasePinotFS {
   public static final String ACCESS_KEY = "accessKey";
   public static final String SECRET_KEY = "secretKey";
   public static final String REGION = "region";
@@ -92,7 +92,7 @@ public class S3PinotFS extends PinotFS {
 
   @Override
   public void init(PinotConfiguration config) {
-    Preconditions.checkArgument(!isNullOrEmpty(config.getProperty(REGION)));
+    Preconditions.checkArgument(!isNullOrEmpty(config.getProperty(REGION)), "Region can't be null or empty");
     String region = config.getProperty(REGION);
     _disableAcl = config.getProperty(DISABLE_ACL_CONFIG_KEY, DEFAULT_DISABLE_ACL);
     String serverSideEncryption = config.getProperty(SERVER_SIDE_ENCRYPTION_CONFIG_KEY);
@@ -363,14 +363,14 @@ public class S3PinotFS extends PinotFS {
   @Override
   public boolean doMove(URI srcUri, URI dstUri)
       throws IOException {
-    if (copy(srcUri, dstUri)) {
+    if (copyDir(srcUri, dstUri)) {
       return delete(srcUri, true);
     }
     return false;
   }
 
   @Override
-  public boolean copy(URI srcUri, URI dstUri)
+  public boolean copyDir(URI srcUri, URI dstUri)
       throws IOException {
     LOGGER.info("Copying uri {} to uri {}", srcUri, dstUri);
     Preconditions.checkState(exists(srcUri), "Source URI '%s' does not exist", srcUri);
