@@ -46,7 +46,6 @@ import org.apache.pinot.spi.config.table.TableTaskConfig;
 import org.apache.pinot.spi.config.table.TableType;
 import org.apache.pinot.spi.config.table.ingestion.BatchIngestionConfig;
 import org.apache.pinot.spi.config.table.ingestion.IngestionConfig;
-import org.apache.pinot.spi.utils.CommonConstants.Segment.SegmentType;
 import org.apache.pinot.spi.utils.builder.TableConfigBuilder;
 import org.testng.annotations.Test;
 
@@ -97,8 +96,8 @@ public class MergeRollupTaskGeneratorTest {
     assertTrue(pinotTaskConfigs.isEmpty());
 
     // Skip task generation, if REFRESH table
-    IngestionConfig ingestionConfig =
-        new IngestionConfig(new BatchIngestionConfig(null, "REFRESH", null), null, null, null, null);
+    IngestionConfig ingestionConfig = new IngestionConfig();
+    ingestionConfig.setBatchIngestionConfig(new BatchIngestionConfig(null, "REFRESH", null));
     offlineTableConfig = getOfflineTableConfig(new HashMap<>());
     offlineTableConfig.setIngestionConfig(ingestionConfig);
     pinotTaskConfigs = generator.generateTasks(Lists.newArrayList(offlineTableConfig));
@@ -108,8 +107,15 @@ public class MergeRollupTaskGeneratorTest {
   private void checkPinotTaskConfig(Map<String, String> pinotTaskConfig, String segments, String mergeLevel,
       String mergeType, String partitionBucketTimePeriod, String roundBucketTimePeriod,
       String maxNumRecordsPerSegments) {
-    assertEquals(pinotTaskConfig.get(MinionConstants.TABLE_NAME_KEY), OFFLINE_TABLE_NAME);
     assertEquals(pinotTaskConfig.get(MinionConstants.SEGMENT_NAME_KEY), segments);
+    checkPinotTaskConfig(pinotTaskConfig, mergeLevel, mergeType, partitionBucketTimePeriod, roundBucketTimePeriod,
+        maxNumRecordsPerSegments);
+  }
+
+  private void checkPinotTaskConfig(Map<String, String> pinotTaskConfig, String mergeLevel,
+      String mergeType, String partitionBucketTimePeriod, String roundBucketTimePeriod,
+      String maxNumRecordsPerSegments) {
+    assertEquals(pinotTaskConfig.get(MinionConstants.TABLE_NAME_KEY), OFFLINE_TABLE_NAME);
     assertTrue("true".equalsIgnoreCase(pinotTaskConfig.get(MinionConstants.ENABLE_REPLACE_SEGMENTS_KEY)));
     assertEquals(pinotTaskConfig.get(MinionConstants.MergeRollupTask.MERGE_LEVEL_KEY), mergeLevel);
     assertEquals(pinotTaskConfig.get(MinionConstants.MergeRollupTask.MERGE_TYPE_KEY), mergeType);
@@ -412,20 +418,20 @@ public class MergeRollupTaskGeneratorTest {
     String segmentName4 = "testTable__4";
     SegmentZKMetadata metadata1 =
         getSegmentZKMetadata(segmentName1, 86_400_000L, 90_000_000L, TimeUnit.MILLISECONDS, null);
-    metadata1.setPartitionMetadata(new SegmentPartitionMetadata(
-        Collections.singletonMap("memberId", new ColumnPartitionMetadata("murmur", 10, Collections.singleton(0)))));
+    metadata1.setPartitionMetadata(new SegmentPartitionMetadata(Collections.singletonMap("memberId",
+        new ColumnPartitionMetadata("murmur", 10, Collections.singleton(0), null))));
     SegmentZKMetadata metadata2 =
         getSegmentZKMetadata(segmentName2, 86_400_000L, 100_000_000L, TimeUnit.MILLISECONDS, null);
-    metadata2.setPartitionMetadata(new SegmentPartitionMetadata(
-        Collections.singletonMap("memberId", new ColumnPartitionMetadata("murmur", 10, Collections.singleton(0)))));
+    metadata2.setPartitionMetadata(new SegmentPartitionMetadata(Collections.singletonMap("memberId",
+        new ColumnPartitionMetadata("murmur", 10, Collections.singleton(0), null))));
     SegmentZKMetadata metadata3 =
         getSegmentZKMetadata(segmentName3, 86_400_000L, 110_000_000L, TimeUnit.MILLISECONDS, null);
-    metadata3.setPartitionMetadata(new SegmentPartitionMetadata(
-        Collections.singletonMap("memberId", new ColumnPartitionMetadata("murmur", 10, Collections.singleton(1)))));
+    metadata3.setPartitionMetadata(new SegmentPartitionMetadata(Collections.singletonMap("memberId",
+        new ColumnPartitionMetadata("murmur", 10, Collections.singleton(1), null))));
     SegmentZKMetadata metadata4 =
         getSegmentZKMetadata(segmentName4, 90_000_000L, 110_000_000L, TimeUnit.MILLISECONDS, null);
-    metadata4.setPartitionMetadata(new SegmentPartitionMetadata(
-        Collections.singletonMap("memberId", new ColumnPartitionMetadata("murmur", 10, Collections.singleton(1)))));
+    metadata4.setPartitionMetadata(new SegmentPartitionMetadata(Collections.singletonMap("memberId",
+        new ColumnPartitionMetadata("murmur", 10, Collections.singleton(1), null))));
     ClusterInfoAccessor mockClusterInfoProvide = mock(ClusterInfoAccessor.class);
     when(mockClusterInfoProvide.getSegmentsZKMetadata(OFFLINE_TABLE_NAME))
         .thenReturn(Lists.newArrayList(metadata1, metadata2, metadata3, metadata4));
@@ -434,10 +440,24 @@ public class MergeRollupTaskGeneratorTest {
     generator.init(mockClusterInfoProvide);
     List<PinotTaskConfig> pinotTaskConfigs = generator.generateTasks(Lists.newArrayList(offlineTableConfig));
     assertEquals(pinotTaskConfigs.size(), 2);
-    checkPinotTaskConfig(pinotTaskConfigs.get(0).getConfigs(), segmentName1 + "," + segmentName2, DAILY, "concat", "1d",
-        null, "1000000");
-    checkPinotTaskConfig(pinotTaskConfigs.get(1).getConfigs(), segmentName3 + "," + segmentName4, DAILY, "concat", "1d",
-        null, "1000000");
+
+    String partitionedSegmentsGroup1 = segmentName1 + "," + segmentName2;
+    String partitionedSegmentsGroup2 = segmentName3 + "," + segmentName4;
+    boolean isPartitionedSegmentsGroup1Seen = false;
+    boolean isPartitionedSegmentsGroup2Seen = false;
+    for (PinotTaskConfig pinotTaskConfig : pinotTaskConfigs) {
+      if (!isPartitionedSegmentsGroup1Seen) {
+        isPartitionedSegmentsGroup1Seen =
+            pinotTaskConfig.getConfigs().get(MinionConstants.SEGMENT_NAME_KEY).equals(partitionedSegmentsGroup1);
+      }
+      if (!isPartitionedSegmentsGroup2Seen) {
+        isPartitionedSegmentsGroup2Seen =
+            pinotTaskConfig.getConfigs().get(MinionConstants.SEGMENT_NAME_KEY).equals(partitionedSegmentsGroup2);
+      }
+      assertTrue(isPartitionedSegmentsGroup1Seen || isPartitionedSegmentsGroup2Seen);
+      checkPinotTaskConfig(pinotTaskConfigs.get(0).getConfigs(), DAILY, "concat", "1d", null, "1000000");
+    }
+    assertTrue(isPartitionedSegmentsGroup1Seen && isPartitionedSegmentsGroup2Seen);
 
     // With numMaxRecordsPerTask constraints
     tableTaskConfigs.put("daily.maxNumRecordsPerTask", "5000000");
@@ -448,10 +468,27 @@ public class MergeRollupTaskGeneratorTest {
 
     pinotTaskConfigs = generator.generateTasks(Lists.newArrayList(offlineTableConfig));
     assertEquals(pinotTaskConfigs.size(), 3);
-    checkPinotTaskConfig(pinotTaskConfigs.get(0).getConfigs(), segmentName1 + "," + segmentName2, DAILY, "concat", "1d",
-        null, "1000000");
-    checkPinotTaskConfig(pinotTaskConfigs.get(1).getConfigs(), segmentName3, DAILY, "concat", "1d", null, "1000000");
-    checkPinotTaskConfig(pinotTaskConfigs.get(2).getConfigs(), segmentName4, DAILY, "concat", "1d", null, "1000000");
+
+    isPartitionedSegmentsGroup1Seen = false;
+    isPartitionedSegmentsGroup2Seen = false;
+    boolean isPartitionedSegmentsGroup3Seen = false;
+    for (PinotTaskConfig pinotTaskConfig : pinotTaskConfigs) {
+      if (!isPartitionedSegmentsGroup1Seen) {
+        isPartitionedSegmentsGroup1Seen =
+            pinotTaskConfig.getConfigs().get(MinionConstants.SEGMENT_NAME_KEY).equals(partitionedSegmentsGroup1);
+      }
+      if (!isPartitionedSegmentsGroup2Seen) {
+        isPartitionedSegmentsGroup2Seen =
+            pinotTaskConfig.getConfigs().get(MinionConstants.SEGMENT_NAME_KEY).equals(segmentName3);
+      }
+      if (!isPartitionedSegmentsGroup3Seen) {
+        isPartitionedSegmentsGroup3Seen =
+            pinotTaskConfig.getConfigs().get(MinionConstants.SEGMENT_NAME_KEY).equals(segmentName4);
+      }
+      assertTrue(isPartitionedSegmentsGroup1Seen || isPartitionedSegmentsGroup2Seen || isPartitionedSegmentsGroup3Seen);
+      checkPinotTaskConfig(pinotTaskConfigs.get(1).getConfigs(), DAILY, "concat", "1d", null, "1000000");
+    }
+    assertTrue(isPartitionedSegmentsGroup1Seen && isPartitionedSegmentsGroup2Seen && isPartitionedSegmentsGroup3Seen);
   }
 
   /**
@@ -766,7 +803,6 @@ public class MergeRollupTaskGeneratorTest {
   private SegmentZKMetadata getSegmentZKMetadata(String segmentName, long startTime, long endTime, TimeUnit timeUnit,
       String downloadURL) {
     SegmentZKMetadata segmentZKMetadata = new SegmentZKMetadata(segmentName);
-    segmentZKMetadata.setSegmentType(SegmentType.OFFLINE);
     segmentZKMetadata.setStartTime(startTime);
     segmentZKMetadata.setEndTime(endTime);
     segmentZKMetadata.setTimeUnit(timeUnit);
