@@ -23,9 +23,11 @@ import com.google.common.base.Preconditions;
 import com.google.common.collect.Iterables;
 import java.io.File;
 import java.io.IOException;
+import java.math.BigDecimal;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
@@ -33,29 +35,19 @@ import org.apache.commons.configuration.ConfigurationException;
 import org.apache.commons.configuration.PropertiesConfiguration;
 import org.apache.pinot.common.utils.FileUtils;
 import org.apache.pinot.segment.local.io.util.PinotDataBitSet;
-import org.apache.pinot.segment.local.io.writer.impl.BaseChunkSVForwardIndexWriter;
-import org.apache.pinot.segment.local.segment.creator.impl.fwd.MultiValueFixedByteRawIndexCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.fwd.MultiValueUnsortedForwardIndexCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.fwd.MultiValueVarByteRawIndexCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.fwd.SingleValueFixedByteRawIndexCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.fwd.SingleValueSortedForwardIndexCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.fwd.SingleValueUnsortedForwardIndexCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.fwd.SingleValueVarByteRawIndexCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.inv.OffHeapBitmapInvertedIndexCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.inv.OnHeapBitmapInvertedIndexCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.inv.geospatial.OffHeapH3IndexCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.inv.geospatial.OnHeapH3IndexCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.inv.json.OffHeapJsonIndexCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.inv.json.OnHeapJsonIndexCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.inv.text.LuceneFSTIndexCreator;
 import org.apache.pinot.segment.local.segment.creator.impl.nullvalue.NullValueVectorCreator;
-import org.apache.pinot.segment.local.segment.creator.impl.text.LuceneTextIndexCreator;
+import org.apache.pinot.segment.local.segment.store.TextIndexUtils;
 import org.apache.pinot.segment.local.utils.GeometrySerializer;
 import org.apache.pinot.segment.spi.V1Constants;
 import org.apache.pinot.segment.spi.compression.ChunkCompressionType;
 import org.apache.pinot.segment.spi.creator.ColumnIndexCreationInfo;
+import org.apache.pinot.segment.spi.creator.IndexCreationContext;
+import org.apache.pinot.segment.spi.creator.IndexCreatorProvider;
 import org.apache.pinot.segment.spi.creator.SegmentCreator;
 import org.apache.pinot.segment.spi.creator.SegmentGeneratorConfig;
+import org.apache.pinot.segment.spi.index.IndexingOverrides;
+import org.apache.pinot.segment.spi.index.creator.BloomFilterCreator;
+import org.apache.pinot.segment.spi.index.creator.CombinedInvertedIndexCreator;
 import org.apache.pinot.segment.spi.index.creator.DictionaryBasedInvertedIndexCreator;
 import org.apache.pinot.segment.spi.index.creator.ForwardIndexCreator;
 import org.apache.pinot.segment.spi.index.creator.GeoSpatialIndexCreator;
@@ -63,10 +55,12 @@ import org.apache.pinot.segment.spi.index.creator.H3IndexConfig;
 import org.apache.pinot.segment.spi.index.creator.JsonIndexCreator;
 import org.apache.pinot.segment.spi.index.creator.SegmentIndexCreationInfo;
 import org.apache.pinot.segment.spi.index.creator.TextIndexCreator;
-import org.apache.pinot.segment.spi.index.creator.TextIndexType;
-import org.apache.pinot.segment.spi.index.reader.H3IndexResolution;
 import org.apache.pinot.segment.spi.partition.PartitionFunction;
+import org.apache.pinot.spi.config.table.BloomFilterConfig;
+import org.apache.pinot.spi.config.table.FSTType;
 import org.apache.pinot.spi.config.table.FieldConfig;
+import org.apache.pinot.spi.config.table.IndexingConfig;
+import org.apache.pinot.spi.config.table.SegmentZKPropsConfig;
 import org.apache.pinot.spi.data.DateTimeFieldSpec;
 import org.apache.pinot.spi.data.DateTimeFormatSpec;
 import org.apache.pinot.spi.data.FieldSpec;
@@ -85,7 +79,6 @@ import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.apache.pinot.segment.spi.V1Constants.MetadataKeys.Column.*;
 import static org.apache.pinot.segment.spi.V1Constants.MetadataKeys.Segment.*;
 
-
 /**
  * Segment creator which writes data in a columnar form.
  */
@@ -98,9 +91,12 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
 
   private SegmentGeneratorConfig _config;
   private Map<String, ColumnIndexCreationInfo> _indexCreationInfoMap;
+  private final IndexCreatorProvider _indexCreatorProvider = IndexingOverrides.getIndexCreatorProvider();
   private final Map<String, SegmentDictionaryCreator> _dictionaryCreatorMap = new HashMap<>();
   private final Map<String, ForwardIndexCreator> _forwardIndexCreatorMap = new HashMap<>();
   private final Map<String, DictionaryBasedInvertedIndexCreator> _invertedIndexCreatorMap = new HashMap<>();
+  private final Map<String, BloomFilterCreator> _bloomFilterCreatorMap = new HashMap<>();
+  private final Map<String, CombinedInvertedIndexCreator> _rangeIndexFilterCreatorMap = new HashMap<>();
   private final Map<String, TextIndexCreator> _textIndexCreatorMap = new HashMap<>();
   private final Map<String, TextIndexCreator> _fstIndexCreatorMap = new HashMap<>();
   private final Map<String, JsonIndexCreator> _jsonIndexCreatorMap = new HashMap<>();
@@ -143,6 +139,20 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
       invertedIndexColumns.add(columnName);
     }
 
+    Set<String> bloomFilterColumns = new HashSet<>();
+    for (String columnName : _config.getBloomFilterCreationColumns()) {
+      Preconditions.checkState(schema.hasColumn(columnName),
+          "Cannot create bloom filter for column: %s because it is not in schema", columnName);
+      bloomFilterColumns.add(columnName);
+    }
+
+    Set<String> rangeIndexColumns = new HashSet<>();
+    for (String columnName : _config.getRangeIndexCreationColumns()) {
+      Preconditions.checkState(schema.hasColumn(columnName),
+          "Cannot create range index for column: %s because it is not in schema", columnName);
+      rangeIndexColumns.add(columnName);
+    }
+
     Set<String> textIndexColumns = new HashSet<>();
     for (String columnName : _config.getTextIndexCreationColumns()) {
       Preconditions.checkState(schema.hasColumn(columnName),
@@ -171,6 +181,8 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
     }
 
     // Initialize creators for dictionary, forward index and inverted index
+    IndexingConfig indexingConfig = _config.getTableConfig().getIndexingConfig();
+    int rangeIndexVersion = indexingConfig.getRangeIndexVersion();
     for (FieldSpec fieldSpec : fieldSpecs) {
       // Ignore virtual columns
       if (fieldSpec.isVirtualColumn()) {
@@ -178,119 +190,102 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
       }
 
       String columnName = fieldSpec.getName();
-      DataType storedType = fieldSpec.getDataType().getStoredType();
-      ColumnIndexCreationInfo indexCreationInfo = indexCreationInfoMap.get(columnName);
-      Preconditions.checkNotNull(indexCreationInfo, "Missing index creation info for column: %s", columnName);
-      boolean dictEnabledColumn = createDictionaryForColumn(indexCreationInfo, segmentCreationSpec, fieldSpec);
+      ColumnIndexCreationInfo columnIndexCreationInfo = indexCreationInfoMap.get(columnName);
+      Preconditions.checkNotNull(columnIndexCreationInfo, "Missing index creation info for column: %s", columnName);
+      boolean dictEnabledColumn = createDictionaryForColumn(columnIndexCreationInfo, segmentCreationSpec, fieldSpec);
+      Preconditions.checkState(dictEnabledColumn || !invertedIndexColumns.contains(columnName),
+          "Cannot create inverted index for raw index column: %s", columnName);
 
+      IndexCreationContext.Common context = IndexCreationContext.builder()
+          .withIndexDir(_indexDir)
+          .withCardinality(columnIndexCreationInfo.getDistinctValueCount())
+          .withDictionary(dictEnabledColumn)
+          .withFieldSpec(fieldSpec)
+          .withTotalDocs(segmentIndexCreationInfo.getTotalDocs())
+          .withMinValue((Comparable<?>) columnIndexCreationInfo.getMin())
+          .withMaxValue((Comparable<?>) columnIndexCreationInfo.getMax())
+          .withTotalNumberOfEntries(columnIndexCreationInfo.getTotalNumberOfEntries())
+          .withColumnIndexCreationInfo(columnIndexCreationInfo)
+          .sorted(columnIndexCreationInfo.isSorted())
+          .onHeap(segmentCreationSpec.isOnHeap())
+          .build();
+      // Initialize forward index creator
+      ChunkCompressionType chunkCompressionType =
+          dictEnabledColumn ? null : getColumnCompressionType(segmentCreationSpec, fieldSpec);
+      _forwardIndexCreatorMap.put(columnName, _indexCreatorProvider.newForwardIndexCreator(
+          context.forForwardIndex(chunkCompressionType, segmentCreationSpec.getColumnProperties())));
+
+      // Initialize inverted index creator; skip creating inverted index if sorted
+      if (invertedIndexColumns.contains(columnName) && !columnIndexCreationInfo.isSorted()) {
+        _invertedIndexCreatorMap.put(columnName,
+            _indexCreatorProvider.newInvertedIndexCreator(context.forInvertedIndex()));
+      }
       if (dictEnabledColumn) {
         // Create dictionary-encoded index
-
         // Initialize dictionary creator
+        // TODO: Dictionary creator holds all unique values on heap. Consider keeping dictionary instead of creator
+        //       which uses off-heap memory.
         SegmentDictionaryCreator dictionaryCreator =
-            new SegmentDictionaryCreator(indexCreationInfo.getSortedUniqueElementsArray(), fieldSpec, _indexDir,
-                indexCreationInfo.isUseVarLengthDictionary());
+            new SegmentDictionaryCreator(fieldSpec, _indexDir, columnIndexCreationInfo.isUseVarLengthDictionary());
         _dictionaryCreatorMap.put(columnName, dictionaryCreator);
-
         // Create dictionary
         try {
-          dictionaryCreator.build();
+          dictionaryCreator.build(columnIndexCreationInfo.getSortedUniqueElementsArray());
         } catch (Exception e) {
           LOGGER.error("Error building dictionary for field: {}, cardinality: {}, number of bytes per entry: {}",
-              fieldSpec.getName(), indexCreationInfo.getDistinctValueCount(), dictionaryCreator.getNumBytesPerEntry());
+              fieldSpec.getName(), columnIndexCreationInfo.getDistinctValueCount(),
+              dictionaryCreator.getNumBytesPerEntry());
           throw e;
         }
+      }
 
-        // Initialize forward index creator
-        int cardinality = indexCreationInfo.getDistinctValueCount();
-        if (fieldSpec.isSingleValueField()) {
-          if (indexCreationInfo.isSorted()) {
-            _forwardIndexCreatorMap.put(columnName,
-                new SingleValueSortedForwardIndexCreator(_indexDir, columnName, cardinality));
-          } else {
-            _forwardIndexCreatorMap.put(columnName,
-                new SingleValueUnsortedForwardIndexCreator(_indexDir, columnName, cardinality, _totalDocs));
-          }
+      if (bloomFilterColumns.contains(columnName)) {
+        if (indexingConfig.getBloomFilterConfigs() != null
+            && indexingConfig.getBloomFilterConfigs().containsKey(columnName)) {
+          _bloomFilterCreatorMap.put(columnName, _indexCreatorProvider.newBloomFilterCreator(
+              context.forBloomFilter(indexingConfig.getBloomFilterConfigs().get(columnName))));
         } else {
-          _forwardIndexCreatorMap.put(columnName,
-              new MultiValueUnsortedForwardIndexCreator(_indexDir, columnName, cardinality, _totalDocs,
-                  indexCreationInfo.getTotalNumberOfEntries()));
+          _bloomFilterCreatorMap.put(columnName, _indexCreatorProvider.newBloomFilterCreator(
+              context.forBloomFilter(new BloomFilterConfig(BloomFilterConfig.DEFAULT_FPP, 0, false))));
         }
+      }
 
-        // Initialize inverted index creator; skip creating inverted index if sorted
-        if (invertedIndexColumns.contains(columnName) && !indexCreationInfo.isSorted()) {
-          if (segmentCreationSpec.isOnHeap()) {
-            _invertedIndexCreatorMap.put(columnName,
-                new OnHeapBitmapInvertedIndexCreator(_indexDir, columnName, cardinality));
-          } else {
-            _invertedIndexCreatorMap.put(columnName,
-                new OffHeapBitmapInvertedIndexCreator(_indexDir, fieldSpec, cardinality, _totalDocs,
-                    indexCreationInfo.getTotalNumberOfEntries()));
-          }
-        }
-      } else {
-        // Create raw index
-        Preconditions.checkState(!invertedIndexColumns.contains(columnName),
-            "Cannot create inverted index for raw index column: %s", columnName);
-
-        ChunkCompressionType compressionType = getColumnCompressionType(segmentCreationSpec, fieldSpec);
-
-        // Initialize forward index creator
-        boolean deriveNumDocsPerChunk =
-            shouldDeriveNumDocsPerChunk(columnName, segmentCreationSpec.getColumnProperties());
-        int writerVersion = rawIndexWriterVersion(columnName, segmentCreationSpec.getColumnProperties());
-        if (fieldSpec.isSingleValueField()) {
-          _forwardIndexCreatorMap.put(columnName,
-              getRawIndexCreatorForSVColumn(_indexDir, compressionType, columnName, storedType, _totalDocs,
-                  indexCreationInfo.getLengthOfLongestEntry(), deriveNumDocsPerChunk, writerVersion));
-        } else {
-          _forwardIndexCreatorMap.put(columnName,
-              getRawIndexCreatorForMVColumn(_indexDir, compressionType, columnName, storedType, _totalDocs,
-                  indexCreationInfo.getMaxNumberOfMultiValueElements(), deriveNumDocsPerChunk, writerVersion,
-                  indexCreationInfo.getMaxRowLengthInBytes()));
-        }
+      if (!columnIndexCreationInfo.isSorted() && rangeIndexColumns.contains(columnName)) {
+        _rangeIndexFilterCreatorMap.put(columnName,
+            _indexCreatorProvider.newRangeIndexCreator(context.forRangeIndex(rangeIndexVersion)));
       }
 
       if (textIndexColumns.contains(columnName)) {
-        // Initialize text index creator
-        Preconditions.checkState(storedType == DataType.STRING,
-            "Text index is currently only supported on STRING type columns");
+        FSTType fstType = FSTType.LUCENE;
+        List<FieldConfig> fieldConfigList = _config.getTableConfig().getFieldConfigList();
+        if (fieldConfigList != null) {
+          for (FieldConfig fieldConfig : fieldConfigList) {
+            if (fieldConfig.getName().equals(columnName)) {
+              Map<String, String> properties = fieldConfig.getProperties();
+              if (TextIndexUtils.isFstTypeNative(properties)) {
+                fstType = FSTType.NATIVE;
+              }
+            }
+          }
+        }
         _textIndexCreatorMap.put(columnName,
-            new LuceneTextIndexCreator(columnName, _indexDir, true /* commitOnClose */));
+            _indexCreatorProvider.newTextIndexCreator(context.forTextIndex(fstType, true)));
       }
 
       if (fstIndexColumns.contains(columnName)) {
-        Preconditions.checkState(fieldSpec.isSingleValueField(),
-            "FST index is currently only supported on single-value columns");
-        Preconditions.checkState(storedType == DataType.STRING,
-            "FST index is currently only supported on STRING type columns");
-        Preconditions.checkState(dictEnabledColumn,
-            "FST index is currently only supported on dictionary-encoded columns");
-        _fstIndexCreatorMap.put(columnName, new LuceneFSTIndexCreator(_indexDir, columnName,
-            (String[]) indexCreationInfo.getSortedUniqueElementsArray()));
+        _fstIndexCreatorMap.put(columnName, _indexCreatorProvider.newTextIndexCreator(
+            context.forFSTIndex(_config.getFSTIndexType(),
+                (String[]) columnIndexCreationInfo.getSortedUniqueElementsArray())));
       }
 
       if (jsonIndexColumns.contains(columnName)) {
-        Preconditions.checkState(fieldSpec.isSingleValueField(),
-            "Json index is currently only supported on single-value columns");
-        Preconditions.checkState(storedType == DataType.STRING,
-            "Json index is currently only supported on STRING columns");
-        JsonIndexCreator jsonIndexCreator =
-            segmentCreationSpec.isOnHeap() ? new OnHeapJsonIndexCreator(_indexDir, columnName)
-                : new OffHeapJsonIndexCreator(_indexDir, columnName);
-        _jsonIndexCreatorMap.put(columnName, jsonIndexCreator);
+        _jsonIndexCreatorMap.put(columnName, _indexCreatorProvider.newJsonIndexCreator(context.forJsonIndex()));
       }
 
       H3IndexConfig h3IndexConfig = h3IndexConfigs.get(columnName);
       if (h3IndexConfig != null) {
-        Preconditions.checkState(fieldSpec.isSingleValueField(),
-            "H3 index is currently only supported on single-value columns");
-        Preconditions.checkState(storedType == DataType.BYTES, "H3 index is currently only supported on BYTES columns");
-        H3IndexResolution resolution = h3IndexConfig.getResolution();
-        GeoSpatialIndexCreator h3IndexCreator =
-            segmentCreationSpec.isOnHeap() ? new OnHeapH3IndexCreator(_indexDir, columnName, resolution)
-                : new OffHeapH3IndexCreator(_indexDir, columnName, resolution);
-        _h3IndexCreatorMap.put(columnName, h3IndexCreator);
+        _h3IndexCreatorMap.put(columnName,
+            _indexCreatorProvider.newGeoSpatialIndexCreator(context.forGeospatialIndex(h3IndexConfig)));
       }
 
       _nullHandlingEnabled = _config.isNullHandlingEnabled();
@@ -301,54 +296,6 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
     }
   }
 
-  public static boolean shouldDeriveNumDocsPerChunk(String columnName,
-      Map<String, Map<String, String>> columnProperties) {
-    if (columnProperties != null) {
-      Map<String, String> properties = columnProperties.get(columnName);
-      return properties != null && Boolean.parseBoolean(
-          properties.get(FieldConfig.DERIVE_NUM_DOCS_PER_CHUNK_RAW_INDEX_KEY));
-    }
-    return false;
-  }
-
-  public static int rawIndexWriterVersion(String columnName, Map<String, Map<String, String>> columnProperties) {
-    if (columnProperties != null && columnProperties.get(columnName) != null) {
-      Map<String, String> properties = columnProperties.get(columnName);
-      String version = properties.get(FieldConfig.RAW_INDEX_WRITER_VERSION);
-      if (version == null) {
-        return BaseChunkSVForwardIndexWriter.DEFAULT_VERSION;
-      }
-      return Integer.parseInt(version);
-    }
-    return BaseChunkSVForwardIndexWriter.DEFAULT_VERSION;
-  }
-
-  /**
-   * Helper method that returns compression type to use based on segment creation spec and field type.
-   * <ul>
-   *   <li> Returns compression type from segment creation spec, if specified there.</li>
-   *   <li> Else, returns PASS_THROUGH for metrics, and SNAPPY for dimensions. This is because metrics are likely
-   *        to be spread in different chunks after applying predicates. Same could be true for dimensions, but in that
-   *        case, clients are expected to explicitly specify the appropriate compression type in the spec. </li>
-   * </ul>
-   * @param segmentCreationSpec Segment creation spec
-   * @param fieldSpec Field spec for the column
-   * @return Compression type to use
-   */
-  private ChunkCompressionType getColumnCompressionType(SegmentGeneratorConfig segmentCreationSpec,
-      FieldSpec fieldSpec) {
-    ChunkCompressionType compressionType = segmentCreationSpec.getRawIndexCompressionType().get(fieldSpec.getName());
-    if (compressionType == null) {
-      if (fieldSpec.getFieldType() == FieldType.METRIC) {
-        return ChunkCompressionType.PASS_THROUGH;
-      } else {
-        return ChunkCompressionType.SNAPPY;
-      }
-    } else {
-      return compressionType;
-    }
-  }
-
   /**
    * Returns true if dictionary should be created for a column, false otherwise.
    * Currently there are two sources for this config:
@@ -371,9 +318,55 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
         .containsKey(column)) {
       return false;
     }
+
+    // Do not create dictionary if index size with dictionary is going to be larger than index size without dictionary
+    // This is done to reduce the cost of dictionary for high cardinality columns
+    // Off by default and needs optimizeDictionaryEnabled to be set to true
+    if (config.isOptimizeDictionaryForMetrics() && spec.getFieldType() == FieldType.METRIC && spec.isSingleValueField()
+        && spec.getDataType().isFixedWidth()) {
+      long dictionarySize = info.getDistinctValueCount() * spec.getDataType().size();
+      long forwardIndexSize =
+          ((long) info.getTotalNumberOfEntries() * PinotDataBitSet.getNumBitsPerValue(info.getDistinctValueCount() - 1)
+              + Byte.SIZE - 1) / Byte.SIZE;
+
+      double indexWithDictSize = dictionarySize + forwardIndexSize;
+      double indexWithoutDictSize = info.getTotalNumberOfEntries() * spec.getDataType().size();
+
+      double indexSizeRatio = indexWithoutDictSize / indexWithDictSize;
+      if (indexSizeRatio <= config.getNoDictionarySizeRatioThreshold()) {
+        return false;
+      }
+    }
+
     return info.isCreateDictionary();
   }
 
+  /**
+   * Helper method that returns compression type to use based on segment creation spec and field type.
+   * <ul>
+   *   <li> Returns compression type from segment creation spec, if specified there.</li>
+   *   <li> Else, returns PASS_THROUGH for metrics, and SNAPPY for dimensions. This is because metrics are likely
+   *        to be spread in different chunks after applying predicates. Same could be true for dimensions, but in that
+   *        case, clients are expected to explicitly specify the appropriate compression type in the spec. </li>
+   * </ul>
+   * @param segmentCreationSpec Segment creation spec
+   * @param fieldSpec Field spec for the column
+   * @return Compression type to use
+   */
+  private ChunkCompressionType getColumnCompressionType(SegmentGeneratorConfig segmentCreationSpec,
+      FieldSpec fieldSpec) {
+    ChunkCompressionType compressionType = segmentCreationSpec.getRawIndexCompressionType().get(fieldSpec.getName());
+    if (compressionType == null) {
+      if (fieldSpec.getFieldType() == FieldSpec.FieldType.METRIC) {
+        return ChunkCompressionType.PASS_THROUGH;
+      } else {
+        return ChunkCompressionType.LZ4;
+      }
+    } else {
+      return compressionType;
+    }
+  }
+
   @Override
   public void indexRow(GenericRow row)
       throws IOException {
@@ -391,6 +384,85 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
       //get dictionaryCreator, will be null if column is not dictionaryEncoded
       SegmentDictionaryCreator dictionaryCreator = _dictionaryCreatorMap.get(columnName);
 
+      // bloom filter
+      BloomFilterCreator bloomFilterCreator = _bloomFilterCreatorMap.get(columnName);
+      if (bloomFilterCreator != null) {
+        if (fieldSpec.isSingleValueField()) {
+          bloomFilterCreator.add(columnValueToIndex.toString());
+        } else {
+          Object[] values = (Object[]) columnValueToIndex;
+          for (Object value : values) {
+            bloomFilterCreator.add(value.toString());
+          }
+        }
+      }
+
+      // range index
+      CombinedInvertedIndexCreator combinedInvertedIndexCreator = _rangeIndexFilterCreatorMap.get(columnName);
+      if (combinedInvertedIndexCreator != null) {
+        if (dictionaryCreator != null) {
+          if (fieldSpec.isSingleValueField()) {
+            combinedInvertedIndexCreator.add(dictionaryCreator.indexOfSV(columnValueToIndex));
+          } else {
+            int[] dictIds = dictionaryCreator.indexOfMV(columnValueToIndex);
+            combinedInvertedIndexCreator.add(dictIds, dictIds.length);
+          }
+        } else {
+          if (fieldSpec.isSingleValueField()) {
+            switch (fieldSpec.getDataType()) {
+              case INT:
+                combinedInvertedIndexCreator.add((Integer) columnValueToIndex);
+                break;
+              case LONG:
+                combinedInvertedIndexCreator.add((Long) columnValueToIndex);
+                break;
+              case FLOAT:
+                combinedInvertedIndexCreator.add((Float) columnValueToIndex);
+                break;
+              case DOUBLE:
+                combinedInvertedIndexCreator.add((Double) columnValueToIndex);
+                break;
+              default:
+                throw new RuntimeException("Unsupported data type " + fieldSpec.getDataType() + " for range index");
+            }
+          } else {
+            Object[] values = (Object[]) columnValueToIndex;
+            switch (fieldSpec.getDataType()) {
+              case INT:
+                int[] intValues = new int[values.length];
+                for (int i = 0; i < values.length; i++) {
+                  intValues[i] = (Integer) values[i];
+                }
+                combinedInvertedIndexCreator.add(intValues, values.length);
+                break;
+              case LONG:
+                long[] longValues = new long[values.length];
+                for (int i = 0; i < values.length; i++) {
+                  longValues[i] = (Long) values[i];
+                }
+                combinedInvertedIndexCreator.add(longValues, values.length);
+                break;
+              case FLOAT:
+                float[] floatValues = new float[values.length];
+                for (int i = 0; i < values.length; i++) {
+                  floatValues[i] = (Float) values[i];
+                }
+                combinedInvertedIndexCreator.add(floatValues, values.length);
+                break;
+              case DOUBLE:
+                double[] doubleValues = new double[values.length];
+                for (int i = 0; i < values.length; i++) {
+                  doubleValues[i] = (Double) values[i];
+                }
+                combinedInvertedIndexCreator.add(doubleValues, values.length);
+                break;
+              default:
+                throw new RuntimeException("Unsupported data type " + fieldSpec.getDataType() + " for range index");
+            }
+          }
+        }
+      }
+
       // text-index
       TextIndexCreator textIndexCreator = _textIndexCreatorMap.get(columnName);
       if (textIndexCreator != null) {
@@ -458,12 +530,22 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
             case DOUBLE:
               forwardIndexCreator.putDouble((double) columnValueToIndex);
               break;
+            case BIG_DECIMAL:
+              forwardIndexCreator.putBigDecimal((BigDecimal) columnValueToIndex);
+              break;
             case STRING:
               forwardIndexCreator.putString((String) columnValueToIndex);
               break;
             case BYTES:
               forwardIndexCreator.putBytes((byte[]) columnValueToIndex);
               break;
+            case JSON:
+              if (columnValueToIndex instanceof String) {
+                forwardIndexCreator.putString((String) columnValueToIndex);
+              } else if (columnValueToIndex instanceof byte[]) {
+                forwardIndexCreator.putBytes((byte[]) columnValueToIndex);
+              }
+              break;
             default:
               throw new IllegalStateException();
           }
@@ -582,6 +664,9 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
   @Override
   public void seal()
       throws ConfigurationException, IOException {
+    for (SegmentDictionaryCreator dictionaryCreator : _dictionaryCreatorMap.values()) {
+      dictionaryCreator.postIndexingCleanup();
+    }
     for (DictionaryBasedInvertedIndexCreator invertedIndexCreator : _invertedIndexCreatorMap.values()) {
       invertedIndexCreator.seal();
     }
@@ -600,6 +685,12 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
     for (NullValueVectorCreator nullValueVectorCreator : _nullValueVectorCreatorMap.values()) {
       nullValueVectorCreator.seal();
     }
+    for (BloomFilterCreator bloomFilterCreator : _bloomFilterCreatorMap.values()) {
+      bloomFilterCreator.seal();
+    }
+    for (CombinedInvertedIndexCreator combinedInvertedIndexCreator : _rangeIndexFilterCreatorMap.values()) {
+      combinedInvertedIndexCreator.seal();
+    }
     writeMetadata();
   }
 
@@ -691,32 +782,14 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
       ColumnIndexCreationInfo columnIndexCreationInfo = entry.getValue();
       SegmentDictionaryCreator dictionaryCreator = _dictionaryCreatorMap.get(column);
       int dictionaryElementSize = (dictionaryCreator != null) ? dictionaryCreator.getNumBytesPerEntry() : 0;
-
-      // TODO: after fixing the server-side dependency on HAS_INVERTED_INDEX and deployed, set HAS_INVERTED_INDEX
-      //  properly
-      // The hasInvertedIndex flag in segment metadata is picked up in ColumnMetadata, and will be used during the query
-      // plan phase. If it is set to false, then inverted indexes are not used in queries even if they are created
-      // via table
-      // configs on segment load. So, we set it to true here for now, until we fix the server to update the value inside
-      // ColumnMetadata, export information to the query planner that the inverted index available is current and can
-      // be used.
-      //
-      //    boolean hasInvertedIndex = invertedIndexCreatorMap.containsKey();
-      boolean hasInvertedIndex = true;
-
-      // for new generated segment we write as NONE if text index does not exist
-      // for reading existing segments that don't have this property, non-existence
-      // of this property will be treated as NONE. See the builder in ColumnMetadata
-      TextIndexType textIndexType =
-          _textIndexCreatorMap.containsKey(column) ? TextIndexType.LUCENE : TextIndexType.NONE;
-
-      boolean hasFSTIndex = _fstIndexCreatorMap.containsKey(column);
-
-      boolean hasJsonIndex = _jsonIndexCreatorMap.containsKey(column);
-
       addColumnMetadataInfo(properties, column, columnIndexCreationInfo, _totalDocs, _schema.getFieldSpecFor(column),
-          _dictionaryCreatorMap.containsKey(column), dictionaryElementSize, hasInvertedIndex, textIndexType,
-          hasFSTIndex, hasJsonIndex);
+          dictionaryCreator != null, dictionaryElementSize);
+    }
+
+    SegmentZKPropsConfig segmentZKPropsConfig = _config.getSegmentZKPropsConfig();
+    if (segmentZKPropsConfig != null) {
+      properties.setProperty(Realtime.START_OFFSET, segmentZKPropsConfig.getStartOffset());
+      properties.setProperty(Realtime.END_OFFSET, segmentZKPropsConfig.getEndOffset());
     }
 
     properties.save();
@@ -724,8 +797,7 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
 
   public static void addColumnMetadataInfo(PropertiesConfiguration properties, String column,
       ColumnIndexCreationInfo columnIndexCreationInfo, int totalDocs, FieldSpec fieldSpec, boolean hasDictionary,
-      int dictionaryElementSize, boolean hasInvertedIndex, TextIndexType textIndexType, boolean hasFSTIndex,
-      boolean hasJsonIndex) {
+      int dictionaryElementSize) {
     int cardinality = columnIndexCreationInfo.getDistinctValueCount();
     properties.setProperty(getKeyFor(column, CARDINALITY), String.valueOf(cardinality));
     properties.setProperty(getKeyFor(column, TOTAL_DOCS), String.valueOf(totalDocs));
@@ -736,12 +808,7 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
     properties.setProperty(getKeyFor(column, DICTIONARY_ELEMENT_SIZE), String.valueOf(dictionaryElementSize));
     properties.setProperty(getKeyFor(column, COLUMN_TYPE), String.valueOf(fieldSpec.getFieldType()));
     properties.setProperty(getKeyFor(column, IS_SORTED), String.valueOf(columnIndexCreationInfo.isSorted()));
-    properties.setProperty(getKeyFor(column, HAS_NULL_VALUE), String.valueOf(columnIndexCreationInfo.hasNulls()));
     properties.setProperty(getKeyFor(column, HAS_DICTIONARY), String.valueOf(hasDictionary));
-    properties.setProperty(getKeyFor(column, TEXT_INDEX_TYPE), textIndexType.name());
-    properties.setProperty(getKeyFor(column, HAS_INVERTED_INDEX), String.valueOf(hasInvertedIndex));
-    properties.setProperty(getKeyFor(column, HAS_FST_INDEX), String.valueOf(hasFSTIndex));
-    properties.setProperty(getKeyFor(column, HAS_JSON_INDEX), String.valueOf(hasJsonIndex));
     properties.setProperty(getKeyFor(column, IS_SINGLE_VALUED), String.valueOf(fieldSpec.isSingleValueField()));
     properties.setProperty(getKeyFor(column, MAX_MULTI_VALUE_ELEMENTS),
         String.valueOf(columnIndexCreationInfo.getMaxNumberOfMultiValueElements()));
@@ -752,9 +819,15 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
 
     PartitionFunction partitionFunction = columnIndexCreationInfo.getPartitionFunction();
     if (partitionFunction != null) {
-      properties.setProperty(getKeyFor(column, PARTITION_FUNCTION), partitionFunction.toString());
+      properties.setProperty(getKeyFor(column, PARTITION_FUNCTION), partitionFunction.getName());
       properties.setProperty(getKeyFor(column, NUM_PARTITIONS), columnIndexCreationInfo.getNumPartitions());
       properties.setProperty(getKeyFor(column, PARTITION_VALUES), columnIndexCreationInfo.getPartitions());
+      if (columnIndexCreationInfo.getPartitionFunctionConfig() != null) {
+        for (Map.Entry<String, String> entry : columnIndexCreationInfo.getPartitionFunctionConfig().entrySet()) {
+          properties.setProperty(getKeyFor(column, String.format("%s.%s", PARTITION_FUNCTION_CONFIG, entry.getKey())),
+              entry.getValue());
+        }
+      }
     }
 
     // datetime field
@@ -817,76 +890,12 @@ public class SegmentColumnarIndexCreator implements SegmentCreator {
     properties.subset(COLUMN_PROPS_KEY_PREFIX + column).clear();
   }
 
-  /**
-   * Helper method to build the raw index creator for the column.
-   * Assumes that column to be indexed is single valued.
-   *
-   * @param file Output index file
-   * @param column Column name
-   * @param totalDocs Total number of documents to index
-   * @param lengthOfLongestEntry Length of longest entry
-   * @param deriveNumDocsPerChunk true if varbyte writer should auto-derive the number of rows per chunk
-   * @param writerVersion version to use for the raw index writer
-   * @return raw index creator
-   */
-  public static ForwardIndexCreator getRawIndexCreatorForSVColumn(File file, ChunkCompressionType compressionType,
-      String column, DataType dataType, int totalDocs, int lengthOfLongestEntry, boolean deriveNumDocsPerChunk,
-      int writerVersion)
-      throws IOException {
-    switch (dataType.getStoredType()) {
-      case INT:
-      case LONG:
-      case FLOAT:
-      case DOUBLE:
-        return new SingleValueFixedByteRawIndexCreator(file, compressionType, column, totalDocs, dataType,
-            writerVersion);
-      case STRING:
-      case BYTES:
-        return new SingleValueVarByteRawIndexCreator(file, compressionType, column, totalDocs, dataType,
-            lengthOfLongestEntry, deriveNumDocsPerChunk, writerVersion);
-      default:
-        throw new UnsupportedOperationException("Data type not supported for raw indexing: " + dataType);
-    }
-  }
-
-  /**
-   * Helper method to build the raw index creator for the column.
-   * Assumes that column to be indexed is single valued.
-   *
-   * @param file Output index file
-   * @param column Column name
-   * @param totalDocs Total number of documents to index
-   * @param deriveNumDocsPerChunk true if varbyte writer should auto-derive the number of rows
-   *     per chunk
-   * @param writerVersion version to use for the raw index writer
-   * @param maxRowLengthInBytes the length of the longest row in bytes
-   * @return raw index creator
-   */
-  public static ForwardIndexCreator getRawIndexCreatorForMVColumn(File file, ChunkCompressionType compressionType,
-      String column, DataType dataType, final int totalDocs, int maxNumberOfMultiValueElements,
-      boolean deriveNumDocsPerChunk, int writerVersion, int maxRowLengthInBytes)
-      throws IOException {
-    switch (dataType.getStoredType()) {
-      case INT:
-      case LONG:
-      case FLOAT:
-      case DOUBLE:
-        return new MultiValueFixedByteRawIndexCreator(file, compressionType, column, totalDocs, dataType,
-            maxNumberOfMultiValueElements, deriveNumDocsPerChunk, writerVersion);
-      case STRING:
-      case BYTES:
-        return new MultiValueVarByteRawIndexCreator(file, compressionType, column, totalDocs, dataType, writerVersion,
-            maxRowLengthInBytes, maxNumberOfMultiValueElements);
-      default:
-        throw new UnsupportedOperationException("Data type not supported for raw indexing: " + dataType);
-    }
-  }
-
   @Override
   public void close()
       throws IOException {
     FileUtils.close(Iterables.concat(_dictionaryCreatorMap.values(), _forwardIndexCreatorMap.values(),
         _invertedIndexCreatorMap.values(), _textIndexCreatorMap.values(), _fstIndexCreatorMap.values(),
-        _jsonIndexCreatorMap.values(), _h3IndexCreatorMap.values(), _nullValueVectorCreatorMap.values()));
+        _jsonIndexCreatorMap.values(), _h3IndexCreatorMap.values(), _nullValueVectorCreatorMap.values(),
+        _bloomFilterCreatorMap.values(), _rangeIndexFilterCreatorMap.values()));
   }
 }
