@@ -19,6 +19,7 @@
 package org.apache.pinot.segment.local.segment.index.loader;
 
 import com.google.common.annotations.VisibleForTesting;
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -29,15 +30,21 @@ import javax.annotation.Nullable;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.pinot.segment.local.segment.index.column.PhysicalColumnIndexContainer;
 import org.apache.pinot.segment.local.segment.index.loader.columnminmaxvalue.ColumnMinMaxValueGeneratorMode;
+import org.apache.pinot.segment.local.segment.store.TextIndexUtils;
+import org.apache.pinot.segment.spi.creator.SegmentGeneratorConfig;
 import org.apache.pinot.segment.spi.creator.SegmentVersion;
 import org.apache.pinot.segment.spi.index.creator.H3IndexConfig;
 import org.apache.pinot.segment.spi.loader.SegmentDirectoryLoaderRegistry;
 import org.apache.pinot.spi.config.instance.InstanceDataManagerConfig;
 import org.apache.pinot.spi.config.table.BloomFilterConfig;
+import org.apache.pinot.spi.config.table.FSTType;
 import org.apache.pinot.spi.config.table.FieldConfig;
 import org.apache.pinot.spi.config.table.IndexingConfig;
 import org.apache.pinot.spi.config.table.StarTreeIndexConfig;
 import org.apache.pinot.spi.config.table.TableConfig;
+import org.apache.pinot.spi.config.table.TimestampIndexGranularity;
+import org.apache.pinot.spi.config.table.ingestion.IngestionConfig;
+import org.apache.pinot.spi.config.table.ingestion.TransformConfig;
 import org.apache.pinot.spi.env.PinotConfiguration;
 import org.apache.pinot.spi.utils.CommonConstants;
 import org.apache.pinot.spi.utils.ReadMode;
@@ -57,6 +64,7 @@ public class IndexLoadingConfig {
   private int _rangeIndexVersion = IndexingConfig.DEFAULT_RANGE_INDEX_VERSION;
   private Set<String> _textIndexColumns = new HashSet<>();
   private Set<String> _fstIndexColumns = new HashSet<>();
+  private FSTType _fstIndexType = FSTType.LUCENE;
   private Set<String> _jsonIndexColumns = new HashSet<>();
   private Map<String, H3IndexConfig> _h3IndexConfigs = new HashMap<>();
   private Set<String> _noDictionaryColumns = new HashSet<>(); // TODO: replace this by _noDictionaryConfig.
@@ -107,7 +115,6 @@ public class IndexLoadingConfig {
     if (invertedIndexColumns != null) {
       _invertedIndexColumns.addAll(invertedIndexColumns);
     }
-    _rangeIndexVersion = indexingConfig.getRangeIndexVersion();
 
     List<String> jsonIndexColumns = indexingConfig.getJsonIndexColumns();
     if (jsonIndexColumns != null) {
@@ -119,6 +126,10 @@ public class IndexLoadingConfig {
       _rangeIndexColumns.addAll(rangeIndexColumns);
     }
 
+    _rangeIndexVersion = indexingConfig.getRangeIndexVersion();
+
+    _fstIndexType = indexingConfig.getFSTIndexType();
+
     List<String> bloomFilterColumns = indexingConfig.getBloomFilterColumns();
     if (bloomFilterColumns != null) {
       for (String bloomFilterColumn : bloomFilterColumns) {
@@ -146,6 +157,33 @@ public class IndexLoadingConfig {
     extractFSTIndexColumnsFromTableConfig(tableConfig);
     extractH3IndexConfigsFromTableConfig(tableConfig);
 
+    Map<String, List<TimestampIndexGranularity>> timestampIndexConfigs =
+        SegmentGeneratorConfig.extractTimestampIndexConfigsFromTableConfig(tableConfig);
+    if (!timestampIndexConfigs.isEmpty()) {
+      // Apply transform function and range index to the timestamp with granularity columns
+      IngestionConfig ingestionConfig = tableConfig.getIngestionConfig();
+      if (ingestionConfig == null) {
+        ingestionConfig = new IngestionConfig();
+        tableConfig.setIngestionConfig(ingestionConfig);
+      }
+      List<TransformConfig> transformConfigs = ingestionConfig.getTransformConfigs();
+      if (transformConfigs == null) {
+        transformConfigs = new ArrayList<>();
+        ingestionConfig.setTransformConfigs(transformConfigs);
+      }
+      for (Map.Entry<String, List<TimestampIndexGranularity>> entry : timestampIndexConfigs.entrySet()) {
+        String column = entry.getKey();
+        for (TimestampIndexGranularity granularity : entry.getValue()) {
+          String columnNameWithGranularity =
+              TimestampIndexGranularity.getColumnNameWithGranularity(column, granularity);
+          TransformConfig transformConfig = new TransformConfig(columnNameWithGranularity,
+              TimestampIndexGranularity.getTransformExpression(column, granularity));
+          transformConfigs.add(transformConfig);
+          _rangeIndexColumns.add(columnNameWithGranularity);
+        }
+      }
+    }
+
     Map<String, String> noDictionaryConfig = indexingConfig.getNoDictionaryConfig();
     if (noDictionaryConfig != null) {
       _noDictionaryConfig.putAll(noDictionaryConfig);
@@ -192,6 +230,10 @@ public class IndexLoadingConfig {
         String column = fieldConfig.getName();
         if (fieldConfig.getIndexType() == FieldConfig.IndexType.TEXT) {
           _textIndexColumns.add(column);
+          Map<String, String> propertiesMap = fieldConfig.getProperties();
+          if (TextIndexUtils.isFstTypeNative(propertiesMap)) {
+            _fstIndexType = FSTType.NATIVE;
+          }
         }
       }
     }
@@ -285,6 +327,10 @@ public class IndexLoadingConfig {
     return _rangeIndexVersion;
   }
 
+  public FSTType getFSTIndexType() {
+    return _fstIndexType;
+  }
+
   /**
    * Used in two places:
    * (1) In {@link PhysicalColumnIndexContainer} to create the index loading info for immutable segments
@@ -349,6 +395,11 @@ public class IndexLoadingConfig {
     _fstIndexColumns = fstIndexColumns;
   }
 
+  @VisibleForTesting
+  public void setFSTIndexType(FSTType fstType) {
+    _fstIndexType = fstType;
+  }
+
   @VisibleForTesting
   public void setJsonIndexColumns(Set<String> jsonIndexColumns) {
     _jsonIndexColumns = jsonIndexColumns;
