@@ -30,6 +30,7 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.Map;
 import java.util.Set;
+import java.util.function.Consumer;
 import org.apache.commons.configuration.PropertiesConfiguration;
 import org.apache.commons.io.FileUtils;
 import org.apache.pinot.segment.local.segment.creator.SegmentTestUtils;
@@ -49,6 +50,7 @@ import org.apache.pinot.segment.spi.store.ColumnIndexType;
 import org.apache.pinot.segment.spi.store.SegmentDirectory;
 import org.apache.pinot.segment.spi.store.SegmentDirectoryPaths;
 import org.apache.pinot.spi.config.table.BloomFilterConfig;
+import org.apache.pinot.spi.config.table.IndexingConfig;
 import org.apache.pinot.spi.config.table.TableConfig;
 import org.apache.pinot.spi.config.table.TableType;
 import org.apache.pinot.spi.config.table.ingestion.IngestionConfig;
@@ -248,17 +250,16 @@ public class SegmentPreProcessorTest {
     _indexLoadingConfig.setFSTIndexColumns(fstColumns);
     _indexLoadingConfig.getNoDictionaryColumns().add(EXISTING_STRING_COL_RAW);
     constructV3Segment();
-    SegmentDirectory segmentDirectory =
-        SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-            .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+    SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
     SegmentPreProcessor v3Processor =
         new SegmentPreProcessor(segmentDirectory, _indexLoadingConfig, _newColumnsSchemaWithFST);
     expectThrows(UnsupportedOperationException.class, () -> v3Processor.process());
 
     constructV1Segment();
-    segmentDirectory =
-        SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-            .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+    segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader().load(_indexDir.toURI(),
+        new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
     SegmentPreProcessor v1Processor =
         new SegmentPreProcessor(segmentDirectory, _indexLoadingConfig, _newColumnsSchemaWithFST);
     expectThrows(UnsupportedOperationException.class, () -> v1Processor.process());
@@ -421,7 +422,8 @@ public class SegmentPreProcessorTest {
       throws Exception {
 
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, _indexLoadingConfig, schema)) {
       processor.process();
       SegmentMetadataImpl segmentMetadata = new SegmentMetadataImpl(_indexDir);
@@ -438,7 +440,8 @@ public class SegmentPreProcessorTest {
       assertEquals(columnMetadata.isAutoGenerated(), isAutoGenerated);
 
       try (SegmentDirectory segmentDirectory1 = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-          .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+          .load(_indexDir.toURI(),
+              new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
           SegmentDirectory.Reader reader = segmentDirectory1.createReader()) {
         assertTrue(reader.hasIndexFor(column, indexType));
         assertTrue(reader.hasIndexFor(column, ColumnIndexType.FORWARD_INDEX));
@@ -525,7 +528,8 @@ public class SegmentPreProcessorTest {
     checkInvertedIndexCreation(false);
     long addedLength = 0L;
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentDirectory.Reader reader = segmentDirectory.createReader()) {
       // 8 bytes overhead is for checking integrity of the segment.
       addedLength += reader.getIndexFor(COLUMN1_NAME, ColumnIndexType.INVERTED_INDEX).size() + 8;
@@ -548,7 +552,8 @@ public class SegmentPreProcessorTest {
   private void checkInvertedIndexCreation(boolean reCreate)
       throws Exception {
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentDirectory.Reader reader = segmentDirectory.createReader()) {
       if (reCreate) {
         assertTrue(reader.hasIndexFor(COLUMN1_NAME, ColumnIndexType.INVERTED_INDEX));
@@ -564,13 +569,15 @@ public class SegmentPreProcessorTest {
     }
 
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, _indexLoadingConfig, null)) {
       processor.process();
     }
 
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentDirectory.Reader reader = segmentDirectory.createReader()) {
       assertTrue(reader.hasIndexFor(COLUMN1_NAME, ColumnIndexType.INVERTED_INDEX));
       assertTrue(reader.hasIndexFor(COLUMN13_NAME, ColumnIndexType.INVERTED_INDEX));
@@ -583,8 +590,10 @@ public class SegmentPreProcessorTest {
   public void testV1UpdateDefaultColumns()
       throws Exception {
     constructV1Segment();
-    _tableConfig.setIngestionConfig(new IngestionConfig(null, null, null,
-        Collections.singletonList(new TransformConfig(NEW_INT_SV_DIMENSION_COLUMN_NAME, "plus(column1, 1)")), null));
+    IngestionConfig ingestionConfig = new IngestionConfig();
+    ingestionConfig.setTransformConfigs(
+        Collections.singletonList(new TransformConfig(NEW_INT_SV_DIMENSION_COLUMN_NAME, "plus(column1, 1)")));
+    _tableConfig.setIngestionConfig(ingestionConfig);
     _indexLoadingConfig.getInvertedIndexColumns().add(NEW_COLUMN_INVERTED_INDEX);
     checkUpdateDefaultColumns();
 
@@ -596,7 +605,8 @@ public class SegmentPreProcessorTest {
     SegmentMetadataImpl segmentMetadata = new SegmentMetadataImpl(_indexDir);
     assertNotNull(segmentMetadata.getColumnMetadataFor(NEW_INT_SV_DIMENSION_COLUMN_NAME));
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, _indexLoadingConfig,
             _newColumnsSchema3)) {
       processor.process();
@@ -627,8 +637,10 @@ public class SegmentPreProcessorTest {
     SegmentMetadataImpl segmentMetadata = new SegmentMetadataImpl(_indexDir);
     assertEquals(segmentMetadata.getVersion(), SegmentVersion.v3);
 
-    _tableConfig.setIngestionConfig(new IngestionConfig(null, null, null,
-        Collections.singletonList(new TransformConfig(NEW_INT_SV_DIMENSION_COLUMN_NAME, "plus(column1, 1)")), null));
+    IngestionConfig ingestionConfig = new IngestionConfig();
+    ingestionConfig.setTransformConfigs(
+        Collections.singletonList(new TransformConfig(NEW_INT_SV_DIMENSION_COLUMN_NAME, "plus(column1, 1)")));
+    _tableConfig.setIngestionConfig(ingestionConfig);
     _indexLoadingConfig.getInvertedIndexColumns().add(NEW_COLUMN_INVERTED_INDEX);
     checkUpdateDefaultColumns();
 
@@ -640,7 +652,8 @@ public class SegmentPreProcessorTest {
     segmentMetadata = new SegmentMetadataImpl(_indexDir);
     assertNotNull(segmentMetadata.getColumnMetadataFor(NEW_INT_SV_DIMENSION_COLUMN_NAME));
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, _indexLoadingConfig,
             _newColumnsSchema3)) {
       processor.process();
@@ -668,7 +681,8 @@ public class SegmentPreProcessorTest {
       throws Exception {
     // Update default value.
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, _indexLoadingConfig,
             _newColumnsSchema1)) {
       processor.process();
@@ -736,7 +750,8 @@ public class SegmentPreProcessorTest {
 
     // Check dictionary and forward index exist.
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentDirectory.Reader reader = segmentDirectory.createReader()) {
       assertTrue(reader.hasIndexFor(NEW_INT_METRIC_COLUMN_NAME, ColumnIndexType.DICTIONARY));
       assertTrue(reader.hasIndexFor(NEW_INT_METRIC_COLUMN_NAME, ColumnIndexType.FORWARD_INDEX));
@@ -758,7 +773,8 @@ public class SegmentPreProcessorTest {
     // For the second schema, we changed the default value for column 'newIntMetric' to 2, and added default value
     // 'abcd' (keep the same length as 'null') to column 'newStringMVDimension'.
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, _indexLoadingConfig,
             _newColumnsSchema2)) {
       processor.process();
@@ -783,21 +799,13 @@ public class SegmentPreProcessorTest {
     constructV1Segment();
 
     // Remove min/max value from the metadata
-    PropertiesConfiguration configuration = SegmentMetadataImpl.getPropertiesConfiguration(_indexDir);
-    Iterator<String> keys = configuration.getKeys();
-    while (keys.hasNext()) {
-      String key = keys.next();
-      if (key.endsWith(V1Constants.MetadataKeys.Column.MIN_VALUE) || key.endsWith(
-          V1Constants.MetadataKeys.Column.MAX_VALUE)) {
-        configuration.clearProperty(key);
-      }
-    }
-    configuration.save();
+    removeMinMaxValuesFromMetadataFile(_indexDir);
 
     IndexLoadingConfig indexLoadingConfig = new IndexLoadingConfig();
     indexLoadingConfig.setColumnMinMaxValueGeneratorMode(ColumnMinMaxValueGeneratorMode.NONE);
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, indexLoadingConfig, null)) {
       processor.process();
     }
@@ -814,7 +822,8 @@ public class SegmentPreProcessorTest {
 
     indexLoadingConfig.setColumnMinMaxValueGeneratorMode(ColumnMinMaxValueGeneratorMode.TIME);
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, indexLoadingConfig, null)) {
       processor.process();
     }
@@ -831,7 +840,8 @@ public class SegmentPreProcessorTest {
 
     indexLoadingConfig.setColumnMinMaxValueGeneratorMode(ColumnMinMaxValueGeneratorMode.NON_METRIC);
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, indexLoadingConfig, null)) {
       processor.process();
     }
@@ -848,7 +858,8 @@ public class SegmentPreProcessorTest {
 
     indexLoadingConfig.setColumnMinMaxValueGeneratorMode(ColumnMinMaxValueGeneratorMode.ALL);
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, indexLoadingConfig, null)) {
       processor.process();
     }
@@ -897,7 +908,8 @@ public class SegmentPreProcessorTest {
 
     // Create all kinds of indices.
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, _indexLoadingConfig, null)) {
       processor.process();
     }
@@ -909,7 +921,8 @@ public class SegmentPreProcessorTest {
 
     // Remove all kinds of indices.
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, new IndexLoadingConfig(), null)) {
       processor.process();
     }
@@ -934,7 +947,8 @@ public class SegmentPreProcessorTest {
     // There are a few indices initially. Remove them to prepare an initial state.
     long initFileSize = singleFileIndex.length();
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, new IndexLoadingConfig(), null)) {
       processor.process();
     }
@@ -953,14 +967,16 @@ public class SegmentPreProcessorTest {
 
     // Create all kinds of indices.
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, _indexLoadingConfig, null)) {
       processor.process();
     }
 
     long addedLength = 0;
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentDirectory.Reader reader = segmentDirectory.createReader()) {
       addedLength += reader.getIndexFor(strColumn, ColumnIndexType.INVERTED_INDEX).size() + 8;
       addedLength += reader.getIndexFor(strColumn, ColumnIndexType.RANGE_INDEX).size() + 8;
@@ -972,7 +988,8 @@ public class SegmentPreProcessorTest {
 
     // Remove all kinds of indices, and size gets back initial size.
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, new IndexLoadingConfig(), null)) {
       processor.process();
     }
@@ -986,7 +1003,8 @@ public class SegmentPreProcessorTest {
 
     // Remove all indices and add the two derived columns for H3 and Json index.
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, new IndexLoadingConfig(),
             _newColumnsSchemaWithH3Json)) {
       processor.process();
@@ -1010,7 +1028,8 @@ public class SegmentPreProcessorTest {
 
     // Create H3 and Json indices.
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, _indexLoadingConfig, null)) {
       processor.process();
     }
@@ -1019,7 +1038,8 @@ public class SegmentPreProcessorTest {
 
     // Remove H3 and Json indices.
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, new IndexLoadingConfig(), null)) {
       processor.process();
     }
@@ -1041,7 +1061,8 @@ public class SegmentPreProcessorTest {
     // There are a few indices initially. Remove them to prepare an initial state.
     // Also use the schema with columns for H3 and Json index to add those columns.
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, new IndexLoadingConfig(),
             _newColumnsSchemaWithH3Json)) {
       processor.process();
@@ -1058,14 +1079,16 @@ public class SegmentPreProcessorTest {
 
     // Create H3 and Json indices.
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, _indexLoadingConfig, null)) {
       processor.process();
     }
 
     long addedLength = 0;
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentDirectory.Reader reader = segmentDirectory.createReader()) {
       addedLength += reader.getIndexFor("newH3Col", ColumnIndexType.H3_INDEX).size() + 8;
       addedLength += reader.getIndexFor("newJsonCol", ColumnIndexType.JSON_INDEX).size() + 8;
@@ -1074,10 +1097,192 @@ public class SegmentPreProcessorTest {
 
     // Remove H3 and Json indices, and size gets back to initial.
     try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
-        .load(_indexDir.toURI(), new SegmentDirectoryLoaderContext(null, null, null, _configuration));
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
         SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, new IndexLoadingConfig(), null)) {
       processor.process();
     }
     assertEquals(singleFileIndex.length(), initFileSize);
   }
+
+  @Test
+  public void testV1IfNeedProcess()
+      throws Exception {
+    constructV1Segment();
+    SegmentMetadataImpl segmentMetadata = new SegmentMetadataImpl(_indexDir);
+    assertEquals(segmentMetadata.getVersion(), SegmentVersion.v1);
+
+    testIfNeedProcess();
+  }
+
+  @Test
+  public void testV3IfNeedProcess()
+      throws Exception {
+    constructV3Segment();
+    SegmentMetadataImpl segmentMetadata = new SegmentMetadataImpl(_indexDir);
+    assertEquals(segmentMetadata.getVersion(), SegmentVersion.v3);
+
+    testIfNeedProcess();
+  }
+
+  private void testIfNeedProcess()
+      throws Exception {
+    // There are a few indices initially. Require to remove them with an empty IndexLoadingConfig.
+    try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
+        SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, new IndexLoadingConfig(), null)) {
+      assertTrue(processor.needProcess());
+      processor.process();
+      assertFalse(processor.needProcess());
+    }
+
+    // Require to add some default columns with new schema.
+    try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
+        SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, new IndexLoadingConfig(),
+            _newColumnsSchemaWithH3Json)) {
+      assertTrue(processor.needProcess());
+      processor.process();
+      assertFalse(processor.needProcess());
+    }
+
+    // No preprocessing needed if required to add certain index on non-existing, sorted or non-dictionary column.
+    for (Map.Entry<String, Consumer<IndexLoadingConfig>> entry : createConfigPrepFunctionNeedNoops().entrySet()) {
+      String testCase = entry.getKey();
+      IndexLoadingConfig config = new IndexLoadingConfig();
+      entry.getValue().accept(config);
+      try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
+          .load(_indexDir.toURI(),
+              new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
+          SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, config,
+              _newColumnsSchemaWithH3Json)) {
+        assertFalse(processor.needProcess(), testCase);
+      }
+    }
+
+    // Require to add different types of indices. Add one new index a time
+    // to test the index handlers separately.
+    IndexLoadingConfig config = new IndexLoadingConfig();
+    for (Map.Entry<String, Consumer<IndexLoadingConfig>> entry : createConfigPrepFunctions().entrySet()) {
+      String testCase = entry.getKey();
+      entry.getValue().accept(config);
+      try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
+          .load(_indexDir.toURI(),
+              new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
+          SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, config,
+              _newColumnsSchemaWithH3Json)) {
+        assertTrue(processor.needProcess(), testCase);
+        processor.process();
+        assertFalse(processor.needProcess(), testCase);
+      }
+    }
+
+    // Require to add startree index.
+    IndexingConfig indexingConfig = new IndexingConfig();
+    indexingConfig.setEnableDynamicStarTreeCreation(true);
+    indexingConfig.setEnableDefaultStarTree(true);
+    _tableConfig.setIndexingConfig(indexingConfig);
+    IndexLoadingConfig configWithStarTreeIndex = new IndexLoadingConfig(null, _tableConfig);
+    createConfigPrepFunctions().forEach((k, v) -> v.accept(configWithStarTreeIndex));
+    try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
+        SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, configWithStarTreeIndex,
+            _newColumnsSchemaWithH3Json)) {
+      assertTrue(processor.needProcess());
+      processor.process();
+      assertFalse(processor.needProcess());
+    }
+
+    // Require to update min and max values.
+    removeMinMaxValuesFromMetadataFile(_indexDir);
+    SegmentMetadataImpl segmentMetadata = new SegmentMetadataImpl(_indexDir);
+    segmentMetadata.getColumnMetadataMap().forEach((k, v) -> {
+      assertNull(v.getMinValue(), "checking column: " + k);
+      assertNull(v.getMaxValue(), "checking column: " + k);
+    });
+    try (SegmentDirectory segmentDirectory = SegmentDirectoryLoaderRegistry.getDefaultSegmentDirectoryLoader()
+        .load(_indexDir.toURI(),
+            new SegmentDirectoryLoaderContext.Builder().setSegmentDirectoryConfigs(_configuration).build());
+        SegmentPreProcessor processor = new SegmentPreProcessor(segmentDirectory, configWithStarTreeIndex,
+            _newColumnsSchemaWithH3Json)) {
+      assertTrue(processor.needProcess());
+      processor.process();
+    }
+    segmentMetadata = new SegmentMetadataImpl(_indexDir);
+    segmentMetadata.getColumnMetadataMap().forEach((k, v) -> {
+      if (v.hasDictionary()) {
+        assertNotNull(v.getMinValue(), "checking column: " + k);
+        assertNotNull(v.getMaxValue(), "checking column: " + k);
+      } else {
+        assertNull(v.getMinValue(), "checking column: " + k);
+        assertNull(v.getMaxValue(), "checking column: " + k);
+      }
+    });
+  }
+
+  private static void removeMinMaxValuesFromMetadataFile(File indexDir)
+      throws Exception {
+    PropertiesConfiguration configuration = SegmentMetadataImpl.getPropertiesConfiguration(indexDir);
+    Iterator<String> keys = configuration.getKeys();
+    while (keys.hasNext()) {
+      String key = keys.next();
+      if (key.endsWith(V1Constants.MetadataKeys.Column.MIN_VALUE) || key.endsWith(
+          V1Constants.MetadataKeys.Column.MAX_VALUE)) {
+        configuration.clearProperty(key);
+      }
+    }
+    configuration.save();
+  }
+
+  private static Map<String, Consumer<IndexLoadingConfig>> createConfigPrepFunctions() {
+    Map<String, Consumer<IndexLoadingConfig>> testCases = new HashMap<>();
+    testCases.put("addInvertedIndex", (IndexLoadingConfig config) -> config.setInvertedIndexColumns(
+        new HashSet<>(Collections.singletonList("column3"))));
+    testCases.put("addRangeIndex", (IndexLoadingConfig config) -> config.setRangeIndexColumns(
+        new HashSet<>(Collections.singletonList("column3"))));
+    testCases.put("addTextIndex",
+        (IndexLoadingConfig config) -> config.setTextIndexColumns(new HashSet<>(Collections.singletonList("column3"))));
+    testCases.put("addFSTIndex",
+        (IndexLoadingConfig config) -> config.setFSTIndexColumns(new HashSet<>(Collections.singletonList("column3"))));
+    testCases.put("addBloomFilter", (IndexLoadingConfig config) -> config.setBloomFilterConfigs(
+        ImmutableMap.of("column3", new BloomFilterConfig(0.1, 1024, true))));
+    testCases.put("addH3Index", (IndexLoadingConfig config) -> config.setH3IndexConfigs(
+        ImmutableMap.of("newH3Col", new H3IndexConfig(ImmutableMap.of("resolutions", "5")))));
+    testCases.put("addJsonIndex", (IndexLoadingConfig config) -> config.setJsonIndexColumns(
+        new HashSet<>(Collections.singletonList("newJsonCol"))));
+    return testCases;
+  }
+
+  private static Map<String, Consumer<IndexLoadingConfig>> createConfigPrepFunctionNeedNoops() {
+    Map<String, Consumer<IndexLoadingConfig>> testCases = new HashMap<>();
+    // daysSinceEpoch is a sorted column, thus inverted index and range index skip it.
+    testCases.put("addInvertedIndexOnSortedColumn", (IndexLoadingConfig config) -> config.setInvertedIndexColumns(
+        new HashSet<>(Collections.singletonList("daysSinceEpoch"))));
+    testCases.put("addRangeIndexOnSortedColumn", (IndexLoadingConfig config) -> config.setRangeIndexColumns(
+        new HashSet<>(Collections.singletonList("daysSinceEpoch"))));
+    // column4 is unsorted non-dictionary encoded column, so inverted index and bloom filter skip it.
+    // In fact, the validation logic when updating index configs already blocks this to happen.
+    testCases.put("addInvertedIndexOnNonDictColumn", (IndexLoadingConfig config) -> config.setInvertedIndexColumns(
+        new HashSet<>(Collections.singletonList("column4"))));
+    // No index is added on non-existing columns.
+    // The validation logic when updating index configs already blocks this to happen.
+    testCases.put("addInvertedIndexOnAbsentColumn", (IndexLoadingConfig config) -> config.setInvertedIndexColumns(
+        new HashSet<>(Collections.singletonList("newColumnX"))));
+    testCases.put("addRangeIndexOnAbsentColumn", (IndexLoadingConfig config) -> config.setRangeIndexColumns(
+        new HashSet<>(Collections.singletonList("newColumnX"))));
+    testCases.put("addTextIndexOnAbsentColumn", (IndexLoadingConfig config) -> config.setTextIndexColumns(
+        new HashSet<>(Collections.singletonList("newColumnX"))));
+    testCases.put("addFSTIndexOnAbsentColumn", (IndexLoadingConfig config) -> config.setFSTIndexColumns(
+        new HashSet<>(Collections.singletonList("newColumnX"))));
+    testCases.put("addBloomFilterOnAbsentColumn", (IndexLoadingConfig config) -> config.setBloomFilterConfigs(
+        ImmutableMap.of("newColumnX", new BloomFilterConfig(0.1, 1024, true))));
+    testCases.put("addH3IndexOnAbsentColumn", (IndexLoadingConfig config) -> config.setH3IndexConfigs(
+        ImmutableMap.of("newColumnX", new H3IndexConfig(ImmutableMap.of("resolutions", "5")))));
+    testCases.put("addJsonIndexOnAbsentColumn", (IndexLoadingConfig config) -> config.setJsonIndexColumns(
+        new HashSet<>(Collections.singletonList("newColumnX"))));
+    return testCases;
+  }
 }
