@@ -33,16 +33,20 @@ import java.util.Set;
 import java.util.concurrent.TimeUnit;
 import java.util.stream.Collectors;
 import javax.annotation.Nullable;
+import org.apache.commons.collections.CollectionUtils;
 import org.apache.pinot.segment.spi.compression.ChunkCompressionType;
 import org.apache.pinot.segment.spi.creator.name.FixedSegmentNameGenerator;
 import org.apache.pinot.segment.spi.creator.name.SegmentNameGenerator;
 import org.apache.pinot.segment.spi.creator.name.SimpleSegmentNameGenerator;
 import org.apache.pinot.segment.spi.index.creator.H3IndexConfig;
+import org.apache.pinot.spi.config.table.FSTType;
 import org.apache.pinot.spi.config.table.FieldConfig;
 import org.apache.pinot.spi.config.table.IndexingConfig;
 import org.apache.pinot.spi.config.table.SegmentPartitionConfig;
+import org.apache.pinot.spi.config.table.SegmentZKPropsConfig;
 import org.apache.pinot.spi.config.table.StarTreeIndexConfig;
 import org.apache.pinot.spi.config.table.TableConfig;
+import org.apache.pinot.spi.config.table.TimestampIndexGranularity;
 import org.apache.pinot.spi.data.DateTimeFieldSpec;
 import org.apache.pinot.spi.data.DateTimeFormatSpec;
 import org.apache.pinot.spi.data.FieldSpec;
@@ -64,16 +68,20 @@ public class SegmentGeneratorConfig implements Serializable {
   }
 
   private static final Logger LOGGER = LoggerFactory.getLogger(SegmentGeneratorConfig.class);
+  public static final double DEFAULT_NO_DICTIONARY_SIZE_RATIO_THRESHOLD = 0.85d;
 
   private TableConfig _tableConfig;
   private final Map<String, String> _customProperties = new HashMap<>();
   private final Set<String> _rawIndexCreationColumns = new HashSet<>();
   private final Map<String, ChunkCompressionType> _rawIndexCompressionType = new HashMap<>();
   private final List<String> _invertedIndexCreationColumns = new ArrayList<>();
+  private final List<String> _bloomFilterCreationColumns = new ArrayList<>();
+  private final List<String> _rangeIndexCreationColumns = new ArrayList<>();
   private final List<String> _textIndexCreationColumns = new ArrayList<>();
   private final List<String> _fstIndexCreationColumns = new ArrayList<>();
   private final List<String> _jsonIndexCreationColumns = new ArrayList<>();
   private final Map<String, H3IndexConfig> _h3IndexConfigs = new HashMap<>();
+  private final Map<String, List<TimestampIndexGranularity>> _timestampIndexConfigs = new HashMap<>();
   private final List<String> _columnSortOrder = new ArrayList<>();
   private List<String> _varLengthDictionaryColumns = new ArrayList<>();
   private String _inputFilePath = null;
@@ -91,6 +99,7 @@ public class SegmentGeneratorConfig implements Serializable {
   private String _segmentEndTime = null;
   private SegmentVersion _segmentVersion = SegmentVersion.v3;
   private Schema _schema = null;
+  private FSTType _fstTypeForFSTIndex = FSTType.LUCENE;
   private RecordReaderConfig _readerConfig = null;
   private List<StarTreeIndexConfig> _starTreeIndexConfigs = null;
   private boolean _enableDefaultStarTree = false;
@@ -105,10 +114,14 @@ public class SegmentGeneratorConfig implements Serializable {
   private boolean _skipTimeValueCheck = false;
   private boolean _nullHandlingEnabled = false;
   private boolean _failOnEmptySegment = false;
+  private boolean _optimizeDictionaryForMetrics = false;
+  private double _noDictionarySizeRatioThreshold = DEFAULT_NO_DICTIONARY_SIZE_RATIO_THRESHOLD;
 
   // constructed from FieldConfig
   private Map<String, Map<String, String>> _columnProperties = new HashMap<>();
 
+  private SegmentZKPropsConfig _segmentZKPropsConfig;
+
   @Deprecated
   public SegmentGeneratorConfig() {
   }
@@ -125,7 +138,8 @@ public class SegmentGeneratorConfig implements Serializable {
   public SegmentGeneratorConfig(TableConfig tableConfig, Schema schema) {
     Preconditions.checkNotNull(schema);
     Preconditions.checkNotNull(tableConfig);
-    setSchema(schema);
+    _timestampIndexConfigs.putAll(extractTimestampIndexConfigsFromTableConfig(tableConfig));
+    setSchema(updateSchemaWithTimestampIndexes(schema, _timestampIndexConfigs));
 
     _tableConfig = tableConfig;
     setTableName(tableConfig.getTableName());
@@ -178,6 +192,18 @@ public class SegmentGeneratorConfig implements Serializable {
         }
       }
 
+      if (indexingConfig.getBloomFilterColumns() != null) {
+        _bloomFilterCreationColumns.addAll(indexingConfig.getBloomFilterColumns());
+      }
+
+      if (indexingConfig.getBloomFilterConfigs() != null) {
+        _bloomFilterCreationColumns.addAll(indexingConfig.getBloomFilterConfigs().keySet());
+      }
+
+      if (indexingConfig.getRangeIndexColumns() != null) {
+        _rangeIndexCreationColumns.addAll(indexingConfig.getRangeIndexColumns());
+      }
+
       if (indexingConfig.getJsonIndexColumns() != null) {
         _jsonIndexCreationColumns.addAll(indexingConfig.getJsonIndexColumns());
       }
@@ -194,10 +220,38 @@ public class SegmentGeneratorConfig implements Serializable {
       extractH3IndexConfigsFromTableConfig(tableConfig);
       extractCompressionCodecConfigsFromTableConfig(tableConfig);
 
+      _fstTypeForFSTIndex = tableConfig.getIndexingConfig().getFSTIndexType();
+
       _nullHandlingEnabled = indexingConfig.isNullHandlingEnabled();
+      _optimizeDictionaryForMetrics = indexingConfig.isOptimizeDictionaryForMetrics();
+      _noDictionarySizeRatioThreshold = indexingConfig.getNoDictionarySizeRatioThreshold();
     }
   }
 
+  public static Schema updateSchemaWithTimestampIndexes(Schema schema,
+      Map<String, List<TimestampIndexGranularity>> timestampIndexConfigs) {
+    if (timestampIndexConfigs.isEmpty()) {
+      return schema;
+    }
+    List<FieldSpec> timestampColumnWithGranularityFieldSpecs = new ArrayList<>();
+    for (Map.Entry<String, List<TimestampIndexGranularity>> entry : timestampIndexConfigs.entrySet()) {
+      String columnName = entry.getKey();
+      Preconditions.checkState(schema.hasColumn(columnName),
+          "Cannot create Timestamp index for column: %s because it is not in schema", columnName);
+      entry.getValue().stream().filter(granularity -> !schema.hasColumn(
+          TimestampIndexGranularity.getColumnNameWithGranularity(columnName, granularity))).forEach(
+          granularity -> timestampColumnWithGranularityFieldSpecs.add(
+              TimestampIndexGranularity.getFieldSpecForTimestampColumnWithGranularity(
+                  schema.getFieldSpecFor(columnName), granularity)));
+    }
+    if (timestampColumnWithGranularityFieldSpecs.isEmpty()) {
+      return schema;
+    }
+    Schema newSchema = schema.clone();
+    timestampColumnWithGranularityFieldSpecs.forEach(fieldSpec -> newSchema.addField(fieldSpec));
+    return newSchema;
+  }
+
   public Map<String, Map<String, String>> getColumnProperties() {
     return _columnProperties;
   }
@@ -210,7 +264,7 @@ public class SegmentGeneratorConfig implements Serializable {
       DateTimeFieldSpec dateTimeFieldSpec = schema.getSpecForTimeColumn(timeColumnName);
       if (dateTimeFieldSpec != null) {
         setTimeColumnName(dateTimeFieldSpec.getName());
-        setDateTimeFormatSpec(new DateTimeFormatSpec(dateTimeFieldSpec.getFormat()));
+        setDateTimeFormatSpec(dateTimeFieldSpec.getFormatSpec());
       }
     }
   }
@@ -257,6 +311,22 @@ public class SegmentGeneratorConfig implements Serializable {
     }
   }
 
+  public static Map<String, List<TimestampIndexGranularity>> extractTimestampIndexConfigsFromTableConfig(
+      TableConfig tableConfig) {
+    if (tableConfig == null) {
+      return Collections.emptyMap();
+    }
+    List<FieldConfig> fieldConfigList = tableConfig.getFieldConfigList();
+    Map<String, List<TimestampIndexGranularity>> timestampIndexConfigs = new HashMap<>();
+    if (CollectionUtils.isNotEmpty(fieldConfigList)) {
+      fieldConfigList.stream()
+          .filter(fieldConfig -> fieldConfig.getIndexTypes().contains(FieldConfig.IndexType.TIMESTAMP))
+          .forEach(fieldConfig -> timestampIndexConfigs.put(fieldConfig.getName(),
+              fieldConfig.getTimestampConfig().getGranularities()));
+    }
+    return timestampIndexConfigs;
+  }
+
   private void extractCompressionCodecConfigsFromTableConfig(TableConfig tableConfig) {
     List<FieldConfig> fieldConfigList = tableConfig.getFieldConfigList();
     if (fieldConfigList != null) {
@@ -312,6 +382,14 @@ public class SegmentGeneratorConfig implements Serializable {
     return _invertedIndexCreationColumns;
   }
 
+  public List<String> getBloomFilterCreationColumns() {
+    return _bloomFilterCreationColumns;
+  }
+
+  public List<String> getRangeIndexCreationColumns() {
+    return _rangeIndexCreationColumns;
+  }
+
   /**
    * Used by org.apache.pinot.core.segment.creator.impl.SegmentColumnarIndexCreator
    * to get the list of text index columns.
@@ -333,6 +411,10 @@ public class SegmentGeneratorConfig implements Serializable {
     return _h3IndexConfigs;
   }
 
+  public Map<String, List<TimestampIndexGranularity>> getTimestampIndexConfigs() {
+    return _timestampIndexConfigs;
+  }
+
   public List<String> getColumnSortOrder() {
     return _columnSortOrder;
   }
@@ -503,6 +585,14 @@ public class SegmentGeneratorConfig implements Serializable {
     return _sequenceId;
   }
 
+  public void setFSTIndexType(FSTType fstType) {
+    _fstTypeForFSTIndex = fstType;
+  }
+
+  public FSTType getFSTIndexType() {
+    return _fstTypeForFSTIndex;
+  }
+
   /**
    * This method should be used instead of setPostfix if you are adding a sequence number.
    */
@@ -694,6 +784,22 @@ public class SegmentGeneratorConfig implements Serializable {
     _nullHandlingEnabled = nullHandlingEnabled;
   }
 
+  public boolean isOptimizeDictionaryForMetrics() {
+    return _optimizeDictionaryForMetrics;
+  }
+
+  public void setOptimizeDictionaryForMetrics(boolean optimizeDictionaryForMetrics) {
+    _optimizeDictionaryForMetrics = optimizeDictionaryForMetrics;
+  }
+
+  public double getNoDictionarySizeRatioThreshold() {
+    return _noDictionarySizeRatioThreshold;
+  }
+
+  public void setNoDictionarySizeRatioThreshold(double noDictionarySizeRatioThreshold) {
+    _noDictionarySizeRatioThreshold = noDictionarySizeRatioThreshold;
+  }
+
   public boolean isFailOnEmptySegment() {
     return _failOnEmptySegment;
   }
@@ -701,4 +807,12 @@ public class SegmentGeneratorConfig implements Serializable {
   public void setFailOnEmptySegment(boolean failOnEmptySegment) {
     _failOnEmptySegment = failOnEmptySegment;
   }
+
+  public SegmentZKPropsConfig getSegmentZKPropsConfig() {
+    return _segmentZKPropsConfig;
+  }
+
+  public void setSegmentZKPropsConfig(SegmentZKPropsConfig segmentZKPropsConfig) {
+    _segmentZKPropsConfig = segmentZKPropsConfig;
+  }
 }
