@@ -30,23 +30,31 @@ import com.fasterxml.jackson.databind.node.ArrayNode;
 import com.fasterxml.jackson.databind.node.JsonNodeFactory;
 import com.fasterxml.jackson.databind.node.ObjectNode;
 import com.google.common.base.Preconditions;
+import com.google.common.collect.MapDifference;
+import com.google.common.collect.Maps;
 import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
+import java.nio.charset.StandardCharsets;
+import java.util.AbstractMap;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.Iterator;
+import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.TreeMap;
 import java.util.concurrent.TimeUnit;
+import java.util.stream.IntStream;
+import java.util.stream.Stream;
 import javax.annotation.Nullable;
+import org.apache.commons.io.IOUtils;
+import org.apache.commons.lang3.tuple.Pair;
 import org.apache.pinot.spi.config.table.ingestion.ComplexTypeConfig;
 import org.apache.pinot.spi.data.DateTimeFieldSpec;
-import org.apache.pinot.spi.data.DateTimeFormatSpec;
-import org.apache.pinot.spi.data.DateTimeGranularitySpec;
 import org.apache.pinot.spi.data.DimensionFieldSpec;
 import org.apache.pinot.spi.data.FieldSpec;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
@@ -71,12 +79,60 @@ public class JsonUtils {
   public static final ObjectReader DEFAULT_READER = DEFAULT_MAPPER.reader();
   public static final ObjectWriter DEFAULT_WRITER = DEFAULT_MAPPER.writer();
   public static final ObjectWriter DEFAULT_PRETTY_WRITER = DEFAULT_MAPPER.writerWithDefaultPrettyPrinter();
+  public static final TypeReference<HashMap<String, Object>> MAP_TYPE_REFERENCE =
+      new TypeReference<HashMap<String, Object>>() {
+      };
 
   public static <T> T stringToObject(String jsonString, Class<T> valueType)
-      throws IOException {
+      throws JsonProcessingException {
     return DEFAULT_READER.forType(valueType).readValue(jsonString);
   }
 
+  public static <T> Pair<T, Map<String, Object>> inputStreamToObjectAndUnrecognizedProperties(
+      InputStream jsonInputStream, Class<T> valueType)
+      throws IOException {
+    String jsonString = IOUtils.toString(jsonInputStream, StandardCharsets.UTF_8);
+    return stringToObjectAndUnrecognizedProperties(jsonString, valueType);
+  }
+
+  public static <T> Pair<T, Map<String, Object>> stringToObjectAndUnrecognizedProperties(String jsonString,
+      Class<T> valueType)
+      throws IOException {
+    T instance = DEFAULT_READER.forType(valueType).readValue(jsonString);
+    Map<String, Object> inputJsonMap = flatten(DEFAULT_MAPPER.readValue(jsonString, MAP_TYPE_REFERENCE));
+
+    String instanceJson = DEFAULT_MAPPER.writeValueAsString(instance);
+    Map<String, Object> instanceJsonMap = flatten(DEFAULT_MAPPER.readValue(instanceJson, MAP_TYPE_REFERENCE));
+
+    MapDifference<String, Object> difference = Maps.difference(inputJsonMap, instanceJsonMap);
+    return Pair.of(instance, difference.entriesOnlyOnLeft());
+  }
+
+  private static Map<String, Object> flatten(Map<String, Object> map) {
+    return map.entrySet().stream().flatMap(JsonUtils::flatten)
+        .collect(LinkedHashMap::new, (m, e) -> m.put("/" + e.getKey(), e.getValue()), LinkedHashMap::putAll);
+  }
+
+  private static Stream<Map.Entry<String, Object>> flatten(Map.Entry<String, Object> entry) {
+    if (entry == null) {
+      return Stream.empty();
+    }
+
+    if (entry.getValue() instanceof Map<?, ?>) {
+      return ((Map<?, ?>) entry.getValue()).entrySet().stream()
+          .flatMap(e -> flatten(new AbstractMap.SimpleEntry<>(entry.getKey() + "/" + e.getKey(), e.getValue())));
+    }
+
+    if (entry.getValue() instanceof List<?>) {
+      List<?> list = (List<?>) entry.getValue();
+      return IntStream.range(0, list.size())
+          .mapToObj(i -> new AbstractMap.SimpleEntry<String, Object>(entry.getKey() + "/" + i, list.get(i)))
+          .flatMap(JsonUtils::flatten);
+    }
+
+    return Stream.of(entry);
+  }
+
   public static <T> T stringToObject(String jsonString, TypeReference<T> valueTypeRef)
       throws IOException {
     return DEFAULT_READER.forType(valueTypeRef).readValue(jsonString);
@@ -108,11 +164,12 @@ public class JsonUtils {
   /**
    * Reads the first json object from the file that can contain multiple objects
    */
+  @Nullable
   public static JsonNode fileToFirstJsonNode(File jsonFile)
       throws IOException {
-    try (InputStream inputStream = new FileInputStream(jsonFile)) {
-      JsonFactory jf = new JsonFactory();
-      JsonParser jp = jf.createParser(inputStream);
+    JsonFactory jf = new JsonFactory();
+    try (InputStream inputStream = new FileInputStream(jsonFile);
+        JsonParser jp = jf.createParser(inputStream)) {
       jp.setCodec(DEFAULT_MAPPER);
       jp.nextToken();
       if (jp.hasCurrentToken()) {
@@ -152,6 +209,11 @@ public class JsonUtils {
     return DEFAULT_READER.forType(valueTypeRef).readValue(jsonNode);
   }
 
+  public static Map<String, Object> jsonNodeToMap(JsonNode jsonNode)
+      throws IOException {
+    return DEFAULT_READER.forType(MAP_TYPE_REFERENCE).readValue(jsonNode);
+  }
+
   public static String objectToString(Object object)
       throws JsonProcessingException {
     return DEFAULT_WRITER.writeValueAsString(object);
@@ -417,6 +479,7 @@ public class JsonUtils {
     if (fieldsToUnnest == null) {
       fieldsToUnnest = new ArrayList<>();
     }
+    Preconditions.checkNotNull(jsonNode, "the JSON data shall be an object but it is null");
     Preconditions.checkState(jsonNode.isObject(), "the JSON data shall be an object");
     return getPinotSchemaFromJsonNode(jsonNode, fieldTypeMap, timeUnit, fieldsToUnnest, delimiter,
         collectionNotUnnestedToJson);
@@ -507,6 +570,8 @@ public class JsonUtils {
       return DataType.BOOLEAN;
     } else if (jsonNode.isBinary()) {
       return DataType.BYTES;
+    } else if (jsonNode.isBigDecimal()) {
+      return DataType.BIG_DECIMAL;
     } else {
       return DataType.STRING;
     }
@@ -531,9 +596,11 @@ public class JsonUtils {
         case DATE_TIME:
           Preconditions.checkState(isSingleValueField, "Time field: %s cannot be multi-valued", name);
           Preconditions.checkNotNull(timeUnit, "Time unit cannot be null");
-          pinotSchema.addField(new DateTimeFieldSpec(name, dataType,
-              new DateTimeFormatSpec(1, timeUnit.toString(), DateTimeFieldSpec.TimeFormat.EPOCH.toString()).getFormat(),
-              new DateTimeGranularitySpec(1, timeUnit).getGranularity()));
+          // TODO: Switch to new format after releasing 0.11.0
+          //       "EPOCH|" + timeUnit.name()
+          String format = "1:" + timeUnit.name() + ":EPOCH";
+          String granularity = "1:" + timeUnit.name();
+          pinotSchema.addField(new DateTimeFieldSpec(name, dataType, format, granularity));
           break;
         default:
           throw new UnsupportedOperationException("Unsupported field type: " + fieldType + " for field: " + name);
