@@ -35,13 +35,12 @@ import java.util.List;
 import java.util.Map;
 import java.util.PriorityQueue;
 import javax.annotation.Nullable;
-import org.apache.helix.PropertyPathConfig;
-import org.apache.helix.PropertyType;
-import org.apache.helix.ZNRecord;
+import org.apache.helix.PropertyPathBuilder;
 import org.apache.helix.manager.zk.ZKHelixAdmin;
-import org.apache.helix.manager.zk.ZNRecordSerializer;
 import org.apache.helix.model.IdealState;
 import org.apache.helix.store.zk.ZkHelixPropertyStore;
+import org.apache.helix.zookeeper.datamodel.ZNRecord;
+import org.apache.helix.zookeeper.datamodel.serializer.ZNRecordSerializer;
 import org.apache.pinot.common.utils.config.TableConfigUtils;
 import org.apache.pinot.common.utils.helix.HelixHelper;
 import org.apache.pinot.spi.config.table.TableConfig;
@@ -88,7 +87,7 @@ public class MoveReplicaGroup extends AbstractBaseAdminCommand implements Comman
       description = "Zookeeper cluster path(Ex: /pinot")
   private String _zkPath;
 
-  @CommandLine.Option(names = {"-exec"}, required = false, 
+  @CommandLine.Option(names = {"-exec"}, required = false,
       description = "Execute replica group move. dryRun(default) if not specified")
   private boolean _exec = false;
 
@@ -121,7 +120,6 @@ public class MoveReplicaGroup extends AbstractBaseAdminCommand implements Comman
 
   @Override
   public void cleanup() {
-
   }
 
   public boolean execute()
@@ -426,7 +424,7 @@ public class MoveReplicaGroup extends AbstractBaseAdminCommand implements Comman
   private TableConfig getTableConfig(String tableName)
       throws IOException {
     ZNRecordSerializer serializer = new ZNRecordSerializer();
-    String path = PropertyPathConfig.getPath(PropertyType.PROPERTYSTORE, _zkPath);
+    String path = PropertyPathBuilder.propertyStore(_zkPath);
     ZkHelixPropertyStore<ZNRecord> propertyStore = new ZkHelixPropertyStore<>(_zkHost, serializer, path);
     ZNRecord tcZnRecord = propertyStore.get("/CONFIGS/TABLE/" + tableName, null, 0);
     TableConfig tableConfig = TableConfigUtils.fromZNRecord(tcZnRecord);
